package digital.laboratory.platform.sewage.controller;

import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.constant.CommonConstants;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.sewage.dto.StatisticsDiffStatusJobDTO;
import digital.laboratory.platform.sewage.entity.SewageJob;
import digital.laboratory.platform.sewage.entity.SewageJobIdentificationMaterial;
import digital.laboratory.platform.sewage.entity.SewageTreatmentPlant;
import digital.laboratory.platform.sewage.entity.*;
import digital.laboratory.platform.sewage.service.SewageJobIdentificationMaterialService;
import digital.laboratory.platform.sewage.service.SewageJobItemService;
import digital.laboratory.platform.sewage.service.SewageJobService;
import digital.laboratory.platform.sewage.service.SewageTreatmentPlantService;
import digital.laboratory.platform.sewage.vo.SewageJobIdentificationMaterialVO;
import digital.laboratory.platform.sewage.vo.SewageJobVO;
import digital.laboratory.platform.sys.entity.SysOrg;
import digital.laboratory.platform.sys.vo.entrustment.MarkersVO;
import digital.laboratory.platform.sewage.vo.SewageJobItemVO;
import digital.laboratory.platform.sys.feign.RemoteOrgService;
import digital.laboratory.platform.sys.feign.RemoteUserService;
import org.apache.commons.lang.StringUtils;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.security.Principal;
import java.time.LocalDateTime;
import java.util.List;


/**
 * 污水检测任务
 *
 * @author Zhang Xiaolong created at  2022-07-26
 * @describe 污水检测任务 前端控制器
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/sewage_job")
@Api(value = "sewage_job", tags = "111-污水检测任务管理")
public class SewageJobController {

    private final SewageJobService sewageJobService;
    private final SewageJobItemService sewageJobItemService;
    private final SewageJobIdentificationMaterialService sewageJobIdentificationMaterialService;
    private final SewageTreatmentPlantService sewageTreatmentPlantService;

    private final RemoteUserService remoteUserService;
    private final RemoteOrgService remoteOrgService;

    /**
     * 通过id查询污水检测任务
     *
     * @param id id
     * @return R
     */
    @ApiOperation(value = "通过id查询", notes = "通过id查询")
    @GetMapping("/{id}")
    @PreAuthorize("@pms.hasPermission('SewageJobGet')")
    public R getById(@PathVariable("id") String id, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        List<SewageJobVO> sjList = sewageJobService.getSewageJobVOList(Wrappers.<SewageJob>query()
                .eq("id", id)
        );
        if ((sjList == null) || (sjList.size() == 0)) {
            throw new RuntimeException(String.format("没有找到id为 %s 的污水检测任务", id));
        }

        SewageJobVO sewageJob = sjList.get(0);
        if (sewageJob == null) {
            throw new RuntimeException(String.format("没有找到id为 %s 的污水检测任务", id));
        }

        return R.ok(sewageJob);
    }

    /**
     * 通过id查询污水检测任务
     *
     * @param id id
     * @return R
     */
    @ApiOperation(value = "通过id查询污水任务", notes = "通过id查询污水任务")
    @GetMapping("/getSewageJobById")
    public R<SewageJob> getSewageJobById(@RequestParam("id") String id) {
        return R.ok(sewageJobService.getById(id));
    }

    /**
     * 分页查询
     *
     * @param page      分页对象
     * @param sewageJob 污水检测任务
     * @return
     */
    @ApiOperation(value = "分页查询任务", notes = "分页查询任务\n" +
            "参数:\n" +
            "page - 分页信息: current, size。例如: current=1&size=10\n" +
            "keyword - 查询关键字, 仅对分中心员工有效。 keyword 查询逻辑如下：\n" +
            "	如果 keyword 与某个检材编号、样本1编号、样本2编号相符，则返回该检材对象\n" +
            "statuses - 状态列表, 例如: statuses=1,2,3\n" +
            "status - 单个状态, 例如: status=1\n" +
            "name - 任务名称")
    @GetMapping("/page")
    @PreAuthorize("@pms.hasPermission('SewageJobList')")
    public R getSewageJobPage(Page page, String keyword, SewageJob sewageJob, @RequestParam(value = "statuses", required = false) List<Integer> statuses, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        final SysOrg userOrg;    // 定义成 final才可以被后面的 Lambda 表达式使用
        R r = remoteOrgService.getById(dlpUser.getOrgId());
        if (r.getCode() == CommonConstants.SUCCESS) {
            userOrg = (SysOrg) r.getData();
        } else {
            return R.failed(String.format("没有找到 orgId 为 %s 的机构, 请确认用户所属机构的正确性!", dlpUser.getOrgId()));
        }
        if (StrUtil.isNotBlank(keyword)) {
            keyword = keyword.trim();
        }
        final String theKeyword = keyword;


        //IPage<SewageJobVO> thePage = null;
        if (dlpUser.isStaff()) {
            // 是鉴定中心员工

            if (StrUtil.isNotBlank(theKeyword)) {
                // 有提供 keyword, 模糊查询

                // 先查是不是检材编号
                List<SewageJobIdentificationMaterialVO> theList = sewageJobIdentificationMaterialService.getSewageJobIdentificationMaterialVOList(Wrappers.<SewageJobIdentificationMaterial>query()
                        .eq("sjim.im_no", theKeyword)
                        .or()
                        .eq("sjim.sample1_no", theKeyword)
                        .or()
                        .eq("sjim.sample2_no", theKeyword)
                );
                if (theList.size() == 1) {
                    // 查到数据了, 直接返回; =0或>1都不对
                    return R.ok(theList.get(0));
                }
            }

            IPage<SewageJobVO> thePage = sewageJobService.getSewageJobVOPage(page, Wrappers.<SewageJob>query()
                    .and(StrUtil.isNotBlank(theKeyword), qw -> qw
                            .like("name", theKeyword)
                            .or()
                            .like("job_no", theKeyword))
                    .in(statuses != null && statuses.size() > 0, "status", statuses)
                    .eq(sewageJob.getStatus() != null, "status", sewageJob.getStatus())
                    .like(StrUtil.isNotBlank(sewageJob.getName()), "name", sewageJob.getName())
                    //.orderByDesc("expiration_date")
                    //.orderByDesc("job_year")
                    //.orderByDesc("job_season")
                    //.orderByAsc("name")
                    .orderByDesc("create_time")
            );
            return R.ok(thePage);
        } else {
            String jobIdSql = "SELECT ji.job_id FROM b_sewage_job_item ji LEFT JOIN b_sewage_treatment_plant stp ON ji.plant_id=stp.id ";
            if (StrUtil.endWith(userOrg.getAreaCode(), "0000")) {
                // 省级
                jobIdSql += "WHERE stp.province_code='" + userOrg.getAreaCode() + "'";
            } else if (StrUtil.endWith(userOrg.getAreaCode(), "00")) {
                // 市级
                jobIdSql += "WHERE stp.city_code='" + userOrg.getAreaCode() + "'";
            } else {
                // 区级
                jobIdSql += "WHERE stp.district_code='" + userOrg.getAreaCode() + "'";
            }

            IPage<SewageJobVO> thePage = sewageJobService.getSewageJobVOPage(page, Wrappers.<SewageJob>query()
                    .in(statuses != null && statuses.size() > 0, "status", statuses)
                    .eq(sewageJob.getStatus() != null, "status", sewageJob.getStatus())
                    .like(StrUtil.isNotBlank(sewageJob.getName()), "name", sewageJob.getName())
                    .like(StrUtil.isNotBlank(theKeyword), "name", theKeyword)
                    .inSql("id", jobIdSql)       // 只查包含本用户所在区域的污水厂的任务
                    //.orderByDesc("expiration_date")
                    //.orderByDesc("job_year")
                    //.orderByDesc("job_season")
                    //.orderByAsc("name")
                    .orderByDesc("create_time")
            );
            return R.ok(thePage);
        }


    }

    /**
     * 列表查询
     * 鉴定中心员工全部可看
     * 其他用户只可看到包含有其下污水处理厂的任务
     *
     * @param sewageJob 污水检测任务
     * @return
     */
    @ApiOperation(value = "列表查询", notes = "列表查询")
    @GetMapping("/list")
    @PreAuthorize("@pms.hasPermission('SewageJobList')")
    public R getSewageJobList(SewageJob sewageJob, @RequestParam(value = "statuses", required = false) List<Integer> statuses, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        final SysOrg userOrg;    // 定义成 final才可以被后面的 Lambda 表达式使用
        R r = remoteOrgService.getById(dlpUser.getOrgId());
        if (r.getCode() == CommonConstants.SUCCESS) {
            userOrg = (SysOrg) r.getData();
        } else {
            return R.failed(String.format("没有找到 orgId 为 %s 的机构, 请确认用户所属机构的正确性!", dlpUser.getOrgId()));
        }

        List<SewageJobVO> list = null;
        if (dlpUser.isStaff()) {
            // 是鉴定中心员工
            list = sewageJobService.getSewageJobVOList(Wrappers.<SewageJob>query()
//					.eq("launche_org_id", dlpUser.getOrgId())       // 只查询当前用户的机构下的任务
                            .in(statuses != null && statuses.size() > 0, "status", statuses)
                            .eq(sewageJob.getStatus() != null, "status", sewageJob.getStatus())
                            .like(StrUtil.isNotBlank(sewageJob.getName()), "name", sewageJob.getName())
                            .orderByDesc("expiration_date")
                            .orderByDesc("job_year")
                            .orderByDesc("job_season")
                            .orderByAsc("name")
            );
        } else {
            String jobIdSql = "SELECT ji.job_id FROM b_sewage_job_item ji LEFT JOIN b_sewage_treatment_plant stp ON ji.plant_id=stp.id ";
            if (StrUtil.endWith(userOrg.getAreaCode(), "0000")) {
                // 省级
                jobIdSql += "WHERE stp.province_code='" + userOrg.getAreaCode() + "'";
            } else if (StrUtil.endWith(userOrg.getAreaCode(), "00")) {
                // 市级
                jobIdSql += "WHERE stp.city_code='" + userOrg.getAreaCode() + "'";
            } else {
                // 区级
                jobIdSql += "WHERE stp.district_code='" + userOrg.getAreaCode() + "'";
            }

            list = sewageJobService.getSewageJobVOList(Wrappers.<SewageJob>query()
                    .eq("status", 1)        // 非鉴定中心用户只能看开到发布后的任务
                    .like(StrUtil.isNotBlank(sewageJob.getName()), "name", sewageJob.getName())
                    .inSql("id", jobIdSql)       // 只查包含本用户所在区域的污水厂的任务
                    .orderByDesc("expiration_date")
                    .orderByDesc("job_year")
                    .orderByDesc("job_season")
                    .orderByAsc("name")
            );

        }
        return R.ok(list);
    }


    /**
     * 新增污水检测任务
     *
     * @param sewageJob 污水检测任务
     * @return R
     */
    @ApiOperation(value = "新增污水检测任务", notes = "新增污水检测任务")
    @SysLog("新增污水检测任务")
    @PostMapping
    @PreAuthorize("@pms.hasPermission('SewageJobCreate')")
    public R postAddObject(@RequestBody SewageJob sewageJob, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        if (StrUtil.isBlank(sewageJob.getName())) {
            return R.failed(sewageJob, "新的污水检测任务创建失败, 任务名称不能为空");
        }
        if (sewageJob.getJobYear() == null) {
            return R.failed(sewageJob, "新的污水检测任务创建失败, 任务年份不能为空");
        }
        if (sewageJob.getJobSeason() == null) {
            return R.failed(sewageJob, "新的污水检测任务创建失败, 任务季度不能为空");
        }
//		if ((sewageJob.getJobSeason()<1) || (sewageJob.getJobSeason()>4)) {
//			return R.failed(sewageJob, "新的污水检测任务创建失败, 任务季度的有效值为1~4");
//		}

        sewageJob.setId(IdWorker.get32UUID().toUpperCase());
        sewageJob.setJobNo(sewageJobService.getNewJobNo(sewageJob.getJobYear(), sewageJob.getJobSeason()));
        sewageJob.setLauncheOrgId(dlpUser.getOrgId());
        sewageJob.setStatus(0);    // 初始的状态是刚创建
        long count = sewageJobService.count(Wrappers.<SewageJob>lambdaQuery().eq(SewageJob::getJobYear, sewageJob.getJobYear()).eq(SewageJob::getJobSeason, sewageJob.getJobSeason()));
        if (count>0){
            return R.failed(sewageJob, "当前月份任务已创建，创建失败!");
        }
        if (sewageJobService.save(sewageJob)) {
            return R.ok(sewageJob, "新的污水检测任务创建成功");
        } else {
            return R.failed(sewageJob, "新的污水检测任务创建失败");
        }
    }

    /**
     * 修改污水检测任务
     *
     * @param sewageJob 污水检测任务
     * @return R
     */
    @ApiOperation(value = "修改污水检测任务 <--- 这个接口会被禁止,", notes = "修改污水检测任务 <--- 这个接口会被禁止,")
    @SysLog("修改污水检测任务")
    @PutMapping
//    @PreAuthorize("@pms.hasPermission('SewageJobEdit')" )
    public R putUpdateById(@RequestBody SewageJob sewageJob, HttpServletRequest theHttpServletRequest) {

        return R.ok(sewageJobService.updateById(sewageJob));
    }

    /**
     * 发布者修改污水检测任务
     *
     * @param sewageJob 污水检测任务
     * @return R
     */
    @ApiOperation(value = "发布者修改污水检测任务", notes = "发布者修改污水检测任务<br>" +
            "保存提交的以下属性:\n" +
            "job_no \n" +
            "name 任务名称\n" +
            "job_year 任务所属年份\n" +
            "job_season 任务所属季度\n" +
            "description 任务描述\n" +
            "launche_org_id 发布单位id(强制设置为当前用户所属机构的id)\n" +
            "source 任务来源\n" +
            "expiration_date 过期时间\n" +
            "start_date 开始时间\n" +
            "comments 备注说明\n" +
            "launche_user_id 发布用户id (强制设置为当前用户id)\n"
    )
    @SysLog("发布者修改污水检测任务")
    @PutMapping("/launcher_save")
    @PreAuthorize("@pms.hasPermission('SewageJobEdit')")
    public R launcherSave(@RequestBody SewageJob sewageJob, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        SewageJob oldSewageJob = sewageJobService.getById(sewageJob.getId());
        if (oldSewageJob == null) {
            throw new RuntimeException(String.format("没有找到id为 %s 的污水检测任务", sewageJob.getId()));
        }

        if (!StrUtil.equalsIgnoreCase(dlpUser.getOrgId(), oldSewageJob.getLauncheOrgId())) {
            throw new RuntimeException(String.format("你没有权限修改id为 %s 的污水检测任务的数据", sewageJob.getId()));
        }

        if (StrUtil.isBlank(sewageJob.getJobNo())) {
            sewageJob.setJobNo(sewageJobService.getNewJobNo(sewageJob.getJobYear(), sewageJob.getJobSeason()));
        }

//		if (oldSewageJob.getStatus() != SewageJobStatusConstants.JOB_STATUS_CREATED.getStatus()     // 刚创建未发布
//				&& oldSewageJob.getStatus() != SewageJobStatusConstants.JOB_STATUS_REVOCATION.getStatus()   // 被撤回
//				&& oldSewageJob.getStatus() != SewageJobStatusConstants.JOB_STATUS_REFUSED.getStatus()  // 被拒绝
//		)
//		{
//			throw new RuntimeException(String.format("id为 %s 的污水检测任务已经不能修改", sewageJob.getId()));
//		}

        UpdateWrapper<SewageJob> updateWrapper = new UpdateWrapper<SewageJob>();
        updateWrapper.eq("id", sewageJob.getId());        // 查询条件是 id 相等

        updateWrapper.set("job_no", sewageJob.getJobNo());
        updateWrapper.set("name", sewageJob.getName());
        updateWrapper.set("job_year", sewageJob.getJobYear());
        updateWrapper.set("job_season", sewageJob.getJobSeason());
        updateWrapper.set("description", sewageJob.getDescription());
        updateWrapper.set("launche_org_id", dlpUser.getOrgId());
        updateWrapper.set("source", sewageJob.getSource());
        updateWrapper.set("expiration_date", sewageJob.getExpirationDate());
        updateWrapper.set("start_date", sewageJob.getStartDate());
        updateWrapper.set("comments", sewageJob.getComments());
        updateWrapper.set("launche_user_id", dlpUser.getId());

        if (!sewageJobService.update(null, updateWrapper)) {
            throw new RuntimeException("保存污水检测任务信息失败");
        }

        return R.ok(sewageJobService.getById(sewageJob.getId()), "保存污水检测任务信息成功");
    }


    /**
     * 发布者发布污水检测任务
     *
     * @param sewageJob 污水检测任务
     * @return R
     */
    @ApiOperation(value = "发布者发布污水检测任务", notes = "发布者发布污水检测任务<br>" +
            "保存提交的以下属性:\n" +
            "job_no \n" +
            "name 任务名称\n" +
            "job_year 任务所属年份\n" +
            "job_season 任务所属季度\n" +
            "description 任务描述\n" +
            "launche_org_id 发布单位id(强制设置为当前用户所属机构的id)\n" +
            "source 任务来源\n" +
            "expiration_date 过期时间\n" +
            "start_date 开始时间\n" +
            "comments 备注说明\n" +
            "launche_user_id 发布用户id (强制设置为当前用户id)\n"
    )
    @SysLog("发布者发布污水检测任务")
    @PutMapping("/launche")
    @PreAuthorize("@pms.hasPermission('SewageJobEdit')")
    public R launche(@RequestBody SewageJob sewageJob, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
        SewageJob oldSewageJob = sewageJobService.getById(sewageJob.getId());
        if (oldSewageJob == null) {
            throw new RuntimeException(String.format("没有找到id为 %s 的污水检测任务", sewageJob.getId()));
        }
        if (!StrUtil.equalsIgnoreCase(dlpUser.getOrgId(), oldSewageJob.getLauncheOrgId())) {
            throw new RuntimeException(String.format("你没有权限修改id为 %s 的污水检测任务的数据", sewageJob.getId()));
        }
        UpdateWrapper<SewageJob> updateWrapper = new UpdateWrapper<SewageJob>();
        updateWrapper.eq("id", sewageJob.getId());        // 查询条件是 id 相等
        //updateWrapper.set(sewageJob.getJobNo() != null, "name", sewageJob.getName());
        //updateWrapper.set(sewageJob.getJobYear() != null, "job_year", sewageJob.getJobYear());
        //updateWrapper.set(sewageJob.getJobSeason() != null, "job_season", sewageJob.getJobSeason());
        //updateWrapper.set(sewageJob.getDescription() != null, "description", sewageJob.getDescription());
        updateWrapper.set("launche_org_id", dlpUser.getOrgId());
        //updateWrapper.set(sewageJob.getSource() != null, "source", sewageJob.getSource());
        //updateWrapper.set(sewageJob.getExpirationDate() != null, "expiration_date", sewageJob.getExpirationDate());
        //updateWrapper.set(sewageJob.getStartDate() != null, "start_date", sewageJob.getStartDate());
        //updateWrapper.set(sewageJob.getComments() != null, "comments", sewageJob.getComments());
        updateWrapper.set("launche_user_id", dlpUser.getId());

        updateWrapper.set("status", 1);
        updateWrapper.set("launche_date", LocalDateTime.now());

        if (!sewageJobService.update(null, updateWrapper)) {
            throw new RuntimeException("发布污水检测任务失败");
        }
        //生成检材信息
        System.out.println("开始生成检材信息");
        sewageJobService.generateSewageMaterial(oldSewageJob);
        return R.ok(sewageJobService.getById(sewageJob.getId()), "发布污水检测任务成功");
    }

    /**
     * 发布者取消发布污水检测任务
     *
     * @param sewageJob 污水检测任务
     * @return R
     */
    @ApiOperation(value = "发布者取消发布污水检测任务", notes = "发布者取消发布污水检测任务<br>" +
            "保存提交的以下属性:\n" +
            "job_no \n" +
            "name 任务名称\n" +
            "job_year 任务所属年份\n" +
            "job_season 任务所属季度\n" +
            "description 任务描述\n" +
            "launche_org_id 发布单位id(强制设置为当前用户所属机构的id)\n" +
            "source 任务来源\n" +
            "expiration_date 过期时间\n" +
            "start_date 开始时间\n" +
            "comments 备注说明\n" +
            "launche_user_id 发布用户id (强制设置为当前用户id)\n" +
            "\n" +
            "如果一个任务，已经录入了检材，则不能取消发布。"
    )
    @SysLog("发布者取消发布污水检测任务")
    @PutMapping("/unlaunche")
    @PreAuthorize("@pms.hasPermission('SewageJobEdit')")
    public R unlaunche(@RequestBody SewageJob sewageJob, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        SewageJob oldSewageJob = sewageJobService.getById(sewageJob.getId());
        if (oldSewageJob == null) {
            throw new RuntimeException(String.format("没有找到id为 %s 的污水检测任务", sewageJob.getId()));
        }

        if (!StrUtil.equalsIgnoreCase(dlpUser.getOrgId(), oldSewageJob.getLauncheOrgId())) {
            throw new RuntimeException(String.format("你没有权限修改id为 %s 的污水检测任务的数据", sewageJob.getId()));
        }

        // 看一下 任务清单的情况
        String jobId = sewageJob.getId();
        List<SewageJobItemVO> jiList = sewageJobItemService.getSewageJobItemVOList(Wrappers.<SewageJobItem>query()
                .eq(StrUtil.isNotBlank(jobId), "ji.job_id", jobId)
        );

        List<SewageJobIdentificationMaterialVO> jimList = sewageJobIdentificationMaterialService.getSewageJobIdentificationMaterialVOList(Wrappers.<SewageJobIdentificationMaterial>query()
                .eq(StrUtil.isNotBlank(jobId), "sjim.job_id", jobId)
        );
        if (jimList.size() > 0) {
            throw new RuntimeException("这个任务存在已录入的检材信息, 不能撤回了");
        }

        UpdateWrapper<SewageJob> updateWrapper = new UpdateWrapper<SewageJob>();
        updateWrapper.eq("id", sewageJob.getId());        // 查询条件是 id 相等

        updateWrapper.set(sewageJob.getJobNo() != null, "name", sewageJob.getName());
        updateWrapper.set(sewageJob.getJobYear() != null, "job_year", sewageJob.getJobYear());
        updateWrapper.set(sewageJob.getJobSeason() != null, "job_season", sewageJob.getJobSeason());
        updateWrapper.set(sewageJob.getDescription() != null, "description", sewageJob.getDescription());
        updateWrapper.set("launche_org_id", dlpUser.getOrgId());
        updateWrapper.set(sewageJob.getSource() != null, "source", sewageJob.getSource());
        updateWrapper.set(sewageJob.getExpirationDate() != null, "expiration_date", sewageJob.getExpirationDate());
        updateWrapper.set(sewageJob.getStartDate() != null, "start_date", sewageJob.getStartDate());
        updateWrapper.set(sewageJob.getComments() != null, "comments", sewageJob.getComments());
        updateWrapper.set("launche_user_id", dlpUser.getId());

        updateWrapper.set("status", 0);
        updateWrapper.set("launche_date", null);

        if (!sewageJobService.update(null, updateWrapper)) {
            throw new RuntimeException("取消发布污水检测任务失败");
        }

        return R.ok(sewageJobService.getById(sewageJob.getId()), "取消发布污水检测任务成功");
    }

    //撤回发布的任务
    @ApiOperation(value = "撤回发布的任务", notes = "撤回发布的任务")
    @PostMapping("/undoTask")
    public R undoTask(String taskID) {
        Assert.notBlank(taskID, "参数任务ID不能为空");
        return R.ok(sewageJobService.undoTask(taskID), "");
    }

    /**
     * 通过id删除污水检测任务
     *
     * @param id id
     * @return R
     */
    @ApiOperation(value = "通过id删除污水检测任务", notes = "通过id删除污水检测任务")
    @SysLog("通过id删除污水检测任务")
    @DeleteMapping("/{id}")
    @PreAuthorize("@pms.hasPermission('SewageJobDelete')")
    @Transactional(rollbackFor = {RuntimeException.class, Exception.class})
    public R deleteById(@PathVariable String id, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        SewageJob oldSewageJob = sewageJobService.getById(id);
        if (oldSewageJob == null) {
            throw new RuntimeException(String.format("没有找到id为 %s 的污水检测任务", id));
        }

        if (!StrUtil.equalsIgnoreCase(dlpUser.getOrgId(), oldSewageJob.getLauncheOrgId())) {
            throw new RuntimeException(String.format("你没有权限修改id为 %s 的污水检测任务的数据", id));
        }

        if (oldSewageJob.getStatus() > 0) {
            throw new RuntimeException(String.format("id为 %s 的污水检测任务已经发布, 只能删除未发布的任务", id));
        }

        sewageJobItemService.remove(Wrappers.<SewageJobItem>query().eq("job_id", id));

        return R.ok(sewageJobService.removeById(id));
    }

    /**
     * 统计各污水厂的送检情况
     */
    @ApiOperation(value = "统计各污水厂的送检情况", notes = "统计各污水厂的送检情况")
    @GetMapping("/querySendData")
    public R querySendData(String jobId, String areaCode, String materialStatus, String treeLayer, HttpServletRequest theHttpServletRequest) {
        Assert.notNull(jobId, "任务ID不能为空");
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
        R ret = remoteOrgService.getById(dlpUser.getOrgId());
        String currentUserAreaCode = "";
        if (ret.getCode() == CommonConstants.SUCCESS) {
            currentUserAreaCode = ((SysOrg) ret.getData()).getAreaCode();
        } else {
            return R.failed(String.format("没有找到 orgId 为 %s 的机构, 请确认用户所属机构的正确性!", dlpUser.getOrgId()));
        }
        int targetTreeLayer = StringUtils.isNotBlank(treeLayer) ? Integer.parseInt(treeLayer) : 5;
        return R.ok(sewageJobService.querySewageJcSendData(jobId, currentUserAreaCode, materialStatus, targetTreeLayer), "获取数据信息成功");
    }

    /**
     * 初始化污水系统
     */
    @GetMapping("/initSewageJobData")
    public R initSewageJobData() {
        //1、清除污水任务之前，需要先清除污水任务明细，污水检材明细，污水厂数据，必须严格按照顺序，防止脏数据的产生
        //1、清除检材表  2、清除任务明细表，3、清除污水厂表，4、清除任务表
        List<SewageJob> sewageJobList = sewageJobService.list();
        for (SewageJob sewageJob : sewageJobList) {
            //删除该任务对应的明细任务项
            sewageJobItemService.remove(Wrappers.<SewageJobItem>query().eq("job_id", sewageJob.getId()));
            //获取该任务下的检材明细
            sewageJobIdentificationMaterialService.remove(Wrappers.<SewageJobIdentificationMaterial>query().eq("job_id", sewageJob.getId()));
            sewageJobService.removeById(sewageJob);
        }
        //清除污水厂
        sewageTreatmentPlantService.remove(Wrappers.<SewageTreatmentPlant>query().ne("id", "999999"));
        return R.ok(1, "操作成功");
    }

    @PostMapping("/lead/into/{year}/{month}")
    @ApiOperation(value = "导入污水任务Excel",notes = "参数：sewageJobIdentificationMaterialList：传入的Excel行的对象数组，year：拼接在路径上的任务年份，month：拼接在对象上的任务月份")
    public R LeadIntoJob(@RequestBody List<SewageJobIdentificationMaterial> sewageJobIdentificationMaterialList, @PathVariable(value = "year") int year, @PathVariable(value = "month") int month, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
        return sewageJobService.LeadIntoJob(sewageJobIdentificationMaterialList, year, month, dlpUser) ? R.ok("导入成功") : R.failed("导入失败");
    }

    @GetMapping("/getJobAndMaterial/page")
    @ApiOperation(value = "分页查询污水任务以及污水检材")
    public R getJobAndMaterialPage(Page page, Integer status, String keywords) {
        return R.ok(sewageJobService.getJobAndMaterialPage(page, status, keywords));
    }

    @GetMapping("/statisticsDiffStatusJob")
    @ApiOperation(value = "根据任务状态 统计不同状态下的任务数量")
    public R<StatisticsDiffStatusJobDTO> statisticsDiffStatusJob() {
        return sewageJobService.statisticsDiffStatusJob();
    }

    @GetMapping("/querySewageJobByStartDate")
    @ApiOperation(value = "根据任务发布时间查询在这之前的任务，并根据发布时间降序，然后根据limit 取不同数量的值")
    public R<List<SewageJobVO>> querySewageJobByStartDate(@RequestParam("startDate") String startDate,
                                                          @RequestParam("limit") int limit) {
        return sewageJobService.querySewageJobByStartDate(startDate, limit);
    }

    @GetMapping("/getIMQuantity")
    @ApiOperation(value = "查询这个账户（机构）下面所送检且已受理的所有污水检材的数量")
    public R<Integer> getIMQuantity(@RequestParam("orgId") String orgId) {
        final SysOrg userOrg = getOrgInfo(orgId);
        return R.ok(sewageJobService.getIMQuantity(userOrg.getAreaCode()));
    }

    @GetMapping("/getSewageMarkers")
    @ApiOperation(value = "查询各个状态下的检材数量")
    public R<List<MarkersVO>> getSewageMarkers(@RequestParam("orgId") String orgId) {
        final SysOrg userOrg = getOrgInfo(orgId);
        return R.ok(sewageJobService.getSewageMarkers(userOrg.getAreaCode()));
    }

    @GetMapping("/getImQuantityForAccept")
    @ApiOperation(value = "受理系统统计已受理的污水检材数量")
    public R<Integer> getImQuantityForAccept() {
        return R.ok(sewageJobService.getImQuantityForAccept());
    }

    private SysOrg getOrgInfo(String orgId) {
        SysOrg userOrg = null;
        R<SysOrg> r = remoteOrgService.getById(orgId);
        if (r.getCode() == CommonConstants.SUCCESS) {
            userOrg = r.getData();
        } else {
            throw new RuntimeException(String.format("没有找到 orgId 为 %s 的机构, 请确认用户所属机构的正确性!", orgId));
        }
        return userOrg;
    }
}
