package digital.laboratory.platform.sewage.controller;

import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.constant.CommonConstants;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.core.util.TestUtils;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.sewage.entity.SewageJobIdentificationMaterial;
import digital.laboratory.platform.sewage.entity.SewageTreatmentPlant;
import digital.laboratory.platform.sewage.entity.UpdateInfo;
import digital.laboratory.platform.sewage.dto.SewageJobIdentificationMaterialDTO;
import digital.laboratory.platform.sewage.entity.SewageJobItem;
import digital.laboratory.platform.sewage.mapper.SewageJobIdentificationMaterialMapper;
import digital.laboratory.platform.sewage.service.*;
import digital.laboratory.platform.sys.dto.entrustment.CommonIdDto;
import digital.laboratory.platform.sys.entity.Deliverer;
import digital.laboratory.platform.sys.entity.SysOrg;
import digital.laboratory.platform.sys.entity.SysUser;
import digital.laboratory.platform.sys.entity.entrustment.Sample;
import digital.laboratory.platform.sys.entity.entrustment.SampleBox;
import digital.laboratory.platform.sewage.vo.SewageJobIdentificationMaterialVO;
import digital.laboratory.platform.sewage.vo.SewageJobItemVO;
import digital.laboratory.platform.sys.feign.*;
import io.seata.spring.annotation.GlobalTransactional;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.security.Principal;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;


/**
 * 污水任务的检材信息
 *
 * @author Zhang Xiaolong created at  2022-07-26
 * @describe 污水任务的检材信息 前端控制器
 */
@RestController
@RequestMapping("/sewage_job_identification_material")
@Api(value = "sewage_job_identification_material", tags = "113-污水任务的检材信息管理")
public class  SewageJobIdentificationMaterialController {

    @Resource
    private SewageJobIdentificationMaterialService sewageJobIdentificationMaterialService;

    @Resource
    private SewageJobItemService sewageJobItemService;

    @Resource
    private SewageJobService sewageJobService;
    
    @Resource
    private SewageTreatmentPlantService sewageTreatmentPlantService;
    
    
    @Resource
    private RemoteDeliverService remoteDeliverService;
    
    @Resource
    private RemoteUserService remoteUserService;

    @Resource
    private RemoteOrgService remoteOrgService;
    
    @Resource
    private SewageJobIdentificationMaterialMapper sewageJobIdentificationMaterialMapper;

    @Resource
    RemoteSampleService remoteSampleService; //远程feign接口

    @Resource
    private RemoteEntrustmentService remoteEntrustmentService;

    /**
     * 通过id查询污水任务的检材信息
     *
     * @param id id
     * @return R
     */
    @ApiOperation(value = "通过id查询", notes = "通过id查询")
    @GetMapping("/{id}")
    @PreAuthorize("@pms.hasPermission('SewageJobIdentificationMaterialGet')")
    public R getById(@PathVariable("id") String id, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        final SysOrg userOrg;    // 定义成 final才可以被后面的 Lambda 表达式使用
        R r = remoteOrgService.getById(dlpUser.getOrgId());
        if (r.getCode() == CommonConstants.SUCCESS) {
            userOrg = (SysOrg) r.getData();
        } else {
            return R.failed(String.format("没有找到 orgId 为 %s 的机构, 请确认用户所属机构的正确性!", dlpUser.getOrgId()));
        }
        int userLevel = 99;
        if (dlpUser.isStaff()) {
            // 分中心
            userLevel = 0;
        } else if (StrUtil.endWith(userOrg.getAreaCode(), "0000")) {
            // 省级
            userLevel = 1;
        } else if (StrUtil.endWith(userOrg.getAreaCode(), "00")) {
            // 市级
            userLevel = 2;
        } else {
            // 区级
            userLevel = 3;
        }
        return R.ok(sewageJobIdentificationMaterialService.getById(id));
    }


    /**
     * 通过id查询污水任务的检材信息VO
     *
     * @param id id
     * @return R
     */
    @ApiOperation(value = "通过id查询VO", notes = "通过id查询VO")
    @GetMapping("/vo/{id}")
    @PreAuthorize("@pms.hasPermission('SewageJobIdentificationMaterialGet')")
    public R<SewageJobIdentificationMaterialVO> getVOById(@PathVariable("id") String id, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        return R.ok(sewageJobIdentificationMaterialService.getVOById(id));
    }


    /**
     * 根据提供的编码取对象
     * 可以查询的编码有 imNo/sample1No/sample2No
     *
     * @param imNo
     * @param sample1No
     * @param sample2No
     * @param code
     * @return
     */
    @ApiOperation(value = "根据提供的编码取对象", notes = "根据提供的编码取对象\n" +
            "<pre>\n" +
            "查询参数:\n" +
            "imNo 检材编号\n" +
            "sample1No 样本1编号\n" +
            "sample2No 样本2编码\n" +
            "code 这个 code 对应 imNo/sample1No/sample2No 中的任意一个, 例如:\n" +
            "	如果有一个检材, 检材编码imNo为 abc, 样本1编码为 abc1, 样本2编码为 abc2 , 我们可以使用查询参数 code=abc 或 code=abc1 或 code=abc2 都能查询到它" +
            "</pre>" +
            "")
    @GetMapping("/get_by_code")
    @PreAuthorize("@pms.hasPermission('SewageJobIdentificationMaterialGet')")
    public R<SewageJobIdentificationMaterial> getByCode(
            @RequestParam(value = "imNo", required = false) String imNo,
            @RequestParam(value = "sample1No", required = false) String sample1No,
            @RequestParam(value = "sample2No", required = false) String sample2No,
            @RequestParam(value = "code", required = false) String code, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        final SysOrg userOrg;    // 定义成 final才可以被后面的 Lambda 表达式使用
        R r = remoteOrgService.getById(dlpUser.getOrgId());
        if (r.getCode() == CommonConstants.SUCCESS) {
            userOrg = (SysOrg) r.getData();
        } else {
            return R.failed(String.format("没有找到 orgId 为 %s 的机构, 请确认用户所属机构的正确性!", dlpUser.getOrgId()));
        }
        int userLevel = 99;
        if (dlpUser.isStaff()) {
            // 分中心
            userLevel = 0;
        } else if (StrUtil.endWith(userOrg.getAreaCode(), "0000")) {
            // 省级
            userLevel = 1;
        } else if (StrUtil.endWith(userOrg.getAreaCode(), "00")) {
            // 市级
            userLevel = 2;
        } else {
            // 区级
            userLevel = 3;
        }

        List<SewageJobIdentificationMaterialVO> list = sewageJobIdentificationMaterialService.getSewageJobIdentificationMaterialVOList(
                Wrappers.<SewageJobIdentificationMaterial>query()
                        .eq(StrUtil.isNotBlank(imNo), "sjim.im_no", imNo)
                        .eq(StrUtil.isNotBlank(sample1No), "sjim.sample1_no", sample1No)
                        .eq(StrUtil.isNotBlank(sample2No), "sjim.sample2_no", sample2No)
                        .and(StrUtil.isNotBlank(code), (wrapper) -> wrapper                // 如果不是分中心员工
                                .eq("sjim.im_no", code)    // im_no
                                .or()
                                .eq("sjim.sample1_no", code)    // sample1_no
                                .or()
                                .eq("sjim.sample2_no", code)    // sample2_no
                        )
                        .and(!dlpUser.isStaff(), (wrapper) -> wrapper                // 如果不是分中心员工
                                .eq("stp.province_code", userOrg.getAreaCode())    // 污水处理厂的省级地区码与用户机构所在地区一致
                                .or()
                                .eq("stp.city_code", userOrg.getAreaCode())    // 污水处理厂的市级地区码与用户机构所在地区一致
                                .or()
                                .eq("stp.district_code", userOrg.getAreaCode())    // 污水处理厂的区级地区码与用户机构所在地区一致
                        )
                        .orderByAsc("sample_name")
        );

        if (list == null) {
            throw new RuntimeException(String.format("查询出错, 返回结果为空"));
        }
        if (list.size() == 0) {
            throw new RuntimeException(String.format("没有找到符合条件的对象"));
        }

        return R.ok(list.get(0));
    }


    /**
     * 列表查询
     *
     * @param sewageJobIdentificationMaterial 污水任务的检材信息, 查询条件
     * @return
     */
    @ApiOperation(value = "列表查询", notes = "列表查询")
    @GetMapping("/list")
    @PreAuthorize("@pms.hasPermission('SewageJobIdentificationMaterialList')")
    public R getSewageJobIdentificationMaterialList(SewageJobIdentificationMaterial sewageJobIdentificationMaterial, @RequestParam(value = "statuses", required = false) List<Integer> statuses, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        final SysOrg userOrg;    // 定义成 final才可以被后面的 Lambda 表达式使用
        R r = remoteOrgService.getById(dlpUser.getOrgId());
        if (r.getCode() == CommonConstants.SUCCESS) {
            userOrg = (SysOrg) r.getData();
        } else {
            return R.failed(String.format("没有找到 orgId 为 %s 的机构, 请确认用户所属机构的正确性!", dlpUser.getOrgId()));
        }
        int userLevel = 99;
        if (dlpUser.isStaff()) {
            // 分中心
            userLevel = 0;
        } else if (StrUtil.endWith(userOrg.getAreaCode(), "0000")) {
            // 省级
            userLevel = 1;
        } else if (StrUtil.endWith(userOrg.getAreaCode(), "00")) {
            // 市级
            userLevel = 2;
        } else {
            // 区级
            userLevel = 3;
        }

        List<SewageJobIdentificationMaterialVO> list = sewageJobIdentificationMaterialService.getSewageJobIdentificationMaterialVOList(
                Wrappers.<SewageJobIdentificationMaterial>query()
                        //.first("true")	// 后面的 .and 会导致 where 语句为空, 加这个 true 确保 where 不空
                        .eq(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getJobId()), "sjim.job_id", sewageJobIdentificationMaterial.getJobId())
                        .eq(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getJobItemId()), "sjim.job_item_id", sewageJobIdentificationMaterial.getJobItemId())
                        .eq(sewageJobIdentificationMaterial.getStatus() != null, "sjim.status", sewageJobIdentificationMaterial.getStatus())
                        .in(statuses != null && statuses.size() > 0, "sjim.status", statuses)
                        .and(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getImNo()) ||
                                        StrUtil.isNotBlank(sewageJobIdentificationMaterial.getSample1No()) ||
                                        StrUtil.isNotBlank(sewageJobIdentificationMaterial.getSample2No()),
                                (wrapper) -> wrapper
                                        .eq(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getImNo()), "sjim.im_no", sewageJobIdentificationMaterial.getImNo())
                                        .or()
                                        .eq(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getSample1No()), "sjim.sample1_no", sewageJobIdentificationMaterial.getSample1No())
                                        .or()
                                        .eq(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getSample2No()), "sjim.sample2_no", sewageJobIdentificationMaterial.getSample2No())
                        )
                        .and(!dlpUser.isStaff(), (wrapper) -> wrapper                // 如果不是分中心员工
                                .eq("stp.province_code", userOrg.getAreaCode())    // 污水处理厂的省级地区码与用户机构所在地区一致
                                .or()
                                .eq("stp.city_code", userOrg.getAreaCode())    // 污水处理厂的市级地区码与用户机构所在地区一致
                                .or()
                                .eq("stp.district_code", userOrg.getAreaCode())    // 污水处理厂的区级地区码与用户机构所在地区一致
                        )
                        // .eq(StrUtil.isNotBlank(sewageTreatmentPlant.getProvinceCode()), "province_code", sewageTreatmentPlant.getProvinceCode())
                        // .eq(StrUtil.isNotBlank(sewageTreatmentPlant.getCityCode()), "city_code", sewageTreatmentPlant.getCityCode())
                        // .eq(StrUtil.isNotBlank(sewageTreatmentPlant.getDistrictCode()), "district_code", sewageTreatmentPlant.getDistrictCode())
                        .orderByAsc("sample_name")
        );

        return R.ok(list);
    }


    /**
     * 分页查询检材
     *
     * @param page                            分页对象
     * @param sewageJobIdentificationMaterial 污水任务的检材信息
     * @return
     */
    @ApiOperation(value = "分页查询检材", notes = "分页查询检材\n" +
            "参数:\n" +
            "page - 分页信息: current, size。例如: current=1&size=10\n" +
            "keyword - 查询关键字。 查询以下字段是否包含 keyword：\n" +
            "	检材编号、样本1编号、样本2编号、样本名称、采集人、采集单位、送检员姓名、污水厂名称、污水厂简称、污水厂地址、污水厂覆盖范围、省禁毒办联系人、市禁毒办联系人、区县禁毒办联系人\n" +
            "statuses - 状态列表, 例如: statuses=1,2,3\n" +
            "status - 单个状态, 例如: status=1\n" +
            "imNo - 检材编号\n" +
            "sample1No - 样本1编号\n" +
            "sample2No - 样本2编号\n" +
            "")
    @GetMapping("/page")
    @PreAuthorize("@pms.hasPermission('SewageJobIdentificationMaterialList')")
    public R getSewageJobIdentificationMaterialPage(Page page, String keyword, SewageJobIdentificationMaterial sewageJobIdentificationMaterial,
                                                    @RequestParam(value = "statuses", required = false) List<Integer> statuses,
                                                    HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        final SysOrg userOrg;    // 定义成 final才可以被后面的 Lambda 表达式使用
        R r = remoteOrgService.getById(dlpUser.getOrgId());
        if (r.getCode() == CommonConstants.SUCCESS) {
            userOrg = (SysOrg) r.getData();
        } else {
            return R.failed(String.format("没有找到 orgId 为 %s 的机构, 请确认用户所属机构的正确性!", dlpUser.getOrgId()));
        }
        int userLevel = 99;
        if (dlpUser.isStaff()) {
            // 分中心
            userLevel = 0;
        } else if (StrUtil.endWith(userOrg.getAreaCode(), "0000")) {
            // 省级
            userLevel = 1;
        } else if (StrUtil.endWith(userOrg.getAreaCode(), "00")) {
            // 市级
            userLevel = 2;
        } else {
            // 区级
            userLevel = 3;
        }

        if (StrUtil.isNotBlank(keyword)) {
            keyword = keyword.trim();
        }
        final String theKeyword = keyword;

//System.out.println(String.format("statuses=%s", statuses));
        IPage<SewageJobIdentificationMaterialVO> thePage = sewageJobIdentificationMaterialService.getSewageJobIdentificationMaterialVOPage(page,
                Wrappers.<SewageJobIdentificationMaterial>query()
                        //.first("true")	// 后面的 .and 会导致 where 语句为空, 加这个 true 确保 where 不空
                        .and(StrUtil.isNotBlank(theKeyword),
                                (wrapper) -> wrapper
                                        .eq("sjim.im_no", theKeyword)
                                        .or()
                                        .eq("sjim.express_number", theKeyword)
                                        .or()
                                        .eq("sjim.sample1_no", theKeyword)
                                        .or()
                                        .eq("sjim.sample2_no", theKeyword)
                                        .or()
                                        .like("sjim.sample_name", theKeyword)
                                        .or()
                                        .like("sjim.collector", theKeyword)
                                        .or()
                                        .like("sjim.collector_group", theKeyword)
                                        .or()
                                        .like("sjim.deliverer_name", theKeyword)
                                        .or()
                                        .like("stp.name", theKeyword)
                                        .or()
                                        .like("stp.short_name", theKeyword)
                                        .or()
                                        .like("stp.address", theKeyword)
                                        .or()
                                        .like("stp.coverage_area", theKeyword)
                                        .or()
                                        .like("stp.province_contacts", theKeyword)
                                        .or()
                                        .like("stp.city_contacts", theKeyword)
                                        .or()
                                        .like("stp.district_contacts", theKeyword)
                        )
                        .eq(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getJobId()), "sjim.job_id", sewageJobIdentificationMaterial.getJobId())
                        .eq(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getJobItemId()), "sjim.job_item_id", sewageJobIdentificationMaterial.getJobItemId())
                        .eq(sewageJobIdentificationMaterial.getStatus() != null, "sjim.status", sewageJobIdentificationMaterial.getStatus())
                        .in(statuses != null && statuses.size() > 0, "sjim.status", statuses)
                        .and(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getImNo()) ||
                                        StrUtil.isNotBlank(sewageJobIdentificationMaterial.getSample1No()) ||
                                        StrUtil.isNotBlank(sewageJobIdentificationMaterial.getSample2No()),
                                (wrapper) -> wrapper
                                        .eq(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getImNo()), "sjim.im_no", sewageJobIdentificationMaterial.getImNo())
                                        .or()
                                        .eq(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getSample1No()), "sjim.sample1_no", sewageJobIdentificationMaterial.getSample1No())
                                        .or()
                                        .eq(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getSample2No()), "sjim.sample2_no", sewageJobIdentificationMaterial.getSample2No())
                        )
                        .and(!dlpUser.isStaff(), (wrapper) -> wrapper                // 如果不是分中心员工
                                .eq("stp.province_code", userOrg.getAreaCode())    // 污水处理厂的省级地区码与用户机构所在地区一致
                                .or()
                                .eq("stp.city_code", userOrg.getAreaCode())    // 污水处理厂的市级地区码与用户机构所在地区一致
                                .or()
                                .eq("stp.district_code", userOrg.getAreaCode())    // 污水处理厂的区级地区码与用户机构所在地区一致
                        )
                        .orderByAsc("sample_name")
        );

        return R.ok(thePage);
    }

    /**
     * 列表查询
     *
     * @param sewageJobIdentificationMaterial 污水任务的检材信息, 查询条件
     * @return
     */
    @ApiOperation(value = "列表查询", notes = "列表查询")
    @GetMapping("/list/gab")
    @PreAuthorize("@pms.hasPermission('SewageJobIdentificationMaterialList')")
    public R getSewageJobGABIdentificationMaterialList(SewageJobIdentificationMaterial sewageJobIdentificationMaterial, @RequestParam(value = "statuses", required = false) List<Integer> statuses, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        final SysOrg userOrg;    // 定义成 final才可以被后面的 Lambda 表达式使用
        R r = remoteOrgService.getById(dlpUser.getOrgId());
        if (r.getCode() == CommonConstants.SUCCESS) {
            userOrg = (SysOrg) r.getData();
        } else {
            return R.failed(String.format("没有找到 orgId 为 %s 的机构, 请确认用户所属机构的正确性!", dlpUser.getOrgId()));
        }
        int userLevel = 99;
        if (dlpUser.isStaff()) {
            // 分中心
            userLevel = 0;
        } else if (StrUtil.endWith(userOrg.getAreaCode(), "0000")) {
            // 省级
            userLevel = 1;
        } else if (StrUtil.endWith(userOrg.getAreaCode(), "00")) {
            // 市级
            userLevel = 2;
        } else {
            // 区级
            userLevel = 3;
        }

        List<SewageJobIdentificationMaterialVO> list = sewageJobIdentificationMaterialService.getSewageJobGABIdentificationMaterialVOList(
                Wrappers.<SewageJobIdentificationMaterial>query()
                        //.first("true")	// 后面的 .and 会导致 where 语句为空, 加这个 true 确保 where 不空
                        .eq(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getJobId()), "sjim.job_id", sewageJobIdentificationMaterial.getJobId())
                        .eq(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getJobItemId()), "sjim.job_item_id", sewageJobIdentificationMaterial.getJobItemId())
                        .eq(sewageJobIdentificationMaterial.getStatus() != null, "sjim.status", sewageJobIdentificationMaterial.getStatus())
                        .in(statuses != null && statuses.size() > 0, "sjim.status", statuses)
                        .and(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getImNo()) ||
                                        StrUtil.isNotBlank(sewageJobIdentificationMaterial.getSample1No()) ||
                                        StrUtil.isNotBlank(sewageJobIdentificationMaterial.getSample2No()),
                                (wrapper) -> wrapper
                                        .eq(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getImNo()), "sjim.im_no", sewageJobIdentificationMaterial.getImNo())
                                        .or()
                                        .eq(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getSample1No()), "sjim.sample1_no", sewageJobIdentificationMaterial.getSample1No())
                                        .or()
                                        .eq(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getSample2No()), "sjim.sample2_no", sewageJobIdentificationMaterial.getSample2No())
                        )
                        .and(!dlpUser.isStaff(), (wrapper) -> wrapper                // 如果不是分中心员工
                                .eq("stp.province_code", userOrg.getAreaCode())    // 污水处理厂的省级地区码与用户机构所在地区一致
                                .or()
                                .eq("stp.city_code", userOrg.getAreaCode())    // 污水处理厂的市级地区码与用户机构所在地区一致
                                .or()
                                .eq("stp.district_code", userOrg.getAreaCode())    // 污水处理厂的区级地区码与用户机构所在地区一致
                        )
                        // .eq(StrUtil.isNotBlank(sewageTreatmentPlant.getProvinceCode()), "province_code", sewageTreatmentPlant.getProvinceCode())
                        // .eq(StrUtil.isNotBlank(sewageTreatmentPlant.getCityCode()), "city_code", sewageTreatmentPlant.getCityCode())
                        // .eq(StrUtil.isNotBlank(sewageTreatmentPlant.getDistrictCode()), "district_code", sewageTreatmentPlant.getDistrictCode())
                        .orderByAsc("sample_name")
        );

        return R.ok(list);
    }


    /**
     * 分页查询检材(公安部)
     *
     * @param page                            分页对象
     * @param sewageJobIdentificationMaterial 污水任务的检材信息
     * @return
     */
    @ApiOperation(value = "分页查询检材(公安部)", notes = "分页查询检材(公安部)\n" +
            "参数:\n" +
            "page - 分页信息: current, size。例如: current=1&size=10\n" +
            "keyword - 查询关键字。 查询以下字段是否包含 keyword：\n" +
            "	检材编号、样本1编号、样本2编号、样本名称、采集人、采集单位、送检员姓名、污水厂名称、污水厂简称、污水厂地址、污水厂覆盖范围、省禁毒办联系人、市禁毒办联系人、区县禁毒办联系人\n" +
            "statuses - 状态列表, 例如: statuses=1,2,3\n" +
            "status - 单个状态, 例如: status=1\n" +
            "imNo - 检材编号\n" +
            "sample1No - 样本1编号\n" +
            "sample2No - 样本2编号\n" +
            "")
    @GetMapping("/page/gab")
    @PreAuthorize("@pms.hasPermission('SewageJobIdentificationMaterialList')")
    public R getSewageJobGABIdentificationMaterialPage(Page page, String keyword, SewageJobIdentificationMaterial sewageJobIdentificationMaterial,
                                                       @RequestParam(value = "statuses", required = false) List<Integer> statuses,
                                                       HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        final SysOrg userOrg;    // 定义成 final才可以被后面的 Lambda 表达式使用
        R r = remoteOrgService.getById(dlpUser.getOrgId());
        if (r.getCode() == CommonConstants.SUCCESS) {
            userOrg = (SysOrg) r.getData();
        } else {
            return R.failed(String.format("没有找到 orgId 为 %s 的机构, 请确认用户所属机构的正确性!", dlpUser.getOrgId()));
        }
        int userLevel = 99;
        if (dlpUser.isStaff()) {
            // 分中心
            userLevel = 0;
        } else if (StrUtil.endWith(userOrg.getAreaCode(), "0000")) {
            // 省级
            userLevel = 1;
        } else if (StrUtil.endWith(userOrg.getAreaCode(), "00")) {
            // 市级
            userLevel = 2;
        } else {
            // 区级
            userLevel = 3;
        }

        if (StrUtil.isNotBlank(keyword)) {
            keyword = keyword.trim();
        }
        final String theKeyword = keyword;

//System.out.println(String.format("statuses=%s", statuses));
        IPage<SewageJobIdentificationMaterialVO> thePage = sewageJobIdentificationMaterialService.getSewageJobGABIdentificationMaterialVOPage(page,
                Wrappers.<SewageJobIdentificationMaterial>query()
                        //.first("true")	// 后面的 .and 会导致 where 语句为空, 加这个 true 确保 where 不空
                        .and(StrUtil.isNotBlank(theKeyword),
                                (wrapper) -> wrapper
                                        .eq("sjim.im_no", theKeyword)
                                        .or()
                                        .eq("sjim.sample1_no", theKeyword)
                                        .or()
                                        .eq("sjim.sample2_no", theKeyword)
                                        .or()
                                        .like("sjim.sample_name", theKeyword)
                                        .or()
                                        .like("sjim.collector", theKeyword)
                                        .or()
                                        .like("sjim.collector_group", theKeyword)
                                        .or()
                                        .like("sjim.deliverer_name", theKeyword)
                                        .or()
                                        .like("stp.name", theKeyword)
                                        .or()
                                        .like("stp.short_name", theKeyword)
                                        .or()
                                        .like("stp.address", theKeyword)
                                        .or()
                                        .like("stp.coverage_area", theKeyword)
                                        .or()
                                        .like("stp.province_contacts", theKeyword)
                                        .or()
                                        .like("stp.city_contacts", theKeyword)
                                        .or()
                                        .like("stp.district_contacts", theKeyword)
                        )
                        .eq(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getJobId()), "sjim.job_id", sewageJobIdentificationMaterial.getJobId())
                        .eq(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getJobItemId()), "sjim.job_item_id", sewageJobIdentificationMaterial.getJobItemId())
                        .eq(sewageJobIdentificationMaterial.getStatus() != null, "sjim.status", sewageJobIdentificationMaterial.getStatus())
                        .in(statuses != null && statuses.size() > 0, "sjim.status", statuses)
                        .and(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getImNo()) ||
                                        StrUtil.isNotBlank(sewageJobIdentificationMaterial.getSample1No()) ||
                                        StrUtil.isNotBlank(sewageJobIdentificationMaterial.getSample2No()),
                                (wrapper) -> wrapper
                                        .eq(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getImNo()), "sjim.im_no", sewageJobIdentificationMaterial.getImNo())
                                        .or()
                                        .eq(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getSample1No()), "sjim.sample1_no", sewageJobIdentificationMaterial.getSample1No())
                                        .or()
                                        .eq(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getSample2No()), "sjim.sample2_no", sewageJobIdentificationMaterial.getSample2No())
                        )
                        .and(!dlpUser.isStaff(), (wrapper) -> wrapper                // 如果不是分中心员工
                                .eq("stp.province_code", userOrg.getAreaCode())    // 污水处理厂的省级地区码与用户机构所在地区一致
                                .or()
                                .eq("stp.city_code", userOrg.getAreaCode())    // 污水处理厂的市级地区码与用户机构所在地区一致
                                .or()
                                .eq("stp.district_code", userOrg.getAreaCode())    // 污水处理厂的区级地区码与用户机构所在地区一致
                        )
                        .orderByAsc("sample_name")
        );

        return R.ok(thePage);
    }


    //污水填报-送检-签收各状态查询接口
    @GetMapping("/getSewageMaterialList")
    @PreAuthorize("@pms.hasPermission('SewageJobIdentificationMaterialList')")
    public R getSewageMaterialList(Page page, String keyword,
                                   SewageJobIdentificationMaterial sewageJobIdentificationMaterial,
                                   @RequestParam(value = "statuses", required = false) List<Integer> statuses,
                                   HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
        SysOrg userOrg;
        R r = remoteOrgService.getById(dlpUser.getOrgId());
        if (r.getCode() == CommonConstants.SUCCESS) {
            userOrg = (SysOrg) r.getData();
        } else {
            return R.failed(String.format("没有找到 orgId 为 %s 的机构, 请确认用户所属机构的正确性!", dlpUser.getOrgId()));
        }
        if (StrUtil.isNotBlank(keyword)) {
            keyword = keyword.trim();
        }
        final String theKeyword = keyword;
        QueryWrapper<SewageJobIdentificationMaterial> queryWrapper = Wrappers.<SewageJobIdentificationMaterial>query()
                .first("true")
                .and(StrUtil.isNotBlank(theKeyword), (wrapper) -> wrapper
                        .eq("sjim.im_no", theKeyword)
                        .or()
                        .eq("sjim.sample1_no", theKeyword)
                        .or()
                        .eq("sjim.sample2_no", theKeyword)
                        .or()
                        .like("sjim.sample_name", theKeyword)
                        .or()
                        .like("sjim.collector", theKeyword)
                        .or()
                        .like("sjim.collector_group", theKeyword)
                        .or()
                        .like("sjim.deliverer_name", theKeyword)
                        .or()
                        .like("stp.name", theKeyword)
                        .or()
                        .like("stp.short_name", theKeyword)
                        .or()
                        .like("stp.address", theKeyword)
                        .or()
                        .like("stp.coverage_area", theKeyword)
                        .or()
                        .like("stp.province_contacts", theKeyword)
                        .or()
                        .like("stp.city_contacts", theKeyword)
                        .or()
                        .like("stp.district_contacts", theKeyword))
                .eq(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getJobId()), "sjim.job_id", sewageJobIdentificationMaterial.getJobId())
                .eq(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getJobItemId()), "sjim.job_item_id", sewageJobIdentificationMaterial.getJobItemId())
                .in(statuses != null && statuses.size() > 0, "sjim.status", statuses)
                .and(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getImNo()) ||
                                StrUtil.isNotBlank(sewageJobIdentificationMaterial.getSample1No()) ||
                                StrUtil.isNotBlank(sewageJobIdentificationMaterial.getSample2No()),
                        (wrapper) -> wrapper
                                .eq(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getImNo()), "sjim.im_no", sewageJobIdentificationMaterial.getImNo())
                                .or()
                                .eq(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getSample1No()), "sjim.sample1_no", sewageJobIdentificationMaterial.getSample1No())
                                .or()
                                .eq(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getSample2No()), "sjim.sample2_no", sewageJobIdentificationMaterial.getSample2No())
                )
                .and(!dlpUser.isStaff(), (wrapper) -> wrapper                // 如果不是分中心员工
                        .eq("stp.province_code", userOrg.getAreaCode())    // 污水处理厂的省级地区码与用户机构所在地区一致
                        .or()
                        .eq("stp.city_code", userOrg.getAreaCode())    // 污水处理厂的市级地区码与用户机构所在地区一致
                        .or()
                        .eq("stp.district_code", userOrg.getAreaCode())    // 污水处理厂的区级地区码与用户机构所在地区一致
                )
                .orderByAsc("sample_name");
        IPage<SewageJobIdentificationMaterialVO> thePage =
                sewageJobIdentificationMaterialService.getSewageJobIdentificationMaterialVOPage(page, queryWrapper);
        return R.ok(thePage);
    }

    /**
     * 新增污水任务的检材信息
     *
     * @param sewageJobIdentificationMaterial 污水任务的检材信息
     * @return R
     */
    @ApiOperation(value = "新增污水任务的检材信息", notes = "新增污水任务的检材信息")
    @SysLog("新增污水任务的检材信息")
    @PostMapping
    @PreAuthorize("@pms.hasPermission('SewageJobIdentificationMaterialCreate')")
    public R postAddObject(@RequestBody SewageJobIdentificationMaterial sewageJobIdentificationMaterial, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();


        final SysOrg userOrg;    // 定义成 final才可以被后面的 Lambda 表达式使用
        R r = remoteOrgService.getById(dlpUser.getOrgId());
        if (r.getCode() == CommonConstants.SUCCESS) {
            userOrg = (SysOrg) r.getData();
        } else {
            return R.failed(String.format("没有找到 orgId 为 %s 的机构, 请确认用户所属机构的正确性!", dlpUser.getOrgId()));
        }
        int userLevel = 99;
        if (dlpUser.isStaff()) {
            // 分中心
            userLevel = 0;
        } else if (StrUtil.endWith(userOrg.getAreaCode(), "0000")) {
            // 省级
            userLevel = 1;
        } else if (StrUtil.endWith(userOrg.getAreaCode(), "00")) {
            // 市级
            userLevel = 2;
        } else {
            // 区级
            userLevel = 3;
        }

        if (StrUtil.isBlank(sewageJobIdentificationMaterial.getJobId())) {
            return R.failed(String.format("必须包含检材所属的任务, jobId 不能为空"));
        }

        SewageJobItem sji = null;
        if (StrUtil.isBlank(sewageJobIdentificationMaterial.getJobItemId())) {
            return R.failed(String.format("必须包含检材所属的任务清单项, getJobItemId 不能为空"));
        } else {
            //
            sji = sewageJobItemService.getById(sewageJobIdentificationMaterial.getJobItemId());
            if (sji == null) {
                return R.failed(String.format("没有找到对应的任务清单项 %s", sewageJobIdentificationMaterial.getJobItemId()));
            }
        }

        sewageJobIdentificationMaterial.setDeliverOrgId(dlpUser.getOrgId());
        sewageJobIdentificationMaterial.setId(IdWorker.get32UUID().toUpperCase());
        sewageJobIdentificationMaterial.setImNo(sewageJobIdentificationMaterialService.getNewCode(sewageJobIdentificationMaterial.getJobId(), sewageJobIdentificationMaterial.getJobItemId()));
        sewageJobIdentificationMaterial.setStatus(0);
        // 如果之前没有生成过样本编号, 现在生成
        if (StrUtil.isBlank(sewageJobIdentificationMaterial.getSample1No())) {
            sewageJobIdentificationMaterial.setSample1No(sewageJobService.getNewSampleNo(sewageJobIdentificationMaterial.getImNo(), 1));
        }
        if (StrUtil.isBlank(sewageJobIdentificationMaterial.getSample2No())) {
            sewageJobIdentificationMaterial.setSample2No(sewageJobService.getNewSampleNo(sewageJobIdentificationMaterial.getImNo(), 2));
        }


        //-- 更新送检员表信息 ----------------
        if (StrUtil.isNotBlank(sewageJobIdentificationMaterial.getDelivererName())) {

            Deliverer deliverer = remoteDeliverService.getByName(sewageJobIdentificationMaterial.getDelivererName()).getData();
            if (deliverer == null) {
                deliverer = new Deliverer();
            }
            deliverer.setOwnerUserId(dlpUser.getId());
            deliverer.setName(sewageJobIdentificationMaterial.getDelivererName());
            deliverer.setPosition(sewageJobIdentificationMaterial.getDelivererPosition());
            deliverer.setCert(sewageJobIdentificationMaterial.getDelivererCert());
            deliverer.setIdnum(sewageJobIdentificationMaterial.getDelivererId());
            deliverer.setPhone(sewageJobIdentificationMaterial.getDelivererPhone());
            remoteDeliverService.renew(deliverer);
        }
        //---------------------------------

        List<UpdateInfo> pis = sewageJobIdentificationMaterial.getUpdateLog();
        if (pis == null) {
            pis = new ArrayList<UpdateInfo>();
        }
        UpdateInfo pi = new UpdateInfo(dlpUser.getId(),
                dlpUser.getName(),
                dlpUser.getOrgId(),
                dlpUser.getOrgName(),
                "创建",
                1);
        pis.add(pi);
        sewageJobIdentificationMaterial.setUpdateLog(pis);

        if (sewageJobIdentificationMaterialService.save(sewageJobIdentificationMaterial)) {
            return R.ok(sewageJobIdentificationMaterial, "新增检材成功");
        } else {
            return R.failed(sewageJobIdentificationMaterial, "新增检材失败");
        }
    }

//
// 	/**
// 	 * 批量新增污水任务的检材信息
// 	 *
// 	 * @param sjimList 污水任务的检材信息
// 	 *
// 	 * @return R
// 	 */
// 	@ApiOperation(value = "批量新增污水任务的检材信息", notes = "批量新增污水任务的检材信息")
// 	@SysLog("批量新增污水任务的检材信息")
// 	@PostMapping("/excel")
// 	@PreAuthorize("@pms.hasPermission('SewageJobIdentificationMaterialCreate')")
// 	public R postAddObjectBatch(@RequestBody List<SewageJobIdentificationMaterial> sjimList, HttpServletRequest theHttpServletRequest) {
// 		Principal principal = theHttpServletRequest.getUserPrincipal();
// 		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
//
//
// 		final SysOrg userOrg;    // 定义成 final才可以被后面的 Lambda 表达式使用
// 		R r = remoteOrgService.getById(dlpUser.getOrgId());
// 		if (r.getCode() == CommonConstants.SUCCESS) {
// 			userOrg = (SysOrg) r.getData();
// 		}
// 		else {
// 			return R.failed(String.format("没有找到 orgId 为 %s 的机构, 请确认用户所属机构的正确性!", dlpUser.getOrgId()));
// 		}
// 		int userLevel = 99;
// 		if (dlpUser.isStaff()) {
// 			// 分中心
// 			userLevel = 0;
// 		}
// 		else if (StrUtil.endWith(userOrg.getAreaCode(), "0000")) {
// 			// 省级
// 			userLevel = 1;
// 		}
// 		else if (StrUtil.endWith(userOrg.getAreaCode(), "00")) {
// 			// 市级
// 			userLevel = 2;
// 		}
// 		else {
// 			// 区级
// 			userLevel = 3;
// 		}
// /*
// 		if (!StrUtil.isNotBlank(sewageJobIdentificationMaterial.getJobId())) {
// 			return R.failed(String.format("必须包含检材所属的任务, jobId 不能为空"));
// 		}
// 		if (!StrUtil.isNotBlank(sewageJobIdentificationMaterial.getJobItemId())) {
// 			return R.failed(String.format("必须包含检材所属的任务清单项, getJobItemId 不能为空"));
// 		}
//
// 		sewageJobIdentificationMaterial.setDeliverOrgId(dlpUser.getOrgId());
// 		sewageJobIdentificationMaterial.setId(IdWorker.get32UUID().toUpperCase());
//
// 		List<UpdateInfo> pis = sewageJobIdentificationMaterial.getUpdateLog();
// 		if (pis == null) {
// 			pis = new ArrayList<UpdateInfo>();
// 		}
// 		UpdateInfo pi = new UpdateInfo(dlpUser.getId(),
// 				dlpUser.getName(),
// 				dlpUser.getOrgId(),
// 				dlpUser.getOrgName(),
// 				"创建",
// 				1);
// 		pis.add(pi);
// 		sewageJobIdentificationMaterial.setUpdateLog(pis);
//
// 		if (sewageJobIdentificationMaterialService.save(sewageJobIdentificationMaterial)) {
// 			return R.ok(sewageJobIdentificationMaterial, "新增检材成功");
// 		}
// 		else {
// 			return R.failed(sewageJobIdentificationMaterial, "新增检材失败");
// 		}
//
//  */
// 		return R.ok(sjimList);
// 	}


    /**
     * 修改污水任务的检材信息
     *
     * @param sewageJobIdentificationMaterial 污水任务的检材信息
     * @return R
     */
    @ApiOperation(value = "修改污水任务的检材信息", notes = "修改污水任务的检材信息")
    @SysLog("修改污水任务的检材信息")
    @PutMapping
    @PreAuthorize("@pms.hasPermission('SewageJobIdentificationMaterialEdit')")
    public R putUpdateById(@RequestBody SewageJobIdentificationMaterial sewageJobIdentificationMaterial, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        final SysOrg userOrg;    // 定义成 final才可以被后面的 Lambda 表达式使用
        R r = remoteOrgService.getById(dlpUser.getOrgId());
        if (r.getCode() == CommonConstants.SUCCESS) {
            userOrg = (SysOrg) r.getData();
        } else {
            return R.failed(String.format("没有找到 orgId 为 %s 的机构, 请确认用户所属机构的正确性!", dlpUser.getOrgId()));
        }
        int userLevel = 99;
        if (dlpUser.isStaff()) {
            // 分中心
            userLevel = 0;
        } else if (StrUtil.endWith(userOrg.getAreaCode(), "0000")) {
            // 省级
            userLevel = 1;
        } else if (StrUtil.endWith(userOrg.getAreaCode(), "00")) {
            // 市级
            userLevel = 2;
        } else {
            // 区级
            userLevel = 3;
        }


        SewageJobIdentificationMaterial oldim = sewageJobIdentificationMaterialService.getById(sewageJobIdentificationMaterial.getId());
        if (oldim == null) {
            return R.failed(String.format("没有找到 id 为 %s 的检材", sewageJobIdentificationMaterial.getId()));
        }
        // if (!StrUtil.equals(oldim.getDeliverOrgId(), dlpUser.getOrgId())) {
        // 	return R.failed(String.format("你没有权限修改这个检材, id=%s", sewageJobIdentificationMaterial.getId()));
        // }


        SewageJobItem sji = null;
        SewageTreatmentPlant stp = null;
        if (StrUtil.isBlank(oldim.getJobItemId())) {
            return R.failed(String.format("检材对应的原任务清单项, getJobItemId 不能为空, 这是个无效的检材"));
        } else {
            //
            sji = sewageJobItemService.getById(oldim.getJobItemId());
            if (sji == null) {
                return R.failed(String.format("没有找到检材对应的原任务清单项 %s, 这是个无效的检材", oldim.getJobItemId()));
            }
            stp = sewageTreatmentPlantService.getById(sji.getPlantId());
            if (stp == null) {
                return R.failed(String.format("没有找到检材对应的原任务清单项 %s 对应的污水处理厂 %s, 这是个无效的任务清单项", oldim.getJobItemId(), sji.getPlantId()));
            }
        }

        if (userLevel == 3) {
            // 区县一级的机构码必须完全一致
            if (!StrUtil.equals(stp.getDistrictCode(), userOrg.getAreaCode())) {
                return R.failed(String.format("你没有权限修改这个检材, id=%s", sewageJobIdentificationMaterial.getId()));
            }
        } else if (userLevel == 2) {
            // 市一级要确认是送检机构的上级
            if (!StrUtil.equalsIgnoreCase(stp.getCityCode(), userOrg.getAreaCode())) {
                return R.failed(String.format("你没有权限修改这个检材, id=%s", sewageJobIdentificationMaterial.getId()));
            }
        } else if (userLevel == 1) {
            // 省一级要确认是送检机构的上级的上级
            if (!StrUtil.equalsIgnoreCase(stp.getProvinceCode(), userOrg.getAreaCode())) {
                return R.failed(String.format("你没有权限修改这个检材, id=%s", sewageJobIdentificationMaterial.getId()));
            }
        }


        if (!StrUtil.equals(oldim.getJobId(), sewageJobIdentificationMaterial.getJobId())) {
            return R.failed(String.format("不能修改检材所属的任务, jobId=%s", sewageJobIdentificationMaterial.getJobId()));
        }
        if (!StrUtil.equals(oldim.getJobItemId(), sewageJobIdentificationMaterial.getJobItemId())) {
            return R.failed(String.format("不能修改检材所属的任务清单项, jobItemId=%s", sewageJobIdentificationMaterial.getJobItemId()));
        }
        if (!StrUtil.equals(oldim.getDeliverOrgId(), sewageJobIdentificationMaterial.getDeliverOrgId())) {
            return R.failed(String.format("不能修改检材的送检单位"));
        }

        // if ((oldim.getStatus() != null) && (oldim.getStatus().equals(sewageJobIdentificationMaterial.getStatus()))) {
        // 	return R.failed(String.format("不能修改检材的状态"));
        // }

        if (oldim.getStatus() > 1) {
            return R.failed(String.format("这个检材已送检, 不能修改"));
        }

//        if (StrUtil.isBlank(sewageJobIdentificationMaterial.getImNo())) {
//            sewageJobIdentificationMaterial.setImNo(sewageJobIdentificationMaterialService.getNewCode(sewageJobIdentificationMaterial.getJobId(), sewageJobIdentificationMaterial.getJobItemId()));
//        }
        // 如果之前没有生成过样本编号, 现在生成
//        if (StrUtil.isBlank(sewageJobIdentificationMaterial.getSample1No())) {
//            sewageJobIdentificationMaterial.setSample1No(sewageJobService.getNewSampleNo(sewageJobIdentificationMaterial.getImNo(), 1));
//        }
//        if (StrUtil.isBlank(sewageJobIdentificationMaterial.getSample2No())) {
//            sewageJobIdentificationMaterial.setSample2No(sewageJobService.getNewSampleNo(sewageJobIdentificationMaterial.getImNo(), 2));
//        }


        //-- 更新送检员表信息 ----------------
        if (StrUtil.isNotBlank(sewageJobIdentificationMaterial.getDelivererName())) {

            Deliverer deliverer = remoteDeliverService.getByName(sewageJobIdentificationMaterial.getDelivererName()).getData();
            if (deliverer == null) {
                deliverer = new Deliverer();
            }
            deliverer.setOwnerUserId(dlpUser.getId());
            deliverer.setName(sewageJobIdentificationMaterial.getDelivererName());
            deliverer.setPosition(sewageJobIdentificationMaterial.getDelivererPosition());
            deliverer.setCert(sewageJobIdentificationMaterial.getDelivererCert());
            deliverer.setIdnum(sewageJobIdentificationMaterial.getDelivererId());
            deliverer.setPhone(sewageJobIdentificationMaterial.getDelivererPhone());
            remoteDeliverService.renew(deliverer);
        }
        //---------------------------------

        List<UpdateInfo> pis = sewageJobIdentificationMaterial.getUpdateLog();
        if (pis == null) {
            pis = new ArrayList<UpdateInfo>();
        }
        UpdateInfo pi = new UpdateInfo(dlpUser.getId(),
                dlpUser.getName(),
                dlpUser.getOrgId(),
                dlpUser.getOrgName(),
                "修改",
                1);
        pis.add(pi);
        sewageJobIdentificationMaterial.setUpdateLog(pis);

//        UpdateWrapper<SewageJobIdentificationMaterial> updateWrapper = new UpdateWrapper<SewageJobIdentificationMaterial>();
//        updateWrapper.eq("id", sewageJobIdentificationMaterial.getId());        // 查询条件是 id 相等
//
//        updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getImNo()), "im_no", sewageJobIdentificationMaterial.getImNo());
//        updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getSample1No()), "sample1_no", sewageJobIdentificationMaterial.getSample1No());
//        updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getSample2No()), "sample2_no", sewageJobIdentificationMaterial.getSample2No());
//        updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getSample1BoxId()), "sample1_box_id", sewageJobIdentificationMaterial.getSample1BoxId());
//        updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getSample2BoxId()), "sample2_box_id", sewageJobIdentificationMaterial.getSample2BoxId());
//        updateWrapper.set(sewageJobIdentificationMaterial.getSampleType() != null, "sample_type", sewageJobIdentificationMaterial.getSampleType());
//        updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getSampleName()), "sample_name", sewageJobIdentificationMaterial.getSampleName());
//        updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getCollectPlaceAreaCode()), "collect_place_area_code", sewageJobIdentificationMaterial.getCollectPlaceAreaCode());
//        updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getCollectPlace()), "collect_place", sewageJobIdentificationMaterial.getCollectPlace());
//        updateWrapper.set(sewageJobIdentificationMaterial.getDailyFlow() != null, "daily_flow", sewageJobIdentificationMaterial.getDailyFlow());
//        updateWrapper.set(sewageJobIdentificationMaterial.getWaterInletCount() != null, "water_inlet_count", sewageJobIdentificationMaterial.getWaterInletCount());
//        updateWrapper.set(sewageJobIdentificationMaterial.getWaterInletNumber() != null, "water_inlet_number", sewageJobIdentificationMaterial.getWaterInletNumber());
//        updateWrapper.set(sewageJobIdentificationMaterial.getDomesticSewageProportion() != null, "domestic_sewage_proportion", sewageJobIdentificationMaterial.getDomesticSewageProportion());
//        updateWrapper.set(sewageJobIdentificationMaterial.getServicePopulation() != null, "service_population", sewageJobIdentificationMaterial.getServicePopulation());
//        updateWrapper.set(sewageJobIdentificationMaterial.getSamplingDepth() != null, "sampling_depth", sewageJobIdentificationMaterial.getSamplingDepth());
//        updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getSampleProperties()), "sample_properties", sewageJobIdentificationMaterial.getSampleProperties());
//        updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getSamplingMethod()), "sampling_method", sewageJobIdentificationMaterial.getSamplingMethod());
//        updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getSamplingMode()), "sampling_mode", sewageJobIdentificationMaterial.getSamplingMode());
//        updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getStorageMethod()), "storage_method", sewageJobIdentificationMaterial.getStorageMethod());
//        updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getCollector()), "collector", sewageJobIdentificationMaterial.getCollector());
//        updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getCollectorGroup()), "collector_group", sewageJobIdentificationMaterial.getCollectorGroup());
//        updateWrapper.set(sewageJobIdentificationMaterial.getCollectTime() != null, "collect_time", sewageJobIdentificationMaterial.getCollectTime());
//        updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getDeliverMethod()), "deliver_method", sewageJobIdentificationMaterial.getDeliverMethod());
//        updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getDelivererName()), "deliverer_name", sewageJobIdentificationMaterial.getDelivererName());
//        updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getDelivererPosition()), "deliverer_position", sewageJobIdentificationMaterial.getDelivererPosition());
//        updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getDelivererCert()), "deliverer_cert", sewageJobIdentificationMaterial.getDelivererCert());
//        updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getDelivererId()), "deliverer_id", sewageJobIdentificationMaterial.getDelivererId());
//        updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getDelivererPhone()), "deliverer_phone", sewageJobIdentificationMaterial.getDelivererPhone());
//        updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getDeliverOrgId()), "deliver_org_id", sewageJobIdentificationMaterial.getDeliverOrgId());
//        updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getComments()), "comments", sewageJobIdentificationMaterial.getComments());
//
//        updateWrapper.set("update_log", sewageJobIdentificationMaterial.getUpdateLog(), "javaType=String,jdbcType=VARCHAR,typeHandler=com.baomidou.mybatisplus.extension.handlers.FastjsonTypeHandler");
//        if (sewageJobIdentificationMaterialService.update(null, updateWrapper)) {
//            return R.ok(sewageJobIdentificationMaterial, "保存检材信息成功");
//        } else {
//            return R.failed(sewageJobIdentificationMaterial, "保存检材信息失败");
//        }

        // if (sewageJobIdentificationMaterialService.updateById(sewageJobIdentificationMaterial)) {
        // 	return R.ok(sewageJobIdentificationMaterial, "保存检材信息成功");
        // }
        // else {
        // 	return R.failed(sewageJobIdentificationMaterial, "保存检材信息失败");
        // }

        sewageJobIdentificationMaterial.setStatus(1);//设置为 已经填报

        sewageJobIdentificationMaterial.setInformantId(dlpUser.getId());
        if (sewageJobIdentificationMaterialService.updateById(sewageJobIdentificationMaterial)) {
            return R.ok(sewageJobIdentificationMaterial, "保存检材信息成功");
        } else {
            return R.failed(sewageJobIdentificationMaterial, "保存检材信息失败");
        }
    }

    /**
     * 通过id删除污水任务的检材信息
     *
     * @param id id
     * @return R
     */
    @ApiOperation(value = "通过id删除污水任务的检材信息", notes = "通过id删除污水任务的检材信息")
    @SysLog("通过id删除污水任务的检材信息")
    @DeleteMapping("/{id}")
    @PreAuthorize("@pms.hasPermission('SewageJobIdentificationMaterialDelete')")
    public R deleteById(@PathVariable String id, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        final SysOrg userOrg;    // 定义成 final才可以被后面的 Lambda 表达式使用
        R r = remoteOrgService.getById(dlpUser.getOrgId());
        if (r.getCode() == CommonConstants.SUCCESS) {
            userOrg = (SysOrg) r.getData();
        } else {
            return R.failed(String.format("没有找到 orgId 为 %s 的机构, 请确认用户所属机构的正确性!", dlpUser.getOrgId()));
        }
        int userLevel = 99;
        if (dlpUser.isStaff()) {
            // 分中心
            userLevel = 0;
        } else if (StrUtil.endWith(userOrg.getAreaCode(), "0000")) {
            // 省级
            userLevel = 1;
        } else if (StrUtil.endWith(userOrg.getAreaCode(), "00")) {
            // 市级
            userLevel = 2;
        } else {
            // 区级
            userLevel = 3;
        }

        SewageJobIdentificationMaterial oldim = sewageJobIdentificationMaterialService.getById(id);
        if (oldim == null) {
            return R.failed(String.format("没有找到 id 为 %s 的检材", id));
        }
        if (!StrUtil.equals(oldim.getDeliverOrgId(), dlpUser.getOrgId())) {
            return R.failed(String.format("你没有权限删除这个检材, id=%s", id));
        }

        if (oldim.getStatus() != 0) {
            return R.failed(String.format("这个检材已送检, 不能删除"));
        }

        if (sewageJobIdentificationMaterialService.removeById(id)) {
            return R.ok(oldim, "检材删除成功");
        } else {
            return R.failed(oldim, "检材删除失败");
        }
    }


    /**
     * 检材送检
     */
    @ApiOperation(value = "检材送检", notes = "检材送检, 同时也保存检材的其他属性")
    @SysLog("检材送检")
    @PutMapping("/deliver")
    @PreAuthorize("@pms.hasPermission('SewageJobIdentificationMaterialDeliver')")
    public R putDeliver(@RequestBody SewageJobIdentificationMaterialDTO
                                sewageJobIdentificationMaterialDTO, HttpServletRequest theHttpServletRequest) {
        if (sewageJobIdentificationMaterialDTO.getIds().size() <= 0) {
            return R.failed("检材数量不你能为0");
        }
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
        SysOrg userOrg;
        R r = remoteOrgService.getById(dlpUser.getOrgId());
        if (r.getCode() == CommonConstants.SUCCESS) {
            userOrg = (SysOrg) r.getData();
        } else {
            return R.failed(String.format("没有找到 orgId 为 %s 的机构, 请确认用户所属机构的正确性!", dlpUser.getOrgId()));
        }
        //int userLevel= QRCodeUtils.getUserLvByAreaCode(userOrg.getAreaCode());
        //调用service 来处理这个业务
        //List<String> materialIdList=sewageJobIdentificationMaterialDTO.getIds();
        return R.ok(sewageJobIdentificationMaterialService.batchSendMaterial(sewageJobIdentificationMaterialDTO), "送检成功");
    }
    //增加代码功能和易读性，所以注释下面的代码，由上面代码来完成功能
//    public R putDeliver(@RequestBody SewageJobIdentificationMaterialDTO sewageJobIdentificationMaterialDTO,
//                        HttpServletRequest theHttpServletRequest) {
//        Principal principal = theHttpServletRequest.getUserPrincipal();
//        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
//
//        final SysOrg userOrg;    // 定义成 final才可以被后面的 Lambda 表达式使用
//        R r = remoteOrgService.getById(dlpUser.getOrgId());
//        if (r.getCode() == CommonConstants.SUCCESS) {
//            userOrg = (SysOrg) r.getData();
//        } else {
//            return R.failed(String.format("没有找到 orgId 为 %s 的机构, 请确认用户所属机构的正确性!", dlpUser.getOrgId()));
//        }
//        int userLevel = 99;
//        if (dlpUser.isStaff()) {
//            // 分中心
//            userLevel = 0;
//        } else if (StrUtil.endWith(userOrg.getAreaCode(), "0000")) {
//            // 省级
//            userLevel = 1;
//        } else if (StrUtil.endWith(userOrg.getAreaCode(), "00")) {
//            // 市级
//            userLevel = 2;
//        } else {
//            // 区级
//            userLevel = 3;
//        }
//        String ids = sewageJobIdentificationMaterialDTO.getIds();
//        String[] splitss=new String[]{ids};
//        String[] split = ids.split(",");
//        List<SewageJobIdentificationMaterial> sewageJobIdentificationMaterialList = sewageJobIdentificationMaterialService.getSewageJobIdentificationMaterialList(split);
//
//        for (SewageJobIdentificationMaterial sewageJobIdentificationMaterial : sewageJobIdentificationMaterialList) {
//
//
//            if (sewageJobIdentificationMaterial == null) {
//                return R.failed(String.format("没有找到 id 为 %s 的检材", sewageJobIdentificationMaterial.getId()));
//            }
//
//            SewageJobItem sji = null;
//            SewageTreatmentPlant stp = null;
//            if (StrUtil.isBlank(sewageJobIdentificationMaterial.getJobItemId())) {
//                return R.failed(String.format("检材对应的原任务清单项, getJobItemId 不能为空, 这是个无效的检材"));
//            } else {
//                //
//                sji = sewageJobItemService.getById(sewageJobIdentificationMaterial.getJobItemId());
//                if (sji == null) {
//                    return R.failed(String.format("没有找到检材对应的原任务清单项 %s, 这是个无效的检材", sewageJobIdentificationMaterial.getJobItemId()));
//                }
//                stp = sewageTreatmentPlantService.getById(sji.getPlantId());
//                if (stp == null) {
//                    return R.failed(String.format("没有找到检材对应的原任务清单项 %s 对应的污水处理厂 %s, 这是个无效的任务清单项", sewageJobIdentificationMaterial.getJobItemId(), sji.getPlantId()));
//                }
//            }
//
//            if (userLevel == 3) {
//                // 区县一级的机构码必须完全一致
//                if (!StrUtil.equals(sewageJobIdentificationMaterial.getDeliverOrgId(), dlpUser.getOrgId())) {
//                    return R.failed(String.format("你没有权限修改这个检材, id=%s", sewageJobIdentificationMaterial.getId()));
//                }
//            } else if (userLevel == 2) {
//                // 市一级要确认是送检机构的上级
//                if (!StrUtil.equalsIgnoreCase(stp.getCityCode(), userOrg.getAreaCode())) {
//                    return R.failed(String.format("你没有权限修改这个检材, id=%s", sewageJobIdentificationMaterial.getId()));
//                }
//            } else if (userLevel == 1) {
//                // 省一级要确认是送检机构的上级的上级
//                if (!StrUtil.equalsIgnoreCase(stp.getProvinceCode(), userOrg.getAreaCode())) {
//                    return R.failed(String.format("你没有权限修改这个检材, id=%s", sewageJobIdentificationMaterial.getId()));
//                }
//            }
//
//            if (!StrUtil.equals(sewageJobIdentificationMaterial.getJobId(), sewageJobIdentificationMaterial.getJobId())) {
//                return R.failed(String.format("不能修改检材所属的任务, jobId=%s", sewageJobIdentificationMaterial.getJobId()));
//            }
//            if (!StrUtil.equals(sewageJobIdentificationMaterial.getJobItemId(), sewageJobIdentificationMaterial.getJobItemId())) {
//                return R.failed(String.format("不能修改检材所属的任务清单项, jobItemId=%s", sewageJobIdentificationMaterial.getJobItemId()));
//            }
//            if (!StrUtil.equals(sewageJobIdentificationMaterial.getDeliverOrgId(), sewageJobIdentificationMaterial.getDeliverOrgId())) {
//                return R.failed(String.format("不能修改检材的送检单位"));
//            }
//            // if (! oldim.getStatus().equals(sewageJobIdentificationMaterial.getStatus())) {
//            // 	return R.failed(String.format("不能修改检材的状态"));
//            // }
//
//            if (sewageJobIdentificationMaterial.getStatus() != 0) {
//                return R.failed(String.format("这个检材已送检, 不能修改"));
//            }
//
//            if (StrUtil.isBlank(sewageJobIdentificationMaterial.getImNo())) {
//                sewageJobIdentificationMaterial.setImNo(sewageJobIdentificationMaterialService.getNewCode(sewageJobIdentificationMaterial.getJobId(), sewageJobIdentificationMaterial.getJobItemId()));
//            }
//            // 如果之前没有生成过样本编号, 现在生成
//            if (StrUtil.isBlank(sewageJobIdentificationMaterial.getSample1No())) {
//                sewageJobIdentificationMaterial.setSample1No(sewageJobService.getNewSampleNo(sewageJobIdentificationMaterial.getImNo(), 1));
//            }
//            if (StrUtil.isBlank(sewageJobIdentificationMaterial.getSample2No())) {
//                sewageJobIdentificationMaterial.setSample2No(sewageJobService.getNewSampleNo(sewageJobIdentificationMaterial.getImNo(), 2));
//            }
//
//            //-- 更新送检员表信息 ----------------
//            if (StrUtil.isNotBlank(sewageJobIdentificationMaterial.getDelivererName())) {
//                Deliverer deliverer = remoteDeliverService.getByName(sewageJobIdentificationMaterial.getDelivererName()).getData();
//                if (deliverer == null) {
//                    deliverer = new Deliverer();
//                }
//                deliverer.setOwnerUserId(dlpUser.getId());
//                deliverer.setName(sewageJobIdentificationMaterial.getDelivererName());
//                deliverer.setPosition(sewageJobIdentificationMaterial.getDelivererPosition());
//                deliverer.setCert(sewageJobIdentificationMaterial.getDelivererCert());
//                deliverer.setIdnum(sewageJobIdentificationMaterial.getDelivererId());
//                deliverer.setPhone(sewageJobIdentificationMaterial.getDelivererPhone());
//                remoteDeliverService.renew(deliverer);
//            }
//            //---------------------------------
//
//
//            List<UpdateInfo> pis = sewageJobIdentificationMaterial.getUpdateLog();
//            if (pis == null) {
//                pis = new ArrayList<UpdateInfo>();
//            }
//            UpdateInfo pi = new UpdateInfo(dlpUser.getId(),
//                    dlpUser.getName(),
//                    dlpUser.getOrgId(),
//                    dlpUser.getOrgName(),
//                    "送检",
//                    1);
//
//            pis.add(pi);
//            sewageJobIdentificationMaterial.setUpdateLog(pis);
//
//
//            UpdateWrapper<SewageJobIdentificationMaterial> updateWrapper = new UpdateWrapper<SewageJobIdentificationMaterial>();
//            updateWrapper.eq("id", sewageJobIdentificationMaterial.getId());        // 查询条件是 id 相等
//
//            updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getImNo()), "im_no", sewageJobIdentificationMaterial.getImNo());
//            updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getSample1No()), "sample1_no", sewageJobIdentificationMaterial.getSample1No());
//            updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getSample2No()), "sample2_no", sewageJobIdentificationMaterial.getSample2No());
//            updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getSample1BoxId()), "sample1_box_id", sewageJobIdentificationMaterial.getSample1BoxId());
//            updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getSample2BoxId()), "sample2_box_id", sewageJobIdentificationMaterial.getSample2BoxId());
//            updateWrapper.set(sewageJobIdentificationMaterial.getSampleType() != null, "sample_type", sewageJobIdentificationMaterial.getSampleType());
//            updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getSampleName()), "sample_name", sewageJobIdentificationMaterial.getSampleName());
//            updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getCollectPlaceAreaCode()), "collect_place_area_code", sewageJobIdentificationMaterial.getCollectPlaceAreaCode());
//            updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getCollectPlace()), "collect_place", sewageJobIdentificationMaterial.getCollectPlace());
//            updateWrapper.set(sewageJobIdentificationMaterial.getDailyFlow() != null, "daily_flow", sewageJobIdentificationMaterial.getDailyFlow());
//            updateWrapper.set(sewageJobIdentificationMaterial.getWaterInletCount() != null, "water_inlet_count", sewageJobIdentificationMaterial.getWaterInletCount());
//            updateWrapper.set(sewageJobIdentificationMaterial.getWaterInletNumber() != null, "water_inlet_number", sewageJobIdentificationMaterial.getWaterInletNumber());
//            updateWrapper.set(sewageJobIdentificationMaterial.getDomesticSewageProportion() != null, "domestic_sewage_proportion", sewageJobIdentificationMaterial.getDomesticSewageProportion());
//            updateWrapper.set(sewageJobIdentificationMaterial.getServicePopulation() != null, "service_population", sewageJobIdentificationMaterial.getServicePopulation());
//            updateWrapper.set(sewageJobIdentificationMaterial.getSamplingDepth() != null, "sampling_depth", sewageJobIdentificationMaterial.getSamplingDepth());
//            updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getSampleProperties()), "sample_properties", sewageJobIdentificationMaterial.getSampleProperties());
//            updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getSamplingMethod()), "sampling_method", sewageJobIdentificationMaterial.getSamplingMethod());
//            updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getSamplingMode()), "sampling_mode", sewageJobIdentificationMaterial.getSamplingMode());
//            updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getStorageMethod()), "storage_method", sewageJobIdentificationMaterial.getStorageMethod());
//            updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getCollector()), "collector", sewageJobIdentificationMaterial.getCollector());
//            updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getCollectorGroup()), "collector_group", sewageJobIdentificationMaterial.getCollectorGroup());
//            updateWrapper.set(sewageJobIdentificationMaterial.getCollectTime() != null, "collect_time", sewageJobIdentificationMaterial.getCollectTime());
//            updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getDeliverMethod()), "deliver_method", sewageJobIdentificationMaterial.getDeliverMethod());
//            updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getDelivererName()), "deliverer_name", sewageJobIdentificationMaterial.getDelivererName());
//            updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getDelivererPosition()), "deliverer_position", sewageJobIdentificationMaterial.getDelivererPosition());
//            updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getDelivererCert()), "deliverer_cert", sewageJobIdentificationMaterial.getDelivererCert());
//            updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getDelivererId()), "deliverer_id", sewageJobIdentificationMaterial.getDelivererId());
//            updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getDelivererPhone()), "deliverer_phone", sewageJobIdentificationMaterial.getDelivererPhone());
//            updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getDeliverOrgId()), "deliver_org_id", sewageJobIdentificationMaterial.getDeliverOrgId());
//            updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterial.getComments()), "comments", sewageJobIdentificationMaterial.getComments());
//            //录入快递信息
//            updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterialDTO.getExpressNumber()), "express_number", sewageJobIdentificationMaterialDTO.getExpressNumber());
//            updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterialDTO.getExpressCompany()), "express_company", sewageJobIdentificationMaterialDTO.getExpressCompany());
//            updateWrapper.set("status", 1);    // 状态:0=刚创建待送检,1=已送检待受理,2=已受理, -1=受理被拒绝
//            updateWrapper.set("update_log", sewageJobIdentificationMaterial.getUpdateLog(), "javaType=String,jdbcType=VARCHAR,typeHandler=com.baomidou.mybatisplus.extension.handlers.FastjsonTypeHandler");
//
//            if (!(sewageJobIdentificationMaterialService.update(sewageJobIdentificationMaterial, updateWrapper))) {
//                return R.ok(sewageJobIdentificationMaterialService.getById(sewageJobIdentificationMaterial.getId()), "保存检材信息并送检失败");
//            }
//
//        }
//        return R.failed(sewageJobIdentificationMaterialList, "保存检材信息并送检成功");
//    }


    /**
     * 检材受理过程中的保存
     *
     * @param sewageJobIdentificationMaterial 污水任务的检材信息
     * @return R
     */
    @ApiOperation(value = "检材受理过程中的保存", notes = "检材受理过程中的保存")
    @SysLog("检材受理过程中的保存")
    @PutMapping("/accept_save")
    @PreAuthorize("@pms.hasPermission('SewageJobIdentificationMaterialAccept')")
    public R bizAccept_Save(@RequestBody SewageJobIdentificationMaterial
                                    sewageJobIdentificationMaterial, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        final SysOrg userOrg;    // 定义成 final才可以被后面的 Lambda 表达式使用
        R r = remoteOrgService.getById(dlpUser.getOrgId());
        if (r.getCode() == CommonConstants.SUCCESS) {
            userOrg = (SysOrg) r.getData();
        } else {
            return R.failed(String.format("没有找到 orgId 为 %s 的机构, 请确认用户所属机构的正确性!", dlpUser.getOrgId()));
        }

        if (!dlpUser.isStaff()) {
            throw new RuntimeException(String.format("你没有受理检材的权限"));
        }

        SewageJobIdentificationMaterial im = sewageJobIdentificationMaterialService.getById(sewageJobIdentificationMaterial.getId());
        if (im == null) {
            return R.failed(String.format("没有找到 id 为 %s 的检材", sewageJobIdentificationMaterial.getId()));
        }

        if (im.getStatus() != 2) {    // 状态:0=刚创建待送检,1=已送检待受理,2=已受理, -1=受理被拒绝
            return R.failed(String.format("这个检材不在[已送检待受理]状态, 无法受理"));
        }

        // if (StrUtil.isBlank(sewageJobIdentificationMaterial.getAcceptComments())) {
        // 	return R.failed(String.format("必须提供 acceptComments 属性"));
        // }

        //==== 样本判断 =====

        // 受理通过, 修改属性

        SampleBox sb1 = null;
        SampleBox sb2 = null;

        // 如果之前没有生成过样本编号, 现在填写样本重量了, 那么就生成样本编号
        //if ((sewageJobIdentificationMaterial.getSample1Present() != null) && (sewageJobIdentificationMaterial.getSample1Present())) {
        if (StrUtil.isBlank(sewageJobIdentificationMaterial.getSample1No())) {
            sewageJobIdentificationMaterial.setSample1No(sewageJobService.getNewSampleNo(sewageJobIdentificationMaterial.getImNo(), 1));
        }
        if (StrUtil.isNotBlank(sewageJobIdentificationMaterial.getSample1BoxId())) {
            R<SampleBox> sb1R = remoteEntrustmentService.getSampleBoxById(sewageJobIdentificationMaterial.getSample1BoxId());
            if (sb1R.getCode() == CommonConstants.SUCCESS) {
                sb1 = sb1R.getData();
                if (sb1 == null) {
                    throw new RuntimeException(String.format("样本1放到的样本盒子 %s 不存在", sewageJobIdentificationMaterial.getSample1BoxId()));
                }
                if (!StrUtil.equalsIgnoreCase(sewageJobIdentificationMaterial.getStorageMethod(), sb1.getStorageMethod())) {
                    throw new RuntimeException(String.format("检材的存储方法是[%s], 样本1放到的样本盒子的存储方法是[%s], 存储方法不一致", sewageJobIdentificationMaterial.getStorageMethod(), sb1.getStorageMethod()));
                }
            }
        }
        //}
        if ((sewageJobIdentificationMaterial.getSample2Present() != null) && (sewageJobIdentificationMaterial.getSample2Present())) {
            if (StrUtil.isBlank(sewageJobIdentificationMaterial.getSample2No())) {
                sewageJobIdentificationMaterial.setSample2No(sewageJobService.getNewSampleNo(sewageJobIdentificationMaterial.getImNo(), 2));
            }
            if (StrUtil.isNotBlank(sewageJobIdentificationMaterial.getSample2BoxId())) {
                R<SampleBox> sb2R = remoteEntrustmentService.getSampleBoxById(sewageJobIdentificationMaterial.getSample1BoxId());
                if (sb2R.getCode() == CommonConstants.SUCCESS) {
                    sb2 = sb2R.getData();
                    if (sb2 == null) {
                        throw new RuntimeException(String.format("样本2放到的样本盒子 %s 不存在", sewageJobIdentificationMaterial.getSample2BoxId()));
                    }
                    if (!StrUtil.equalsIgnoreCase(sewageJobIdentificationMaterial.getStorageMethod(), sb2.getStorageMethod())) {
                        throw new RuntimeException(String.format("检材的存储方法是[%s], 样本2放到的样本盒子的存储方法是[%s], 存储方法不一致", sewageJobIdentificationMaterial.getStorageMethod(), sb2.getStorageMethod()));
                    }
                }
            }
        }

        // if ((sewageJobIdentificationMaterial.getSample1Present() == null) || (!sewageJobIdentificationMaterial.getSample1Present())) {
        // 	if ((sewageJobIdentificationMaterial.getSample2Present() == null) || (!sewageJobIdentificationMaterial.getSample2Present())) {
        // 		throw new RuntimeException(String.format("这个检材至少要有一份样本"));
        // 	}
        // }

        if ((sb1 != null) && (sb2 != null)) {
            // 两个样本都装到盒子中了
            if (StrUtil.equalsIgnoreCase(sb1.getSampleType(), sb2.getSampleType())) {
                throw new RuntimeException(String.format("两个不同的样本应该装入不同类型的盒子中"));
            }
        }

        UpdateWrapper<SewageJobIdentificationMaterial> updateWrapper = new UpdateWrapper<SewageJobIdentificationMaterial>();
        updateWrapper.eq("id", sewageJobIdentificationMaterial.getId());        // 查询条件是 id 相等

        // updateWrapper.set("sample1_present", sewageJobIdentificationMaterial.getSample1Present());
        updateWrapper.set("sample2_present", sewageJobIdentificationMaterial.getSample2Present());
        updateWrapper.set("sample1_no", sewageJobIdentificationMaterial.getSample1No());
        updateWrapper.set("sample2_no", sewageJobIdentificationMaterial.getSample2No());
        updateWrapper.set("sample1_box_id", sewageJobIdentificationMaterial.getSample1BoxId());
        updateWrapper.set("sample2_box_id", sewageJobIdentificationMaterial.getSample2BoxId());

        if (!sewageJobIdentificationMaterialService.update(null, updateWrapper)) {
            return R.failed(sewageJobIdentificationMaterial, "保存检材受理信息失败");
        } else {
            return R.ok(sewageJobIdentificationMaterial, "保存检材受理信息成功");
        }
    }


    /**
     * 检材受理
     *
     * @param sewageJobIdentificationMaterialDTO 污水任务的检材信息
     * @return R
     */
    @GlobalTransactional
    @ApiOperation(value = "检材受理", notes = "检材受理")
    @SysLog("检材受理")
    @PutMapping("/accept")
    @PreAuthorize("@pms.hasPermission('SewageJobIdentificationMaterialAccept')")
    public R bizAccept_Apply(@RequestBody SewageJobIdentificationMaterialDTO
                                     sewageJobIdentificationMaterialDTO, Integer opCode, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        final SysOrg userOrg = getOrgInfo(dlpUser.getOrgId());

        if (!dlpUser.isStaff()) {
            throw new RuntimeException(String.format("你没有受理检材的权限"));
        }

        List<String> ids = sewageJobIdentificationMaterialDTO.getIds();


        List<SewageJobIdentificationMaterial> sewageJobIdentificationMaterialList = sewageJobIdentificationMaterialService.getSewageJobIdentificationMaterialList(ids);

        for (SewageJobIdentificationMaterial sewageJobIdentificationMaterial : sewageJobIdentificationMaterialList) {

            if (sewageJobIdentificationMaterial == null) {
                return R.failed(String.format("没有找到 id 为 %s 的检材", sewageJobIdentificationMaterial.getId()));
            }

            if (sewageJobIdentificationMaterial.getStatus() != 2) {    // 状态:0=刚创建待送检,1=已送检待受理,2=已受理, -1=受理被拒绝
                return R.failed(String.format("这个检材不在[已送检待受理]状态, 无法受理"));
            }
            if (opCode == -1) {
                if (StrUtil.isBlank(sewageJobIdentificationMaterial.getAcceptComments())) {
                    return R.failed(String.format("必须提供 acceptComments 属性"));
                }
            }
            //==== 样本判断 =====
            if (opCode == 1) {
                // 受理通过, 修改属性

                SampleBox sb1 = null;
                SampleBox sb2 = null;

                // 如果之前没有生成过样本编号, 现在填写样本重量了, 那么就生成样本编号
                //if ((sewageJobIdentificationMaterial.getSample1Present() != null) && (sewageJobIdentificationMaterial.getSample1Present())) {
                if (StrUtil.isBlank(sewageJobIdentificationMaterial.getSample1No())) {
                    sewageJobIdentificationMaterial.setSample1No(sewageJobService.getNewSampleNo(sewageJobIdentificationMaterial.getImNo(), 1));
                }
//                if (StrUtil.isBlank(sewageJobIdentificationMaterial.getSample1BoxId())) {
//                    throw new RuntimeException(String.format("样本1没有放到某个样本盒子中"));
//                }
//                sb1 = sampleBoxService.getById(sewageJobIdentificationMaterial.getSample1BoxId());
//                if (sb1 == null) {
//                    throw new RuntimeException(String.format("样本1放到的样本盒子 %s 不存在", sewageJobIdentificationMaterial.getSample1BoxId()));
//                }
//                if (!StrUtil.equalsIgnoreCase(sewageJobIdentificationMaterial.getStorageMethod(), sb1.getStorageMethod())) {
//                    throw new RuntimeException(String.format("检材的存储方法是[%s], 样本1放到的样本盒子的存储方法是[%s], 存储方法不一致", sewageJobIdentificationMaterial.getStorageMethod(), sb1.getStorageMethod()));
//                }

                //}
                if ((sewageJobIdentificationMaterial.getSample2Present() != null) && (sewageJobIdentificationMaterial.getSample2Present())) {
                    if (StrUtil.isBlank(sewageJobIdentificationMaterial.getSample2No())) {
                        sewageJobIdentificationMaterial.setSample2No(sewageJobService.getNewSampleNo(sewageJobIdentificationMaterial.getImNo(), 2));
                    }

//                    if (StrUtil.isBlank(sewageJobIdentificationMaterial.getSample2BoxId())) {
//                        throw new RuntimeException(String.format("样本2没有放到某个样本盒子中"));
//                    }
//                    sb2 = sampleBoxService.getById(sewageJobIdentificationMaterial.getSample2BoxId());
//                    if (sb2 == null) {
//                        throw new RuntimeException(String.format("样本2放到的样本盒子 %s 不存在", sewageJobIdentificationMaterial.getSample2BoxId()));
//                    }
//                    if (!StrUtil.equalsIgnoreCase(sewageJobIdentificationMaterial.getStorageMethod(), sb2.getStorageMethod())) {
//                        throw new RuntimeException(String.format("检材的存储方法是[%s], 样本2放到的样本盒子的存储方法是[%s], 存储方法不一致", sewageJobIdentificationMaterial.getStorageMethod(), sb2.getStorageMethod()));
//                    }

                }

                // if ((sewageJobIdentificationMaterial.getSample1Present() == null) || (!sewageJobIdentificationMaterial.getSample1Present())) {
                // 	if ((sewageJobIdentificationMaterial.getSample2Present() == null) || (!sewageJobIdentificationMaterial.getSample2Present())) {
                // 		throw new RuntimeException(String.format("这个检材至少要有一份样本"));
                // 	}
                // }

//                if ((sb1 != null) && (sb2 != null)) {
//                    // 两个样本都装到盒子中了
//                    if (StrUtil.equalsIgnoreCase(sb1.getSampleType(), sb2.getSampleType())) {
//                        throw new RuntimeException(String.format("两个不同的样本应该装入不同类型的盒子中"));
//                    }
//                }
            } else {
                if (ids.size() != 1) {
                    throw new RuntimeException(String.format("不支持批量拒绝受理"));
                }
            }

            List<UpdateInfo> pis = sewageJobIdentificationMaterial.getUpdateLog();
            if (pis == null) {
                pis = new ArrayList<UpdateInfo>();
            }
            UpdateInfo pi = new UpdateInfo(dlpUser.getId(),
                    dlpUser.getName(),
                    dlpUser.getOrgId(),
                    dlpUser.getOrgName(),
                    "受理",
                    opCode);
            pis.add(pi);
            sewageJobIdentificationMaterial.setUpdateLog(pis);

            if (opCode == 1) {
                sewageJobIdentificationMaterial.setStatus(3);    // 状态:0=刚创建待送检,1=已填报待送检,2=已送检待受理,3=已受理, -1=受理被拒绝 -2作废
            } else if (opCode == -1) {
                sewageJobIdentificationMaterial.setStatus(-1);
            } else {
                throw new RuntimeException(String.format("受理的操作码 opCode 只能是 1(受理) 或 -1(拒绝受理), 不能是 %d", opCode));
            }

            UpdateWrapper<SewageJobIdentificationMaterial> updateWrapper = new UpdateWrapper<SewageJobIdentificationMaterial>();
            updateWrapper.eq("id", sewageJobIdentificationMaterial.getId());        // 查询条件是 id 相等

            if (opCode == 1) {
                //updateWrapper.set("sample1_present", sewageJobIdentificationMaterial.getSample1Present());
                updateWrapper.set("sample2_present", sewageJobIdentificationMaterial.getSample2Present());
                updateWrapper.set("sample1_no", sewageJobIdentificationMaterial.getSample1No());
                updateWrapper.set("sample2_no", sewageJobIdentificationMaterial.getSample2No());
                updateWrapper.set("sample1_box_id", sewageJobIdentificationMaterial.getSample1BoxId());
                updateWrapper.set("sample2_box_id", sewageJobIdentificationMaterial.getSample2BoxId());
            } else {
                updateWrapper.set(StrUtil.isNotBlank(sewageJobIdentificationMaterialDTO.getAcceptComments()), "accept_comments", sewageJobIdentificationMaterialDTO.getAcceptComments());
            }

            updateWrapper.set("status", sewageJobIdentificationMaterial.getStatus());    // 状态:0=刚创建待送检,1=已送检待受理,2=已受理, -1=受理被拒绝
            updateWrapper.set("accept_time", LocalDateTime.now());
            updateWrapper.set("update_log", sewageJobIdentificationMaterial.getUpdateLog(), "javaType=String,jdbcType=VARCHAR,typeHandler=com.baomidou.mybatisplus.extension.handlers.FastjsonTypeHandler");

            if (!sewageJobIdentificationMaterialService.update(sewageJobIdentificationMaterial, updateWrapper)) {
                return R.failed(sewageJobIdentificationMaterial, "保存检材受理信息失败");

            }

            if (opCode == 1) {
                List<Sample> oldSampleList = sewageJobService.createSample(sewageJobIdentificationMaterial, dlpUser.getId());
                //将样本数据写到流转系统中-----------------------
                List<Sample> newSampleList = new ArrayList<Sample>();
                for (Sample oldSamObj : oldSampleList) {
                    Sample newSamObj = new Sample();
                    BeanUtils.copyProperties(oldSamObj, newSamObj);
                    newSampleList.add(newSamObj);
                }
                System.out.println("-------------写入远程检材系统的数据量为：" + newSampleList.size());
                remoteSampleService.enterSample(newSampleList);


            } else {
                return R.ok(sewageJobIdentificationMaterial, "检材拒绝受理完成");
            }
        }
        return R.ok(sewageJobIdentificationMaterialList, "检材受理完成");

    }
    //打印污水样品标签

    /**
     * 获取污水样品标签打印内容
     *
     * @param id
     * @param httpServletResponse
     * @return
     * @throws Exception
     */
    @ApiOperation(value = "打印污水检材标签", notes = "打印污水检材标签")
    @GetMapping("/printSewageSampleLabel")
    public String printSewageSampleLabel(String id, String sampleNo, HttpServletResponse httpServletResponse) throws
            Exception {

        return sewageJobIdentificationMaterialService.buildSewageSampleLableContent(id, sampleNo);
    }

    //污水作废
    @PostMapping("/cancellationMaterial")
    @ApiOperation(value = "污水检材作废，只有分中心管理员用户才可以作废污水", notes = "污水检材作废，只有分中心管理员用户才可以作废污水")
    public R cancellationMaterial(@RequestBody CommonIdDto commonIdDto) {
        Assert.isTrue(commonIdDto.getIdList().size() > 0, "作废检材数量为0，不能完成此操作");
        List<SewageJobIdentificationMaterial> cancelMaterialList = sewageJobIdentificationMaterialService.cancellationMaterial(commonIdDto.getIdList(), commonIdDto.getCauseOfCancellation());
        return R.ok(cancelMaterialList, "操作成功");
    }

    //查看联系人
    @GetMapping("/getContactsInfo")
    @ApiOperation(value = "获取检材的送检联系人", notes = "获取检材的送检联系人")
    public R<SysUser> getContactsInfo(String materialId) {
        ;
        Assert.isTrue(StringUtils.isNotBlank(materialId), "检材ID不能为空");
        SewageJobIdentificationMaterial sewageJobIdentificationMaterial = sewageJobIdentificationMaterialService.getById(materialId);
        String pid = sewageJobIdentificationMaterial.getInformantId();
        R<SysUser> userInfo = remoteUserService.innerGetById(pid);
        return userInfo;
    }

    //获取污水未签收的数量
    @GetMapping("/getSewageMaterialNoSignCount")
    @ApiOperation(value = "获取未签收的检材数量", notes = "获取未签收的检材数量")
    public R getSewageMaterialNoSignCount(@RequestParam(value = "materialStatus") List<Integer> materialStatus) {
        Assert.notEmpty(materialStatus, "参数状态不能为空");
        return R.ok(sewageJobIdentificationMaterialService.getSewageMaterialNoSignCount(materialStatus));
    }

    @PutMapping("/edit/material/{id}")
    @ApiOperation(value = "退回样本", notes = "退回样本，将样本状态置为已填报状态")
    public R<String> editMaterial(@PathVariable(value = "id") String id) {
        SewageJobIdentificationMaterial sewageJobIdentificationMaterial = sewageJobIdentificationMaterialService.getById(id);
        if (sewageJobIdentificationMaterial == null) {
            throw new RuntimeException(String.format("没有查询到ID为" + id + "的样本"));
        }
        sewageJobIdentificationMaterial.setStatus(1);//将样本状态改为已填报未送检状态
        sewageJobIdentificationMaterial.setReturnOrNot(-1);
        return sewageJobIdentificationMaterialService.updateById(sewageJobIdentificationMaterial) ? R.ok("退回成功") : R.failed("退回失败");
    }

    //=====================================================================================
    // 测试, 创建 污水任务的检材
    @GetMapping("/create100")
    public R xxx_TestCreate100() {
        List<SewageJobItemVO> theList = sewageJobItemService.getSewageJobItemVOList(Wrappers.<SewageJobItem>query());

        for (SewageJobItemVO sji : theList) {
            List<SewageJobIdentificationMaterial> oldIMList = sewageJobIdentificationMaterialService.list(Wrappers.<SewageJobIdentificationMaterial>query()
                    .eq("job_item_id", sji.getId())
            );
            if (oldIMList.isEmpty()) {
                // 这个任务清单项对应的检材不存在
                String deliverOrgId = null;
                String collectorGroup = null;

                //SysOrg qo = new SysOrg();
                //qo.setAreaCode(sji.getSewageTreatmentPlantDistrictCode());
                R<List<SysOrg>> orgList = remoteOrgService.innerGetOrgListByAreaCode(sji.getSewageTreatmentPlantDistrictCode());
                if ((orgList != null) && (orgList.getData() != null) && (orgList.getData().size() > 0)) {
                    SysOrg org = orgList.getData().get(0);
                    deliverOrgId = org.getOrgId();
                    collectorGroup = org.getName();
                }

                SewageJobIdentificationMaterial sjim = new SewageJobIdentificationMaterial();
                sjim.setId(IdWorker.get32UUID().toUpperCase());
                sjim.setJobId(sji.getJobId());
                sjim.setJobItemId(sji.getId());
                sjim.setImNo(null);
                sjim.setSample1No(null);
                sjim.setSample2No(null);
                sjim.setSample1BoxId(null);
                sjim.setSample2BoxId(null);
                sjim.setSampleType(1);
                sjim.setSampleName(sji.getSewageTreatmentPlantName());
                sjim.setCollectPlaceAreaCode(sji.getSewageTreatmentPlantCityCode());
                sjim.setCollectPlace(sji.getSewageTreatmentPlantAddress());
                sjim.setDailyFlow(sji.getSewageTreatmentPlantDailyFlowReal());
                sjim.setWaterInletCount(sji.getSewageTreatmentPlantWaterInletCount());
                //sjim.setWaterInletNumber(RandomUtil.randomInt(1, sji.getSewageTreatmentPlantWaterInletCount() + 1));
                sjim.setDomesticSewageProportion(sji.getSewageTreatmentPlantDomesticSewageProportion());
                sjim.setServicePopulation(sji.getSewageTreatmentPlantServicePopulation());
                sjim.setSamplePh(7.0);
                sjim.setSamplingDepth(Math.round(100 * RandomUtil.randomDouble(0.1, 5.0)) / 100.0);
                sjim.setSampleProperties("灰色液体");
                sjim.setSamplingMethod(RandomUtil.randomEle(new String[]{"自动采样仪", "人工采样"}));
                sjim.setSamplingMode(RandomUtil.randomEle(new String[]{"24小时混合样", "瞬间水样"}));
                sjim.setStorageMethod(RandomUtil.randomEle(new String[]{"冷藏", "冷冻"}));
                sjim.setCollector(TestUtils.genPersonName());
                sjim.setCollectorGroup(collectorGroup);
                sjim.setCollectTime(LocalDateTime.now());
                sjim.setDeliverMethod(RandomUtil.randomEle(new String[]{"人工", "邮寄"}));
                sjim.setAcceptTime(null);
                sjim.setStatus(0);
                sjim.setComments(null);

                sjim.setDelivererName(TestUtils.genPersonName());
                sjim.setDelivererCert(TestUtils.genCertificateType());
                sjim.setDelivererId(RandomUtil.randomNumbers(18));
                sjim.setDelivererPhone(TestUtils.genMobile());
                sjim.setDelivererPosition(TestUtils.genPosition());
                sjim.setDeliverOrgId(deliverOrgId);


                sewageJobIdentificationMaterialService.save(sjim);
            }
        }
        return R.ok("全部任务的清单都已添加对应的检材");
    }

    @GetMapping("/get/any/sjIm")
    @ApiOperation(value = "查询当前机构用户，对应不同状态的检材列表", notes = "查询当前机构用户，对应不同状态的检材列表\n" +
            "status = 0 : 未填报的检材列表（未超过截止日期的检材）\n" +
            "status = 1 : 已填报的检材列表（未超过截止日期的检材）\n" +
            "status = 2 : 已送检的检材列表（所有已送检的检材）\n" +
            "status = -2 : 已作废的检材（所有作废的检材\n" +
            "keywords：默认传空，视为查所有，可通过样本编号、样本名称查询")

    public R<IPage<SewageJobIdentificationMaterialVO>> getAnyIm(Page page, HttpServletRequest theHttpServletRequest, @RequestParam(value = "statuses") Integer[] statuses, @RequestParam(value = "keyword", required = false) String keyword) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        boolean r = false;
        for (Integer status : statuses) {
            if (status == 2) {
                r = true;
            } else if (status == 3) {
                r = true;
            } else if (status == -2) {
                r = true;
            }
        }
        SysOrg orgInfo = getOrgInfo(dlpUser.getOrgId());
        if (!r) {//代表查询的是已填报或者未填报未到截止日期的检材
            List<String> itemId = sewageJobIdentificationMaterialMapper.getItemId(orgInfo.getAreaCode());//查询未过期的任务清单
            if (itemId != null && itemId.size() > 0) {
                IPage<SewageJobIdentificationMaterialVO> sewageJobIdentificationMaterialVOS = sewageJobIdentificationMaterialMapper.getAnyIm(page, Wrappers.<SewageJobIdentificationMaterialVO>query()
                        .and(StrUtil.isNotBlank(keyword), wrapper -> wrapper
                                .like("im_no", keyword)
                                .or()
                                .like("sample_name", keyword)
                                .or()
                                .like("job_name", keyword))
                        .in(statuses != null && statuses.length != 0, "status", statuses)
                        .in("job_item_id", itemId)

                        .orderByAsc("im_no", "job_name")
                );
                return R.ok(sewageJobIdentificationMaterialVOS);
            }
        } else {//查询已送检、已拒收的检材列表
            List<String> itemId = sewageJobIdentificationMaterialMapper.getNotTimeOutItemId(orgInfo.getAreaCode());//查询所有的任务清单
            if (itemId.size() == 0) {
                itemId.add("");
            }
            IPage<SewageJobIdentificationMaterialVO> sewageJobIdentificationMaterialVOS = sewageJobIdentificationMaterialMapper.getAnyIm(page, Wrappers.<SewageJobIdentificationMaterialVO>query()
                    .and(StrUtil.isNotBlank(keyword), wrapper -> wrapper
                            .like("im_no", keyword)
                            .or()
                            .like("sample_name", keyword)
                            .or()
                            .like("job_name", keyword))

                    .in(statuses != null && statuses.length != 0,"status", statuses)
                    .in("job_item_id", itemId)
                    .orderByAsc("im_no", "job_name"));
            return R.ok(sewageJobIdentificationMaterialVOS);
        }
        return R.ok(new Page<SewageJobIdentificationMaterialVO>(), "没有查询到相关信息！");
    }

    @GetMapping("/get/imNo")
    public R getByImNo(@RequestParam(value = "imNo") String imNo) {
        SewageJobIdentificationMaterial byImNo = sewageJobIdentificationMaterialService.getByImNo(imNo);
        return R.ok(byImNo);
    }

    @GetMapping("/get/voList")
    public R<List<SewageJobIdentificationMaterialVO>> getSewageJobIdentificationMaterialVOListByJobId(@RequestParam(value = "jobId") String jobId) {

        List<SewageJobIdentificationMaterialVO> voList = sewageJobIdentificationMaterialMapper
                .getSewageJobIdentificationMaterialVOListByJobId(Wrappers.<SewageJobIdentificationMaterial>query()
                        .eq("sm.job_id", jobId)
                        .orderByDesc("sm.im_no"));
        return R.ok(voList);
    }

    private SysOrg getOrgInfo(String orgId) {
        SysOrg userOrg = null;
        R<SysOrg> r = remoteOrgService.getById(orgId);
        if (r.getCode() == CommonConstants.SUCCESS) {
            userOrg = r.getData();
        } else {
            throw new RuntimeException(String.format("没有找到 orgId 为 %s 的机构, 请确认用户所属机构的正确性!", orgId));
        }
        return userOrg;
    }
}
