package digital.laboratory.platform.sewage.controller;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.constant.CommonConstants;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.sewage.entity.SewageJobIdentificationMaterial;
import digital.laboratory.platform.sewage.entity.SewageTreatmentPlant;
import digital.laboratory.platform.sewage.entity.*;
import digital.laboratory.platform.sewage.service.*;
import digital.laboratory.platform.sewage.vo.SewageJobIdentificationMaterialVO;
import digital.laboratory.platform.sewage.vo.SewageJobItemVO;
import digital.laboratory.platform.sys.entity.SysOrg;
import digital.laboratory.platform.sys.feign.RemoteOrgService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.security.Principal;
import java.util.ArrayList;
import java.util.List;


/**
 * 污水检测任务清单
 *
 * @author Zhang Xiaolong created at  2022-09-13
 * @describe 污水检测任务清单 前端控制器
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/sewage_job_item")
@Api(value = "sewage_job_item", tags = "112-污水检测任务清单管理")
public class SewageJobItemController {

	private final SewageJobItemService sewageJobItemService;
	private final SewageJobIdentificationMaterialService sewageJobIdentificationMaterialService;
	private final SewageTreatmentPlantService sewageTreatmentPlantService;
	private final SewageImportLogService sewageImportLogService;

	private final SewageJobService sewageJobService;

	private final RemoteOrgService remoteOrgService;

	/**
	 * 通过id查询污水检测任务清单
	 *
	 * @param id id
	 *
	 * @return R
	 */
	@ApiOperation(value = "通过id查询", notes = "通过id查询")
	@GetMapping("/{id}")
	@PreAuthorize("@pms.hasPermission('SewageJobItemGet')")
	public R getById(@PathVariable("id") String id, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		List<SewageJobItemVO> theList = sewageJobItemService.getSewageJobItemVOList(Wrappers.<SewageJobItem>query()
				.eq("ji.id", id)
		);

		if ((theList == null) && (theList.size() == 0)) {
			return R.failed(String.format("没有找到 id 为 %s 的任务项", id));
		}

		return R.ok(theList.get(0));
	}

	/**
	 * 分页查询
	 *
	 * @param page          分页对象
	 * @param jobId
	 * @param name
	 * @param address
	 *
	 * @return
	 */
	@ApiOperation(value = "分页查询", notes = "分页查询, 查询条件是 jobId 或 污水处理厂名字 或 污水处理厂地址")
	@GetMapping("/page")
	@PreAuthorize("@pms.hasPermission('SewageJobItemList')")
	public R getSewageJobItemPage(Page page,
	                              @RequestParam(value = "jobId", required = false) String jobId,
	                              @RequestParam(value = "name", required = false) String name,
	                              @RequestParam(value = "address", required = false) String address,
	                              HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		final SysOrg userOrg;    // 定义成 final才可以被后面的 Lambda 表达式使用
		R r = remoteOrgService.getById(dlpUser.getOrgId());
		if (r.getCode() == CommonConstants.SUCCESS) {
			userOrg = (SysOrg) r.getData();
		}
		else {
			return R.failed(String.format("没有找到 orgId 为 %s 的机构, 请确认用户所属机构的正确性!", dlpUser.getOrgId()));
		}

		IPage<SewageJobItemVO> thePage = sewageJobItemService.getSewageJobItemVOPage(page, Wrappers.<SewageJobItem>query()
						.eq(StrUtil.isNotBlank(jobId), "ji.job_id", jobId)
						.and(!dlpUser.isStaff(), (wrapper) -> wrapper                // 如果不是分中心员工
								.eq("stp.province_code", userOrg.getAreaCode())    // 污水处理厂的省级地区码与用户机构所在地区一致
								.or()
								.eq("stp.city_code", userOrg.getAreaCode())    // 污水处理厂的市级地区码与用户机构所在地区一致
								.or()
								.eq("stp.district_code", userOrg.getAreaCode())    // 污水处理厂的区级地区码与用户机构所在地区一致
						)
						.like(StrUtil.isNotBlank(name), "stp.name", name)
						.like(StrUtil.isNotBlank(address), "stp.address", address)
						.orderByDesc("stp.province_code")
						.orderByDesc("stp.city_code")
						.orderByDesc("stp.district_code")
						.orderByDesc("stp.name")
		);


		for (SewageJobItemVO item : thePage.getRecords()) {
			item.setCanEdit(false);
			if (dlpUser.isStaff()) {
				item.setCanEdit(true);
			}
			else {
				if (item.getCreateByOrgLevel()>=userOrg.getOrgLevel()) {
					item.setCanEdit(true);
				}
			}
		}

		return R.ok(thePage);
	}
	//获取任务清单项
	@ApiOperation(value = "污水厂列表查询   --add", notes = "通过任务ID污水厂列表查询  --add")
	@PreAuthorize("@pms.hasPermission('SewageJobItemList')")
	@PostMapping("/getSewageJobItemListByTaskID")
	public R getSewageJobItemListByTaskID(@RequestBody JSONObject jsonParms){
		String taskId=jsonParms.getString("taskId");
		List<String> areaCodeList=JSONObject.parseObject(jsonParms.getJSONArray("areaCodeList").toJSONString(),List.class);
		List<String> test=new ArrayList<>();
		List<String> sewageJobItemList=sewageJobItemService.getSewageJobItemListByTaskID(taskId,areaCodeList);
		return R.ok(sewageJobItemList,"获取数据成功");
	}
	//从任务项里，获取检材
	@ApiOperation(value = "查询检材根据污水厂ID   --add", notes = "查询检材根据污水厂ID  --add")
	@PreAuthorize("@pms.hasPermission('SewageJobItemList')")
	@PostMapping("/getSewageMaterialByTaskItem")
	public  R getSewageMaterialByTaskItem(@RequestBody JSONObject  jsonParms){
		String taskId=jsonParms.getString("taskId");
		List<String> stpIdList=JSONObject.parseObject(jsonParms.getJSONArray("stpList").toJSONString(),List.class);
		return R.ok(sewageJobItemService.getSewageMaterialListByStpId(taskId,stpIdList),"获取数据成功");
	}
	/**
	 * 列表查询
	 *
	 * @param sewageJobItem 污水检测任务清单
	 *
	 * @return
	 */
	@ApiOperation(value = "列表查询", notes = "列表查询, 查询条件是 jobId 或 污水处理厂名字 或 污水处理厂地址")
	@GetMapping("/list")
	@PreAuthorize("@pms.hasPermission('SewageJobItemList')")
	public R getSewageJobItemList(SewageJobItem sewageJobItem,
	                              @RequestParam(value = "jobId", required = false) String jobId,
	                              @RequestParam(value = "name", required = false) String name,
	                              @RequestParam(value = "address", required = false) String address,
	                              HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		final SysOrg userOrg;    // 定义成 final才可以被后面的 Lambda 表达式使用
		R r = remoteOrgService.getById(dlpUser.getOrgId());
		if (r.getCode() == CommonConstants.SUCCESS) {
			userOrg = (SysOrg) r.getData();
		}
		else {
			return R.failed(String.format("没有找到 orgId 为 %s 的机构, 请确认用户所属机构的正确性!", dlpUser.getOrgId()));
		}

		List<SewageJobItemVO> theList = sewageJobItemService.getSewageJobItemVOList(Wrappers.<SewageJobItem>query()
				.eq(StrUtil.isNotBlank(jobId), "ji.job_id", jobId)
				.and(!dlpUser.isStaff(), (wrapper) -> wrapper                // 如果不是分中心员工
						.eq("stp.province_code", userOrg.getAreaCode())    // 污水处理厂的省级地区码与用户机构所在地区一致
						.or()
						.eq("stp.city_code", userOrg.getAreaCode())    // 污水处理厂的市级地区码与用户机构所在地区一致
						.or()
						.eq("stp.district_code", userOrg.getAreaCode())    // 污水处理厂的区级地区码与用户机构所在地区一致
				)
				.like(StrUtil.isNotBlank(name), "stp.name", name)
				.like(StrUtil.isNotBlank(address), "stp.address", address)
				.orderByDesc("stp.province_code")
				.orderByDesc("stp.city_code")
				.orderByDesc("stp.district_code")
				.orderByDesc("stp.name")
		);
        //将污水厂下的样品一起加载出来，传回客户端
		for (SewageJobItemVO item : theList) {
			item.setCanEdit(false);
			List<SewageJobIdentificationMaterialVO> tmpList=sewageJobIdentificationMaterialService.getSewageJobIdentificationMaterialVOList(Wrappers.<SewageJobIdentificationMaterial>query()
					.eq("job_item_id",item.getId()));
			item.setSewageJobIdentificationMaterialVO(tmpList);
			if (dlpUser.isStaff()) {
				item.setCanEdit(true);
			}
			else {
				if (item.getCreateByOrgLevel()>=userOrg.getOrgLevel()) {
					item.setCanEdit(true);
				}
			}
		}
		return R.ok(theList);
	}


	/**
	 * 新增污水检测任务清单
	 *
	 * @param sewageJobItem 污水检测任务清单
	 *
	 * @return R
	 */
	@ApiOperation(value = "新增污水检测任务清单", notes = "新增污水检测任务清单")
	@SysLog("新增污水检测任务清单")
	@PostMapping
	@PreAuthorize("@pms.hasPermission('SewageJobItemCreate')")
	public R postAddObject(@RequestBody SewageJobItem sewageJobItem, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		final SysOrg userOrg;    // 定义成 final才可以被后面的 Lambda 表达式使用
		R r = remoteOrgService.getById(dlpUser.getOrgId());
		if (r.getCode() == CommonConstants.SUCCESS) {
			userOrg = (SysOrg) r.getData();
		}
		else {
			return R.failed(String.format("没有找到 orgId 为 %s 的机构, 请确认用户所属机构的正确性!", dlpUser.getOrgId()));
		}
		int userLevel = 99;
		if (dlpUser.isStaff()) {
			// 分中心
			userLevel = 0;
		}
		else if (StrUtil.endWith(userOrg.getAreaCode(), "0000")) {
			// 省级
			userLevel = 1;
		}
		else if (StrUtil.endWith(userOrg.getAreaCode(), "00")) {
			// 市级
			userLevel = 2;
		}
		else {
			// 区级
			userLevel = 3;
		}

		sewageJobItem.setCreateByOrgLevel(userLevel);

		sewageJobItem.setId(IdWorker.get32UUID().toUpperCase());
		return R.ok(sewageJobItemService.save(sewageJobItem));

	}


	/**
	 * 批量导入污水检测任务清单
	 *
	 * @param jsonParam 污水检测任务清单
	 *
	 * @return R
	 */
	@ApiOperation(value = "批量导入污水检测任务清单", notes = "批量导入污水检测任务清单, json格式, 格式大致如下:\n" +
			"<pre>\n格式:\n" +
			"{\n" +
			"  \"jobId\": \"179DE0C6CEAAA5F11ED0B9FFA7044023\",\n" +
			"  \"sheetList\": [\n" +
			"    {\n" +
			"      \"name\": \"sheet1\",\n" +
			"      \"dataList\": [\n" +
			"        {\n" +
			"          \"序号\": 1,\n" +
			"          \"污水处理厂名称\":\"第9个污水处理厂\"\n" +
			"        },\n" +
			"        {\n" +
			"          \"序号\": 2,\n" +
			"          \"污水处理厂名称\":\"第19个污水处理厂\"\n" +
			"        }\n" +
			"      ]\n" +
			"    },\n" +
			"    {\n" +
			"      \"name\": \"sheet2\",\n" +
			"      \"dataList\": [\n" +
			"        {\n" +
			"          \"序号\": 11,\n" +
			"          \"污水处理厂名称\":\"第11个污水处理厂\"\n" +
			"        },\n" +
			"        {\n" +
			"          \"序号\": 12,\n" +
			"          \"污水处理厂名称\":\"第12个污水处理厂\"\n" +
			"        }\n" +
			"      ]\n" +
			"    }\n" +
			"  ]\n" +
			"}\n" +
			"</pre>")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "jsonParam", value = "需要导入的数据", required = true, paramType = "body"),
	})
	@SysLog("批量导入污水检测任务清单")
	@PostMapping("/excel")
	@PreAuthorize("@pms.hasPermission('SewageJobItemCreate')")
	@Transactional(rollbackFor = {RuntimeException.class, Exception.class})
	public R postImportObjectBatch(@RequestBody JSONObject jsonParam, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		final SysOrg userOrg;    // 定义成 final才可以被后面的 Lambda 表达式使用
		R r = remoteOrgService.getById(dlpUser.getOrgId());
		if (r.getCode() == CommonConstants.SUCCESS) {
			userOrg = (SysOrg) r.getData();
		}
		else {
			return R.failed(String.format("没有找到 orgId 为 %s 的机构, 请确认用户所属机构的正确性!", dlpUser.getOrgId()));
		}
		int userLevel = 99;
		if (dlpUser.isStaff()) {
			// 分中心
			userLevel = 0;
		}
		else if (StrUtil.endWith(userOrg.getAreaCode(), "0000")) {
			// 省级
			userLevel = 1;
		}
		else if (StrUtil.endWith(userOrg.getAreaCode(), "00")) {
			// 市级
			userLevel = 2;
		}
		else {
			// 区级
			userLevel = 3;
		}
		String errorInfo = "";  // 错误信息
		Boolean errorExists = false;

		List<SewageJobItem> itemList = new ArrayList<SewageJobItem>();
		String jobId = jsonParam.getString("jobId");
		JSONArray sheetList = jsonParam.getJSONArray("sheetList");
		for (int i=0; i<sheetList.size(); i++) {
			JSONObject sheet = sheetList.getJSONObject(i);
			String sheetName = sheet.getString("name");    // sheet 的名称
			JSONArray dataList = sheet.getJSONArray("dataList");
			for (int j=0; j<dataList.size(); j++) {
				JSONObject jsonSJI = dataList.getJSONObject(j);
				Integer itemNo = jsonSJI.getInteger("序号");
				String stpName = jsonSJI.getString("污水处理厂名称");

				List<SewageTreatmentPlant> stpList = sewageTreatmentPlantService.list(Wrappers.<SewageTreatmentPlant>query()
								.eq("name", stpName)
								);
				String stpId = "";
				if ((stpList!=null) && (stpList.size()==1)) {
					stpId = stpList.get(0).getId();


					List<SewageJobItem> oldItemList = sewageJobItemService.list(Wrappers.<SewageJobItem>query()
							.eq("plant_id", stpId)
							.eq("job_id", jobId)
					);
					if ((oldItemList != null) && (oldItemList.size()==0)) {
						SewageJobItem sji = new SewageJobItem();
						sji.setId(IdWorker.get32UUID().toUpperCase());
						sji.setJobId(jobId);
						sji.setCreateByOrgLevel(userOrg.getOrgLevel());
						sji.setPlantId(stpId);
						sji.setComments("导入的数据");
						itemList.add(sji);
					}
					else if (oldItemList==null) {
						errorInfo += String.format("错误! sheet:%s 序号:%d 污水处理厂名称:%s, 错误信息: 查询现有任务清单出错, 返回数据为 null\n", sheetName, itemNo, stpName );
						errorExists = true;
					}
					else if (oldItemList.size()>0) {
						errorInfo += String.format("错误! sheet:%s 序号:%d 污水处理厂名称:%s, 错误信息: 现有任务清单中已经有该污水处理厂, 无需再次添加\n", sheetName, itemNo, stpName );
						errorExists = true;
					}
				}
				else if (stpList==null) {
					errorInfo += String.format("错误! sheet:%s 序号:%d 污水处理厂名称:%s, 错误信息: 查询污水处理厂信息返回数据为 null\n", sheetName, itemNo, stpName );
					errorExists = true;
				}
				else if (stpList.size()==0) {
					errorInfo += String.format("错误! sheet:%s 序号:%d 污水处理厂名称:%s, 错误信息: 污水处理厂不存在, 应该先添加污水处理厂后才能导入此条数据\n", sheetName, itemNo, stpName );
					errorExists = true;
				}


			}
		}


		// sewageJobItem.setCreateByOrgLevel(userLevel);
		//
		// sewageJobItem.setId(IdWorker.get32UUID().toUpperCase());
		// return R.ok(sewageJobItemService.save(sewageJobItem));

		if (errorExists) {
			// 出错了
			itemList = null;
		}
		else {
			// 没有出错
			for (SewageJobItem sji : itemList) {
				sewageJobItemService.save(sji);
			}
			if (StrUtil.isBlank(errorInfo)) {
				errorInfo = String.format("导入数据成功, 共有 %d 条任务清单数据", itemList.size());
			}
		}

		SewageImportLog siLog = new SewageImportLog();
		siLog.setId(IdWorker.get32UUID().toUpperCase());
		siLog.setModule("任务清单导入");
		siLog.setLog(errorInfo);
		sewageImportLogService.save(siLog);

		return R.ok(itemList, errorInfo);

	}



	/**
	 * 修改污水检测任务清单
	 *
	 * @param sewageJobItem 污水检测任务清单
	 *
	 * @return R
	 */
	@ApiOperation(value = "修改污水检测任务清单", notes = "修改污水检测任务清单" +
			"用户只能修改本级机构或下级机构创建的任务清单。\n" +
			"只能修改 comments 属性\n" +
			"其他属性应该保持原样\n" +
			"如果需要修改 jobId, 正确的做法是删除本条任务清单, 然后为对应的任务添加任务清单\n" +
			"如果需要修改污水处理 plantId, 正确的做法是新加一条任务清单, 并指定正确的污水处理厂\n"
	)
	@SysLog("修改污水检测任务清单")
	@PutMapping
	@PreAuthorize("@pms.hasPermission('SewageJobItemEdit')")
	public R putUpdateById(@RequestBody SewageJobItem sewageJobItem, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		final SysOrg userOrg;    // 定义成 final才可以被后面的 Lambda 表达式使用
		R r = remoteOrgService.getById(dlpUser.getOrgId());
		if (r.getCode() == CommonConstants.SUCCESS) {
			userOrg = (SysOrg) r.getData();
		}
		else {
			return R.failed(String.format("没有找到 orgId 为 %s 的机构, 请确认用户所属机构的正确性!", dlpUser.getOrgId()));
		}
		int userLevel = 99;
		if (dlpUser.isStaff()) {
			// 分中心
			userLevel = 0;
		}
		else if (StrUtil.endWith(userOrg.getAreaCode(), "0000")) {
			// 省级
			userLevel = 1;
		}
		else if (StrUtil.endWith(userOrg.getAreaCode(), "00")) {
			// 市级
			userLevel = 2;
		}
		else {
			// 区级
			userLevel = 3;
		}

		SewageJobItem oldsji = sewageJobItemService.getById(sewageJobItem.getId());
		if (oldsji.getCreateByOrgLevel() < userLevel) {
			return R.failed(String.format("这条任务清单项 %s 是由上级机构创建的，你没有权限修改!", sewageJobItem.getId()));
		}

		if (!StrUtil.equals(oldsji.getJobId(), sewageJobItem.getJobId())) {
			return R.failed(String.format("任务清单项中的任务id不能修改!"));
		}

		if (!StrUtil.equals(oldsji.getPlantId(), sewageJobItem.getPlantId())) {
			return R.failed(String.format("任务清单项中的污水处理厂不能修改!"));
		}

		if (sewageJobItemService.updateById(sewageJobItem)) {
			return R.ok(sewageJobItem, "修改任务清单项成功");
		}
		else {
			return R.failed(sewageJobItem, "修改任务清单项失败");
		}
	}

	/**
	 * 通过id删除污水检测任务清单
	 *
	 * @param id id
	 *
	 * @return R
	 */
	@ApiOperation(value = "通过id删除污水检测任务清单", notes = "通过id删除污水检测任务清单" +
			"用户只能删除本级机构或下级机构创建的任务清单。\n"
	)
	@SysLog("通过id删除污水检测任务清单")
	@DeleteMapping("/{id}")
	@PreAuthorize("@pms.hasPermission('SewageJobItemDelete')")
	public R deleteById(@PathVariable String id, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		final SysOrg userOrg;    // 定义成 final才可以被后面的 Lambda 表达式使用
		R r = remoteOrgService.getById(dlpUser.getOrgId());
		if (r.getCode() == CommonConstants.SUCCESS) {
			userOrg = (SysOrg) r.getData();
		}
		else {
			return R.failed(String.format("没有找到 orgId 为 %s 的机构, 请确认用户所属机构的正确性!", dlpUser.getOrgId()));
		}
		int userLevel = 99;
		if (dlpUser.isStaff()) {
			// 分中心
			userLevel = 0;
		}
		else if (StrUtil.endWith(userOrg.getAreaCode(), "0000")) {
			// 省级
			userLevel = 1;
		}
		else if (StrUtil.endWith(userOrg.getAreaCode(), "00")) {
			// 市级
			userLevel = 2;
		}
		else {
			// 区级
			userLevel = 3;
		}

		SewageJobItem oldsji = sewageJobItemService.getById(id);
		if (oldsji.getCreateByOrgLevel() < userLevel) {
			return R.failed(String.format("这条任务清单项 %s 是由上级机构创建的，你没有权限删除!", id));
		}

		List<SewageJobIdentificationMaterial> sjimList = sewageJobIdentificationMaterialService.list(Wrappers.<SewageJobIdentificationMaterial>query()
				.eq("job_item_id", id));
		if (sjimList.size() > 0) {
			return R.failed(String.format("这条任务清单项 %s 已经创建了对应的检材, 不能删除它", id));
		}

		if (sewageJobItemService.removeById(id)) {
			return R.ok(oldsji, "任务清单项删除成功");
		}
		else {
			return R.failed(oldsji, "任务清单项删除失败");
		}

	}




	/**
	 * 批量新增污水检测任务清单
	 *
	 * @param jsonParam 污水检测任务清单
	 *
	 * @return R
	 */
	@ApiOperation(value = "批量新增污水检测任务清单", notes = "批量新增污水检测任务清单, json格式, 格式大致如下:\n" +
			"<pre>\n格式:\n" +
			"{\n" +
			"  \"jobId\": \"179DE0C6CEAAA5F11ED0B9FFA7044023\",\n" +
			"  \"sewageTreatmentPlantIds\": [\"179DE0C6CEAAA5F11ED0B9FFA7044023\", \"123DE0C6CEAAA5F11ED0B9FFA7044123\"]\n" +
			"}\n" +
			"</pre>")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "jsonParam", value = "需要新增的数据", required = true, paramType = "body"),
	})
	@SysLog("批量新增污水检测任务清单")
	@PostMapping("/batch")
	@PreAuthorize("@pms.hasPermission('SewageJobItemCreate')")
	@Transactional(rollbackFor = {RuntimeException.class, Exception.class})
	public R postAddObjectBatch(@RequestBody JSONObject jsonParam, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		final SysOrg userOrg;    // 定义成 final才可以被后面的 Lambda 表达式使用
		R r = remoteOrgService.getById(dlpUser.getOrgId());
		if (r.getCode() == CommonConstants.SUCCESS) {
			userOrg = (SysOrg) r.getData();
		}
		else {
			return R.failed(String.format("没有找到 orgId 为 %s 的机构, 请确认用户所属机构的正确性!", dlpUser.getOrgId()));
		}
		//int userLevel = getUserLvByAreaCode(userOrg.getAreaCode());//获取用户级别
		//保存清单的逻辑是：获取污水厂的ID，如果发现该任务已经有清单了，那么我们就删除之前清单，以现在提交的清单为主，如果之前的清单没有，则直接插入
		JSONArray sewageTreatmentPlantList = jsonParam.getJSONArray("sewageTreatmentPlantIds");//本次提交的污水厂名单
		String taskJobId=jsonParam.getString("jobId");//获取本次任务的任务ID
		List<SewageJobItem> retItemList=sewageJobItemService.batchAddJobItemToTask(taskJobId,userOrg,sewageTreatmentPlantList,dlpUser);
		return  R.ok(retItemList, String.format("批量加入任务清单成功, 共有 %d 条任务清单数据", retItemList.size()));
		//下面为之前的历史代码，看起来有点混乱，我修改一下，增加一点可读性  xy  add 07-01,
		/*
		List<SewageJobItem> itemList = new ArrayList<SewageJobItem>();
		List<SewageJobIdentificationMaterial> jcList = new ArrayList<SewageJobIdentificationMaterial>();

		String jobId = jsonParam.getString("jobId");
		JSONArray sewageTreatmentPlantIds = jsonParam.getJSONArray("sewageTreatmentPlantIds");//获取到本次选择的污水厂ID

		for (int i = 0; i < sewageTreatmentPlantIds.size(); i++) {
			String sewageTreatmentPlantId = sewageTreatmentPlantIds.getString(i);

			List<SewageTreatmentPlant> stpList = sewageTreatmentPlantService.list(Wrappers.<SewageTreatmentPlant>query()
					.eq("id", sewageTreatmentPlantId)
			);
			String stpId = "";
			if ((stpList != null) && (stpList.size() == 1)) {
				SewageTreatmentPlant stp = stpList.get(0);
				//如果不是分中心的员工，则
				if (!dlpUser.isStaff()) {
					if (!(StrUtil.equals(userOrg.getAreaCode(), stp.getProvinceCode())
							|| StrUtil.equals(userOrg.getAreaCode(), stp.getCityCode())
							|| StrUtil.equals(userOrg.getAreaCode(), stp.getDistrictCode())
					))
					{
						throw new RuntimeException(String.format("[%s]不在你管辖范围内，你没有权限添加到任务清单中", stp.getName()));
					}
				}
				stpId = stp.getId();

				List<SewageJobItem> oldItemList = sewageJobItemService.list(Wrappers.<SewageJobItem>query()
						.eq("plant_id", stpId)
						.eq("job_id", jobId)
				);
				if ((oldItemList != null) && (oldItemList.size() == 0)) {

					SewageJobItem sji = new SewageJobItem();
					sji.setId(IdWorker.get32UUID().toUpperCase());
					sji.setJobId(jobId);
					sji.setCreateByOrgLevel(userOrg.getOrgLevel());
					sji.setPlantId(stpId);
					sji.setComments("");
					itemList.add(sji);
				}
				else if (oldItemList == null) {
					throw new RuntimeException(String.format("错误信息: 查询现有任务清单出错, 返回数据为 null! 污水处理厂id为 %s\n", sewageTreatmentPlantIds));
				}
				else if (oldItemList.size() > 0) {

					System.out.println(String.format("错误信息: 现有任务清单中已经有该污水处理厂, 无需再次添加! 污水处理厂id为 %s, 名称为: %s\n", sewageTreatmentPlantIds, stp.getName()));
					//throw new RuntimeException(String.format("错误信息: 现有任务清单中已经有该污水处理厂, 无需再次添加! 污水处理厂id为 %s\n", sewageTreatmentPlantIds));
				}
			}
			else if (stpList == null) {
				throw new RuntimeException(String.format("错误信息: 查询污水处理厂信息返回数据为 null! 污水处理厂id为 %s\n", sewageTreatmentPlantIds));
			}
			else if (stpList.size() == 0) {
				throw new RuntimeException(String.format("错误信息: 污水处理厂不存在, 应该先添加污水处理厂后才能导入此条数据! 污水处理厂id为 %s\n", sewageTreatmentPlantIds));
			}
		}

		for (SewageJobItem sji : itemList) {
			sewageJobItemService.save(sji);

			//获取检材编号
			List<String> ImNoList=GenJCNo(sji.getPlantId(),sji.getJobId());
			List<SewageJobIdentificationMaterial> jcBeanList=new ArrayList<SewageJobIdentificationMaterial>();
			for (String jcNo:ImNoList) {
				//插入检材
				SewageJobIdentificationMaterial sewageJobIdentificationMaterial=new SewageJobIdentificationMaterial();
				sewageJobIdentificationMaterial.setId(IdWorker.get32UUID().toUpperCase());
				sewageJobIdentificationMaterial.setJobId(sji.getJobId());
				sewageJobIdentificationMaterial.setJobItemId(sji.getId());
				sewageJobIdentificationMaterial.setImNo(jcNo);
				jcBeanList.add(sewageJobIdentificationMaterial);
			}
			sewageJobIdentificationMaterialService.saveBatch(jcBeanList);

		}
		return R.ok(itemList, String.format("批量加入任务清单成功, 共有 %d 条任务清单数据", itemList.size()));
		*/
	}

	/**
	 * 批量删除污水检测任务清单
	 *
	 * @param jsonParam json格式的参数
	 *
	 * @return R
	 */
	@ApiOperation(value = "批量删除污水检测任务清单", notes = "批量删除污水检测任务清单\n" +
			"用户只能删除本级机构或下级机构创建的任务清单。<b>注意: 这是POST方法, 参数在请求体 RequestBody 中</b>\n" +
			"<pre>\n" +
			"参数(RequestBody):\n" +
			"  jobId 任务id" +
			"  sewageTreatmentPlantIds 污水处理厂id数组\n" +
			"\n" +
			"格式:\n" +
			"{\n" +
			"  \"jobId\": \"179DE0C6CEAAA5F11ED0B9FFA7044023\",\n" +
			"  \"sewageTreatmentPlantIds\": [\"179DE0C6CEAAA5F11ED0B9FFA7044023\", \"123DE0C6CEAAA5F11ED0B9FFA7044123\"]\n" +
			"}\n" +
			"</pre>"
	)
	@SysLog("批量删除污水检测任务清单")
	@PostMapping("/delete_batch")
	@PreAuthorize("@pms.hasPermission('SewageJobItemDelete')")
	@Transactional(rollbackFor = {RuntimeException.class, Exception.class})
	public R deleteBatch(@RequestBody JSONObject jsonParam, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		final SysOrg userOrg;    // 定义成 final才可以被后面的 Lambda 表达式使用
		R r = remoteOrgService.getById(dlpUser.getOrgId());
		if (r.getCode() == CommonConstants.SUCCESS) {
			userOrg = (SysOrg) r.getData();
		}
		else {
			return R.failed(String.format("没有找到 orgId 为 %s 的机构, 请确认用户所属机构的正确性!", dlpUser.getOrgId()));
		}
		int userLevel = 99;
		if (dlpUser.isStaff()) {
			// 分中心
			userLevel = 0;
		}
		else if (StrUtil.endWith(userOrg.getAreaCode(), "0000")) {
			// 省级
			userLevel = 1;
		}
		else if (StrUtil.endWith(userOrg.getAreaCode(), "00")) {
			// 市级
			userLevel = 2;
		}
		else {
			// 区级
			userLevel = 3;
		}

		String jobId = jsonParam.getString("jobId");
		JSONArray jonSewageTreatmentPlantIds = jsonParam.getJSONArray("sewageTreatmentPlantIds");

		ArrayList<String> sewageTreatmentPlantIds = new ArrayList<>();
		for (int i = 0; i < jonSewageTreatmentPlantIds.size(); i++) {
			String sewageTreatmentPlantId = jonSewageTreatmentPlantIds.getString(i);
			sewageTreatmentPlantIds.add(sewageTreatmentPlantId);
		}

		ArrayList<String> jobItemIdList = new ArrayList<>();
		for (String sewageTreatmentPlantId : sewageTreatmentPlantIds) {
			List<SewageJobItemVO> theList = sewageJobItemService.getSewageJobItemVOList(Wrappers.<SewageJobItem>query()
							.eq("ji.job_id", jobId)
							.eq("stp.id", sewageTreatmentPlantId)
					// .and(!dlpUser.isStaff(), (wrapper) -> wrapper						// 如果不是分中心员工
					// 		.eq("stp.province_code", userOrg.getAreaCode())		// 污水处理厂的省级地区码与用户机构所在地区一致
					// 		.or()
					// 		.eq("stp.city_code", userOrg.getAreaCode())			// 污水处理厂的市级地区码与用户机构所在地区一致
					// 		.or()
					// 		.eq("stp.district_code", userOrg.getAreaCode())		// 污水处理厂的区级地区码与用户机构所在地区一致
					// )
					// .orderByDesc("stp.province_code")
					// .orderByDesc("stp.city_code")
					// .orderByDesc("stp.district_code")
					// .orderByDesc("stp.name")
			);
			if ((theList == null) || (theList.size() == 0)) {
//				throw new RuntimeException(String.format("任务清单中不存在 id 为 %d 的污水处理厂", sewageTreatmentPlantId));
			}
			else {
				SewageJobItemVO sji = theList.get(0);
				if (!dlpUser.isStaff()) {
					if (sji.getCreateByOrgLevel() == 0) {
						throw new RuntimeException(String.format("[%s]是分中心创建的任务清单项，你没有权限删除", sji.getSewageTreatmentPlantName()));
					}
					if (userOrg.getOrgLevel() > sji.getCreateByOrgLevel()) {
						throw new RuntimeException(String.format("[%s]是上级机构创建的任务清单项，你没有权限删除", sji.getSewageTreatmentPlantName()));
					}
					if (!(StrUtil.equals(userOrg.getAreaCode(), sji.getSewageTreatmentPlantProvinceCode())
							|| StrUtil.equals(userOrg.getAreaCode(), sji.getSewageTreatmentPlantCityCode())
							|| StrUtil.equals(userOrg.getAreaCode(), sji.getSewageTreatmentPlantDistrictCode())
					))
					{
						throw new RuntimeException(String.format("[%s]不在你管辖范围内，你没有权限删除", sji.getSewageTreatmentPlantName()));
					}
				}

				// 看一下有没有检材, 如果有检材, 就不能删除了
				List<SewageJobIdentificationMaterial> sjimList = sewageJobIdentificationMaterialService.list(
						Wrappers.<SewageJobIdentificationMaterial>query()
								.eq( "job_item_id", sji.getId())
				);

				if ((sjimList!=null) && (sjimList.size()>0)) {
					throw new RuntimeException(String.format("[%s]这条任务清单项已经录入检材信息, 不能删除", sji.getSewageTreatmentPlantName()));
				}

				jobItemIdList.add(sji.getId());
			}
		}

		for (String id : jobItemIdList) {
//System.out.println(String.format("sewageJobItemService.removeById -> %s ", id));
			sewageJobItemService.removeById(id);
		}
		return R.ok(sewageTreatmentPlantIds, "任务清单项删除成功");
	}

}
