package digital.laboratory.platform.sewage.controller;

import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.constant.CommonConstants;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.core.util.TestUtils;
import digital.laboratory.platform.sewage.mapper.SewageJobMapper;
import digital.laboratory.platform.sewage.service.SewageJobService;
import digital.laboratory.platform.sys.entity.Area;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.sewage.entity.SewageImportLog;
import digital.laboratory.platform.sewage.service.SewageImportLogService;
import digital.laboratory.platform.sewage.service.SewageTreatmentPlantService;
import digital.laboratory.platform.sewage.vo.SewageTreatmentPlantVO;
import digital.laboratory.platform.sewage.entity.SewageTreatmentPlant;
import digital.laboratory.platform.sys.entity.SysOrg;
import digital.laboratory.platform.sys.feign.RemoteAreaService;
import digital.laboratory.platform.sys.feign.RemoteOrgService;
import digital.laboratory.platform.sys.feign.RemoteUserService;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.security.Principal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * 污水处理厂
 *
 * @author Zhang Xiaolong created at  2022-03-22
 * @describe 污水处理厂 前端控制器
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/sewage_treatment_plant")
@Api(tags = "110-污水处理厂管理")
public class SewageTreatmentPlantController {

    private final SewageTreatmentPlantService sewageTreatmentPlantService;
    private final SewageImportLogService sewageImportLogService;

    private final RemoteAreaService remoteAreaService;
    private final RemoteUserService remoteUserService;
    private final RemoteOrgService remoteOrgService;

    private final SewageJobMapper sewageJobMapper;

    private final SewageJobService sewageJobService;

    /**
     * 通过id查询污水处理厂
     *
     * @param id id
     * @return R
     */
    @ApiOperation(value = "通过id查询", notes = "通过id查询")
    @GetMapping("/{id}")
    @PreAuthorize("@pms.hasPermission('SewageTreatmentPlantGet')")
    public R getById(@PathVariable("id") String id, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
        SysOrg userOrg = null;
        R r = remoteOrgService.getById(dlpUser.getOrgId());
        if (r.getCode() == CommonConstants.SUCCESS) {
            userOrg = (SysOrg) r.getData();
        } else {
            return R.failed(String.format("没有找到 orgId 为 %s 的机构, 请确认用户所属机构的正确性!", dlpUser.getOrgId()));
        }

        SewageTreatmentPlant sewageTreatmentPlant = sewageTreatmentPlantService.getById(id);
        if (sewageTreatmentPlant != null) {
            if (dlpUser.isStaff() ||
                    StrUtil.equalsIgnoreCase(userOrg.getAreaCode(), sewageTreatmentPlant.getProvinceCode()) ||
                    StrUtil.equalsIgnoreCase(userOrg.getAreaCode(), sewageTreatmentPlant.getCityCode()) ||
                    StrUtil.equalsIgnoreCase(userOrg.getAreaCode(), sewageTreatmentPlant.getDistrictCode())
            ) {
                return R.ok(sewageTreatmentPlant);
            } else {
                return R.failed(String.format("你没有权限访问id为 %s 的污水处理厂的数据", id));
            }
        } else {
            return R.failed(String.format("没有找到id为 %s 的污水处理厂", id));
        }

    }


    /**
     * 列表查询
     *
     * @param sewageTreatmentPlant – 污水处理厂 查询条件
     * @return
     */
    @ApiOperation(value = "列表查询", notes = "列表查询")
    @GetMapping("/list")
    @PreAuthorize("@pms.hasPermission('SewageTreatmentPlantList')")
    public R getSewageTreatmentPlantList(SewageTreatmentPlant sewageTreatmentPlant, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
        final SysOrg userOrg;    // 定义成 final才可以被后面的 Lambda 表达式使用
        R r = remoteOrgService.getById(dlpUser.getOrgId());
        if (r.getCode() == CommonConstants.SUCCESS) {
            userOrg = (SysOrg) r.getData();
        } else {
            return R.failed(String.format("没有找到 orgId 为 %s 的机构, 请确认用户所属机构的正确性!", dlpUser.getOrgId()));
        }

        List<SewageTreatmentPlantVO> list = sewageTreatmentPlantService.getSewageTreatmentPlantVOList(Wrappers.<SewageTreatmentPlant>query()
                .like(StrUtil.isNotBlank(sewageTreatmentPlant.getName()), "name", sewageTreatmentPlant.getName())
                .and(!dlpUser.isStaff(), (wrapper) -> wrapper                // 如果不是分中心员工
                        .eq("province_code", userOrg.getAreaCode())    // 污水处理厂的省级地区码与用户机构所在地区一致
                        .or()
                        .eq("city_code", userOrg.getAreaCode())    // 污水处理厂的市级地区码与用户机构所在地区一致
                        .or()
                        .eq("district_code", userOrg.getAreaCode())    // 污水处理厂的区级地区码与用户机构所在地区一致
                )
                .eq(StrUtil.isNotBlank(sewageTreatmentPlant.getProvinceCode()), "province_code", sewageTreatmentPlant.getProvinceCode())
                .eq(StrUtil.isNotBlank(sewageTreatmentPlant.getCityCode()), "city_code", sewageTreatmentPlant.getCityCode())
                .eq(StrUtil.isNotBlank(sewageTreatmentPlant.getDistrictCode()), "district_code", sewageTreatmentPlant.getDistrictCode())
                .orderByDesc("name")
        );
        return R.ok(list);
    }

    /**
     * 分页查询
     *
     * @param page                 分页对象
     * @param sewageTreatmentPlant 污水处理厂
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page")
    @PreAuthorize("@pms.hasPermission('SewageTreatmentPlantList')")
    public R getSewageTreatmentPlantPage(Page page, SewageTreatmentPlant sewageTreatmentPlant, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
        final SysOrg userOrg;    // 定义成 final才可以被后面的 Lambda 表达式使用
        R r = remoteOrgService.getById(dlpUser.getOrgId());
        if (r.getCode() == CommonConstants.SUCCESS) {
            userOrg = (SysOrg) r.getData();
        } else {
            return R.failed(String.format("没有找到 orgId 为 %s 的机构, 请确认用户所属机构的正确性!", dlpUser.getOrgId()));
        }
        return R.ok(sewageTreatmentPlantService.getSewageTreatmentPlantVOPage(page, Wrappers.<SewageTreatmentPlant>query()
                .like(StrUtil.isNotBlank(sewageTreatmentPlant.getName()), "name", sewageTreatmentPlant.getName().trim())
                .and(!dlpUser.isStaff(), (wrapper) -> wrapper                // 如果不是分中心员工
                        .eq("province_code", userOrg.getAreaCode())    // 污水处理厂的省级地区码与用户机构所在地区一致
                        .or()
                        .eq("city_code", userOrg.getAreaCode())    // 污水处理厂的市级地区码与用户机构所在地区一致
                        .or()
                        .eq("district_code", userOrg.getAreaCode())    // 污水处理厂的区级地区码与用户机构所在地区一致
                )
                .eq(StrUtil.isNotBlank(sewageTreatmentPlant.getProvinceCode()), "province_code", sewageTreatmentPlant.getProvinceCode())
                .eq(StrUtil.isNotBlank(sewageTreatmentPlant.getCityCode()), "city_code", sewageTreatmentPlant.getCityCode())
                .eq(StrUtil.isNotBlank(sewageTreatmentPlant.getDistrictCode()), "district_code", sewageTreatmentPlant.getDistrictCode())
                .orderByDesc("name")
        ));

    }


    /**
     * 新增污水处理厂
     *
     * @param sewageTreatmentPlant 污水处理厂
     * @return R
     */
    @ApiOperation(value = "新增污水处理厂", notes = "新增污水处理厂")
    @SysLog("新增污水处理厂")
    @PostMapping
    @PreAuthorize("@pms.hasPermission('SewageTreatmentPlantCreate')")
    public R postAddObject(@RequestBody SewageTreatmentPlant sewageTreatmentPlant, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
        final SysOrg userOrg;    // 定义成 final才可以被后面的 Lambda 表达式使用
        R r = remoteOrgService.getById(dlpUser.getOrgId());
        if (r.getCode() == CommonConstants.SUCCESS) {
            userOrg = (SysOrg) r.getData();
        } else {
            return R.failed(String.format("没有找到 orgId 为 %s 的机构, 请确认用户所属机构的正确性!", dlpUser.getOrgId()));
        }
        sewageTreatmentPlant.setId(IdWorker.get32UUID().toUpperCase());

        if (sewageTreatmentPlantService.save(sewageTreatmentPlant)) {
            return R.ok(sewageTreatmentPlant, "新增污水处理厂成功");
        } else {
            return R.failed(sewageTreatmentPlant, "新增污水处理厂失败");
        }

    }

    /**
     * 批量新增污水处理厂
     *
     * @param jsonParam 污水处理厂
     * @return R
     */
    @ApiOperation(value = "批量新增污水处理厂", notes = "批量新增污水处理厂\n" +
            "格式:\n" +
            "<pre>\n" +
            "{\n" +
            "\t\"sheetList\": [\n" +
            "\t\t{\n" +
            "\t\t\t\"name\": \"sheet1\",\n" +
            "\t\t\t\"dataList\": [\n" +
            "\t\t\t\t{\n" +
            "\t\t\t\t\t\"序号\": 1,\n" +
            "\t\t\t\t\t\"污水处理厂名称\": \"xxxxxxxxx\",\n" +
            "\t\t\t\t\t\"污水处理厂简称\": \"xx\",\n" +
            "\t\t\t\t\t\"地址\": \"某省某市某区某某街道某某号\",\n" +
            "\t\t\t\t\t\"管网覆盖区域\": \"大部分城区\",\n" +
            "\t\t\t\t\t\"省行政区编码\": \"610000\",\n" +
            "\t\t\t\t\t\"省禁毒办联系人和电话\": \"张三 13900000000\",\n" +
            "\t\t\t\t\t\"市行政区编码\": \"610100\",\n" +
            "\t\t\t\t\t\"市禁毒办联系人和电话\": \"李四 13700000000\",\n" +
            "\t\t\t\t\t\"区县行政区编码\": \"610112\",\n" +
            "\t\t\t\t\t\"区县禁毒办联系人和电话\": \"王五 18900000000\",\n" +
            "\t\t\t\t\t\"污水进水口数量\": 5,\n" +
            "\t\t\t\t\t\"设计日均处理污水总量(万立方米/日)\": 180.30,\n" +
            "\t\t\t\t\t\"实际日均处理污水总量(万立方米/日)\": 179.21,\n" +
            "\t\t\t\t\t\"生活污水占比(%)\": 20.23,\n" +
            "\t\t\t\t\t\"服务人口数(万人)\": 23.32,\n" +
            "\t\t\t\t\t\"备注\": \"某某县2021年1月上报\"\n" +
            "\t\t\t\t},\n" +
            "\t\t\t\t{\n" +
            "\t\t\t\t\t\"序号\": 1,\n" +
            "\t\t\t\t\t\"污水处理厂名称\": \"测试潜水处理厂\",\n" +
            "\t\t\t\t\t\"污水处理厂简称\": \"测厂\",\n" +
            "\t\t\t\t\t\"地址\": \"xx某省某市某区某某街道某某号\",\n" +
            "\t\t\t\t\t\"管网覆盖区域\": \"大部分城区\",\n" +
            "\t\t\t\t\t\"省行政区编码\": \"610000\",\n" +
            "\t\t\t\t\t\"省禁毒办联系人和电话\": \"x张三 15900000000\",\n" +
            "\t\t\t\t\t\"市行政区编码\": \"610100\",\n" +
            "\t\t\t\t\t\"市禁毒办联系人和电话\": \"李四x 14700000000\",\n" +
            "\t\t\t\t\t\"区县行政区编码\": \"610112\",\n" +
            "\t\t\t\t\t\"区县禁毒办联系人和电话\": \"王x五 12900000000\",\n" +
            "\t\t\t\t\t\"污水进水口数量\": 3,\n" +
            "\t\t\t\t\t\"设计日均处理污水总量(万立方米/日)\": 80.30,\n" +
            "\t\t\t\t\t\"实际日均处理污水总量(万立方米/日)\": 79.21,\n" +
            "\t\t\t\t\t\"生活污水占比(%)\": 90.23,\n" +
            "\t\t\t\t\t\"服务人口数(万人)\": 123.32,\n" +
            "\t\t\t\t\t\"备注\": \"某某县2020年5月上报\"\n" +
            "\t\t\t\t}\n" +
            "\t\t\t]\n" +
            "\t\t}\n" +
            "\t]\n" +
            "}\n" +
            "</pre>")
    @SysLog("批量新增污水处理厂")
    @PostMapping("/excel")
    @PreAuthorize("@pms.hasPermission('SewageTreatmentPlantCreate')")
    public R postAddObjectBatch(@RequestBody JSONObject jsonParam, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
        final SysOrg userOrg;    // 定义成 final才可以被后面的 Lambda 表达式使用
        R r = remoteOrgService.getById(dlpUser.getOrgId());
        if (r.getCode() == CommonConstants.SUCCESS) {
            userOrg = (SysOrg) r.getData();
        } else {
            return R.failed(String.format("没有找到 orgId 为 %s 的机构, 请确认用户所属机构的正确性!", dlpUser.getOrgId()));
        }
        String errorInfo = "";  // 错误信息
        Boolean errorExists = false;
        List<SewageTreatmentPlant> stpListRet = new ArrayList<SewageTreatmentPlant>();
        JSONObject excelDataObj = (JSONObject) jsonParam.getJSONArray("sheetList").get(0);
        JSONArray excelDataObjDetailList = (JSONArray) excelDataObj.get("dataList");
        for (int j = 0; j < excelDataObjDetailList.size(); j++) {
            JSONObject jobj = (JSONObject) excelDataObjDetailList.get(j);
            String provinceName = jobj.get("省").toString().trim();
            String cityName = jobj.get("市").toString().trim();
            String districtName = jobj.get("区县").toString().trim();
            String status = jobj.get("状态").toString().trim();
//			String provinceCode=jobj.get("省级编码").toString().trim();
//			String cityCode=jobj.get("市级编码").toString().trim();
//			String districtCode=jobj.get("县级编码").toString().trim();
            // 根据名称去查询地区编码
            R<Map<String, String>> mapR = remoteAreaService.listAreaVOByName(provinceName, cityName, districtName);
            Map<String, String> areaMap = new HashMap<>();
            if (mapR.getCode() == CommonConstants.SUCCESS) {
                areaMap = mapR.getData();
            } else {
                throw new RuntimeException(String.format("省市县名字导入出错，请检查名称的正确性！"));
            }

            String factoryName = jobj.get("污水处理厂名称").toString().trim();
            String factoryNo = jobj.get("污水厂编号").toString().trim();
            String factoryAddress = jobj.get("地址").toString().trim();
            SewageTreatmentPlant sewageTreatmentPlant = new SewageTreatmentPlant();
            sewageTreatmentPlant.setId(IdWorker.get32UUID());
            sewageTreatmentPlant.setNumber(factoryNo);
            sewageTreatmentPlant.setName(factoryName);
            sewageTreatmentPlant.setAddress(factoryAddress);
            sewageTreatmentPlant.setProvinceName(provinceName);
            sewageTreatmentPlant.setCityName(cityName);
            sewageTreatmentPlant.setDistrictName(districtName);
            sewageTreatmentPlant.setProvinceCode(areaMap.get("provinceId"));
            sewageTreatmentPlant.setCityCode(areaMap.get("cityId"));

            String districtCode = areaMap.get("areaId");

            //如果没有查到行政编码，就从99递增创建区县级的行政编码
            if (!StrUtil.isNotBlank(districtCode)) {
                String code = sewageJobService.getDistrictCode(provinceName, cityName, districtName);
                sewageTreatmentPlant.setDistrictCode(code);
            } else {
                sewageTreatmentPlant.setDistrictCode(districtCode);
            }
            sewageTreatmentPlant.setWaterInletCount(1);
            if (status.equals("启用")) {
                sewageTreatmentPlant.setStatus(1);
            } else if (status.equals("停用")) {
                sewageTreatmentPlant.setStatus(-1);
            } else {
                throw new RuntimeException(String.format("未填写状态或状态填写有误，请填入启用/停用即可!!!"));
            }
            SewageTreatmentPlant temp = sewageTreatmentPlantService.getOne(Wrappers.<SewageTreatmentPlant>query()
                    .eq("name", sewageTreatmentPlant.getName())
                    .eq("city_name", sewageTreatmentPlant.getCityName())
                    .eq("province_name", sewageTreatmentPlant.getProvinceName())
                    .eq("district_name", sewageTreatmentPlant.getDistrictName()));

            if (temp != null) {
                System.out.println("污水厂：【" + sewageTreatmentPlant.getCityName() + "】【" + sewageTreatmentPlant.getName() + "】已经导入，请不要重复导入");
                errorInfo += String.format("错误! 污水处理厂名称:%s, 错误信息: 同名的污水处理厂已经存在!\n", sewageTreatmentPlant.getName());
                errorExists = true;
            } else {
                stpListRet.add(sewageTreatmentPlant);
                sewageTreatmentPlantService.save(sewageTreatmentPlant);
            }
        }
        System.out.println("本次导入污水厂数量：" + stpListRet.size());

//		sewageTreatmentPlantService.saveBatch(stpListRet);
        SewageImportLog siLog = new SewageImportLog();
        siLog.setId(IdWorker.get32UUID().toUpperCase());
        siLog.setModule("污水处理厂导入");
        siLog.setLog(errorInfo);
        sewageImportLogService.save(siLog);
        return R.ok(stpListRet, errorInfo);

		/*
		String errorInfo = "";  // 错误信息
		Boolean errorExists = false;
		List<SewageTreatmentPlant> stpList = new ArrayList<SewageTreatmentPlant>();
		JSONArray sheetList = jsonParam.getJSONArray("sheetList");
		for (int i=0; i<sheetList.size(); i++) {
			JSONObject sheet = sheetList.getJSONObject(i);
			String sheetName = sheet.getString("name");    // sheet 的名称
			JSONArray dataList = sheet.getJSONArray("dataList");
			for (int j = 0; j < dataList.size(); j++) {
				JSONObject jsonSTP = dataList.getJSONObject(j);

				JSONObject formSTP = new JSONObject();

				// 因为前端提交的 key 有可能会含有空格或回车, 我们需要专门判断一下
				for (String key : jsonSTP.keySet()) {
					String pureKey = BusinessCodeUtils.removeSymbols(key);
					if (StrUtil.equals(pureKey, "序号")) {
						formSTP.put("序号", jsonSTP.get(key));
					}
					else if (StrUtil.equals(pureKey, "污水处理厂名称")) {
						formSTP.put("污水处理厂名称", jsonSTP.get(key));
					}
					else if (StrUtil.equals(pureKey, "名称")) {
						formSTP.put("污水处理厂名称", jsonSTP.get(key));
					}
					else if (StrUtil.equals(pureKey, "污水处理厂简称")) {
						formSTP.put("污水处理厂简称", jsonSTP.get(key));
					}
					else if (StrUtil.equals(pureKey, "简称")) {
						formSTP.put("污水处理厂简称", jsonSTP.get(key));
					}
					else if (StrUtil.equals(pureKey, "地址")) {
						formSTP.put("地址", jsonSTP.get(key));
					}
					else if (StrUtil.equals(pureKey, "管网覆盖区域")) {
						formSTP.put("管网覆盖区域", jsonSTP.get(key));
					}
					else if (StrUtil.equals(pureKey, "管网覆盖")) {
						formSTP.put("管网覆盖区域", jsonSTP.get(key));
					}
					else if (StrUtil.equals(pureKey, "省行政区编码")) {
						formSTP.put("省行政区编码", jsonSTP.get(key));
					}
					else if (StrUtil.equals(pureKey, "省禁毒办联系人和电话")) {
						formSTP.put("省禁毒办联系人和电话", jsonSTP.get(key));
					}
					else if (StrUtil.equals(pureKey, "省禁毒办联系人")) {
						formSTP.put("省禁毒办联系人和电话", jsonSTP.get(key));
					}
					else if (StrUtil.equals(pureKey, "市行政区编码")) {
						formSTP.put("市行政区编码", jsonSTP.get(key));
					}
					else if (StrUtil.equals(pureKey, "市禁毒办联系人和电话")) {
						formSTP.put("市禁毒办联系人和电话", jsonSTP.get(key));
					}
					else if (StrUtil.equals(pureKey, "市禁毒办联系人")) {
						formSTP.put("市禁毒办联系人和电话", jsonSTP.get(key));
					}
					else if (StrUtil.equals(pureKey, "区县行政区编码")) {
						formSTP.put("区县行政区编码", jsonSTP.get(key));
					}
					else if (StrUtil.equals(pureKey, "区县行政区编码")) {
						formSTP.put("区县行政区编码", jsonSTP.get(key));
					}
					else if (StrUtil.equals(pureKey, "区行政区编码")) {
						formSTP.put("区县行政区编码", jsonSTP.get(key));
					}
					else if (StrUtil.equals(pureKey, "区县禁毒办联系人和电话")) {
						formSTP.put("区县禁毒办联系人和电话", jsonSTP.get(key));
					}
					else if (StrUtil.equals(pureKey, "县禁毒办联系人和电话")) {
						formSTP.put("区县禁毒办联系人和电话", jsonSTP.get(key));
					}
					else if (StrUtil.equals(pureKey, "县禁毒办联系人和电话")) {
						formSTP.put("区县禁毒办联系人和电话", jsonSTP.get(key));
					}
					else if (StrUtil.equals(pureKey, "区禁毒办联系人和电话")) {
						formSTP.put("区县禁毒办联系人和电话", jsonSTP.get(key));
					}
					else if (StrUtil.equals(pureKey, "区县禁毒办联系人")) {
						formSTP.put("区县禁毒办联系人和电话", jsonSTP.get(key));
					}
					else if (StrUtil.equals(pureKey, "区县禁毒办")) {
						formSTP.put("区县禁毒办联系人和电话", jsonSTP.get(key));
					}
					else if (StrUtil.equals(pureKey, "污水进水口数量")) {
						formSTP.put("污水进水口数量", jsonSTP.get(key));
					}
					else if (StrUtil.equals(pureKey, "污水进水口")) {
						formSTP.put("污水进水口数量", jsonSTP.get(key));
					}
					else if (StrUtil.equals(pureKey, "进水口数量")) {
						formSTP.put("污水进水口数量", jsonSTP.get(key));
					}
					else if (StrUtil.equals(pureKey, "进水口")) {
						formSTP.put("污水进水口数量", jsonSTP.get(key));
					}
					else if (StrUtil.equals(pureKey, "设计日均处理污水总量(万立方米/日)")) {
						formSTP.put("设计日均处理污水总量(万立方米/日)", jsonSTP.get(key));
					}
					else if (StrUtil.equals(pureKey, "设计日均处理污水总量")) {
						formSTP.put("设计日均处理污水总量(万立方米/日)", jsonSTP.get(key));
					}
					else if (StrUtil.equals(pureKey, "设计日均处理污水")) {
						formSTP.put("设计日均处理污水总量(万立方米/日)", jsonSTP.get(key));
					}
					else if (StrUtil.equals(pureKey, "实际日均处理污水总量(万立方米/日)")) {
						formSTP.put("实际日均处理污水总量(万立方米/日)", jsonSTP.get(key));
					}
					else if (StrUtil.equals(pureKey, "实际日均处理污水总量")) {
						formSTP.put("实际日均处理污水总量(万立方米/日)", jsonSTP.get(key));
					}
					else if (StrUtil.equals(pureKey, "实际日均处理污水")) {
						formSTP.put("实际日均处理污水总量(万立方米/日)", jsonSTP.get(key));
					}
					else if (StrUtil.equals(pureKey, "生活污水占比(%)")) {
						formSTP.put("生活污水占比(%)", jsonSTP.get(key));
					}
					else if (StrUtil.equals(pureKey, "生活污水占比")) {
						formSTP.put("生活污水占比(%)", jsonSTP.get(key));
					}
					else if (StrUtil.equals(pureKey, "服务人口数(万人)")) {
						formSTP.put("服务人口数(万人)", jsonSTP.get(key));
					}
					else if (StrUtil.equals(pureKey, "服务人口数")) {
						formSTP.put("服务人口数(万人)", jsonSTP.get(key));
					}
					else if (StrUtil.equals(pureKey, "服务人口")) {
						formSTP.put("服务人口数(万人)", jsonSTP.get(key));
					}
					else if (StrUtil.equals(pureKey, "备注")) {
						formSTP.put("备注", jsonSTP.get(key));
					}
				}

				Integer itemNo = formSTP.getInteger("序号");
				String name = formSTP.getString("污水处理厂名称");
				String shortName = formSTP.getString("污水处理厂简称");
				String address = formSTP.getString("地址");
				String coverageArea = formSTP.getString("管网覆盖区域");
				String provinceCode = formSTP.getString("省行政区编码");
				String provinceContacts = formSTP.getString("省禁毒办联系人和电话");
				String cityCode = formSTP.getString("市行政区编码");
				String cityContacts = formSTP.getString("市禁毒办联系人和电话");
				String districtCode = formSTP.getString("区县行政区编码");
				String districtContacts = formSTP.getString("区县禁毒办联系人和电话");
				Integer waterInletCount = formSTP.getInteger("污水进水口数量");
				Double dailyFlowDesign = formSTP.getDouble("设计日均处理污水总量(万立方米/日)");
				Double dailyFlowReal = formSTP.getDouble("实际日均处理污水总量(万立方米/日)");
				Double domesticSewageProportion = formSTP.getDouble("生活污水占比(%)");
				Double servicePopulation = formSTP.getDouble("服务人口数(万人)");
				String comments = formSTP.getString("备注");

				SewageTreatmentPlant oldstp =sewageTreatmentPlantService.getOne(Wrappers.<SewageTreatmentPlant>query()
						.eq("name", name)
				);
				if (oldstp == null) {
					SewageTreatmentPlant stp = new SewageTreatmentPlant();
					stp.setId(IdWorker.get32UUID().toUpperCase());
					stp.setName(name);
					stp.setShortName(shortName);
					stp.setAddress(address);
					stp.setCoverageArea(coverageArea);
					stp.setProvinceCode(provinceCode);
					stp.setProvinceContacts(provinceContacts);
					stp.setCityCode(cityCode);
					stp.setCityContacts(cityContacts);
					stp.setDistrictCode(districtCode);
					stp.setDistrictContacts(districtContacts);
					stp.setWaterInletCount(waterInletCount);
					stp.setDailyFlowDesign(dailyFlowDesign);
					stp.setDailyFlowReal(dailyFlowReal);
					stp.setDomesticSewageProportion(domesticSewageProportion);
					stp.setServicePopulation(servicePopulation);
					stp.setComments(comments);
					stpList.add(stp);
				}
				else {
					errorInfo += String.format("错误! sheet:%s 序号:%d 污水处理厂名称:%s, 错误信息: 同名的污水处理厂已经存在!\n", sheetName, itemNo, name );
					errorExists = true;
				}
			}
		}


		// sewageJobItem.setCreateByOrgLevel(userLevel);
		//
		// sewageJobItem.setId(IdWorker.get32UUID().toUpperCase());
		// return R.ok(sewageJobItemService.save(sewageJobItem));

		if (errorExists) {
			// 出错了
			stpList = null;
		}
		else {
			// 没有出错
			for (SewageTreatmentPlant stp : stpList) {
				sewageTreatmentPlantService.save(stp);
			}
			if (StrUtil.isBlank(errorInfo)) {
				errorInfo = String.format("导入数据成功, 共有 %d 条污水处理厂数据", stpList.size());
			}
		}

		SewageImportLog siLog = new SewageImportLog();
		siLog.setId(IdWorker.get32UUID().toUpperCase());
		siLog.setModule("污水处理厂导入");
		siLog.setLog(errorInfo);
		sewageImportLogService.save(siLog);

		return R.ok(stpList, errorInfo);*/
    }

    /**
     * 修改污水处理厂
     *
     * @param sewageTreatmentPlant 污水处理厂
     * @return R
     */
    @ApiOperation(value = "修改污水处理厂", notes = "修改污水处理厂")
    @SysLog("修改污水处理厂")
    @PutMapping
    @PreAuthorize("@pms.hasPermission('SewageTreatmentPlantEdit')")
    public R putUpdateById(@RequestBody SewageTreatmentPlant sewageTreatmentPlant, HttpServletRequest
            theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
        final SysOrg userOrg;    // 定义成 final才可以被后面的 Lambda 表达式使用
        R r = remoteOrgService.getById(dlpUser.getOrgId());
        if (r.getCode() == CommonConstants.SUCCESS) {
            userOrg = (SysOrg) r.getData();
        } else {
            return R.failed(String.format("没有找到 orgId 为 %s 的机构, 请确认用户所属机构的正确性!", dlpUser.getOrgId()));
        }

        SewageTreatmentPlant oldSewageTreatmentPlant = sewageTreatmentPlantService.getById(sewageTreatmentPlant.getId());
        if (oldSewageTreatmentPlant != null) {
            if (dlpUser.isStaff() ||
                    StrUtil.equalsIgnoreCase(userOrg.getAreaCode(), oldSewageTreatmentPlant.getProvinceCode()) ||
                    StrUtil.equalsIgnoreCase(userOrg.getAreaCode(), oldSewageTreatmentPlant.getCityCode()) ||
                    StrUtil.equalsIgnoreCase(userOrg.getAreaCode(), oldSewageTreatmentPlant.getDistrictCode())
            ) {
                return R.ok(sewageTreatmentPlantService.updateById(sewageTreatmentPlant));
            } else {
                return R.failed(String.format("你没有权限修改id为 %s 的污水处理厂的数据", sewageTreatmentPlant.getId()));
            }
        } else {
            return R.failed(String.format("没有找到id为 %s 的污水处理厂", sewageTreatmentPlant.getId()));
        }
    }

    /**
     * 通过id删除污水处理厂
     *
     * @param id id
     * @return R
     */
    @ApiOperation(value = "通过id删除污水处理厂", notes = "通过id删除污水处理厂")
    @SysLog("通过id删除污水处理厂")
    @DeleteMapping("/{id}")
    @PreAuthorize("@pms.hasPermission('SewageTreatmentPlantDelete')")
    public R deleteById(@PathVariable String id, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
        final SysOrg userOrg;    // 定义成 final才可以被后面的 Lambda 表达式使用
        R r = remoteOrgService.getById(dlpUser.getOrgId());
        if (r.getCode() == CommonConstants.SUCCESS) {
            userOrg = (SysOrg) r.getData();
        } else {
            return R.failed(String.format("没有找到 orgId 为 %s 的机构, 请确认用户所属机构的正确性!", dlpUser.getOrgId()));
        }

        SewageTreatmentPlant oldSewageTreatmentPlant = sewageTreatmentPlantService.getById(id);
        if (oldSewageTreatmentPlant != null) {
            if (dlpUser.isStaff() ||
                    StrUtil.equalsIgnoreCase(userOrg.getAreaCode(), oldSewageTreatmentPlant.getProvinceCode()) ||
                    StrUtil.equalsIgnoreCase(userOrg.getAreaCode(), oldSewageTreatmentPlant.getCityCode()) ||
                    StrUtil.equalsIgnoreCase(userOrg.getAreaCode(), oldSewageTreatmentPlant.getDistrictCode())
            ) {
                if (sewageTreatmentPlantService.removeById(id)) {
                    return R.ok(oldSewageTreatmentPlant, "污水处理厂删除成功");
                } else {
                    return R.failed(oldSewageTreatmentPlant, "污水处理厂删除失败");
                }
            } else {
                return R.failed(String.format("你没有权限删除id为 %s 的污水处理厂", id));
            }
        } else {
            return R.failed(String.format("没有找到id为 %s 的污水处理厂", id));
        }

    }

    //获取污水厂的树形结构
    @ApiOperation(value = "获取污水厂的树形结构", notes = "获取污水厂的树形结构")
    @GetMapping("/getSewageTreatmentTreeData")
    public R getSewageTreatmentTreeData(HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
        R r = remoteOrgService.getById(dlpUser.getOrgId());
        String userAreaCode = "";
        if (r.getCode() == CommonConstants.SUCCESS) {
            userAreaCode = ((SysOrg) r.getData()).getAreaCode();
        } else {
            return R.failed(String.format("没有找到 orgId 为 %s 的机构, 请确认用户所属机构的正确性!", dlpUser.getOrgId()));
        }
        return R.ok(sewageTreatmentPlantService.getSewageTreatmentTreeList(userAreaCode), "获取数据成功");
    }

    @ApiOperation(value = "获取污水厂省市县3级联动数据", notes = "获取污水厂省市县3级联动数据")
    @GetMapping("/getThreeAreaTreeData")
    public R getThreeAreaTreeData() {
        return R.ok(sewageTreatmentPlantService.getThreeAreaTreeData());
    }

    //=====================================================================================
    // 测试, 创建 100 个污水处理厂
    @GetMapping("/create100")
    public R xxx_TestCreate100() {
        String provinceCode = "610000";
        for (int i = 0; i < 100; i++) {
            SewageTreatmentPlant stp = new SewageTreatmentPlant();
            stp.setId(IdWorker.get32UUID().toUpperCase());
            stp.setName("第" + i + "个污水处理厂");
            stp.setShortName(i + "污水厂");
            stp.setAddress(RandomUtil.randomString(100));
            stp.setCoverageArea("全部城区");
            stp.setProvinceCode(provinceCode);
            List<Area> citis = remoteAreaService.getCitys(provinceCode).getData();
            Area city = RandomUtil.randomEle(citis);
            stp.setCityCode(city.getId());
            List<Area> districts = remoteAreaService.getDistricts(city.getId()).getData();
            Area district = RandomUtil.randomEle(districts);
            stp.setDistrictCode(district.getId());
            stp.setProvinceContacts(TestUtils.genPersonName() + " " + TestUtils.genMobile());
            stp.setCityContacts(TestUtils.genPersonName() + " " + TestUtils.genMobile());
            stp.setDistrictContacts(TestUtils.genPersonName() + " " + TestUtils.genMobile());
            stp.setWaterInletCount(RandomUtil.randomInt(1, 5));
            stp.setDailyFlowDesign(Math.round(100 * RandomUtil.randomDouble(10, 200)) / 100.0);
            stp.setDailyFlowReal(Math.round(100 * RandomUtil.randomDouble(10, 200)) / 100.0);
            stp.setDomesticSewageProportion(Math.round(100 * RandomUtil.randomDouble(50, 100)) / 100.0);
            stp.setServicePopulation(Math.round(100 * RandomUtil.randomDouble(100, 200)) / 100.0);
            stp.setComments("这是第" + i + "个污水处理厂的备注");
            //stp.setUpdateLog();

            sewageTreatmentPlantService.save(stp);
        }
        return R.ok("100 个污水处理厂已成功创建.");
    }

}
