package digital.laboratory.platform.sewage.service.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.common.core.constant.CommonConstants;
import digital.laboratory.platform.common.feign.RemoteTemplate2htmlService;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.sewage.dto.SewageJobIdentificationMaterialDTO;
import digital.laboratory.platform.sewage.service.SewageJobIdentificationMaterialService;
import digital.laboratory.platform.sewage.service.SewageTreatmentPlantService;
import digital.laboratory.platform.sewage.entity.SewageJobItem;
import digital.laboratory.platform.sewage.handler.AppStartupRunner;
import digital.laboratory.platform.sewage.mapper.SewageJobIdentificationMaterialMapper;
import digital.laboratory.platform.sewage.service.SewageJobItemService;
import digital.laboratory.platform.sewage.service.SewageJobService;
import digital.laboratory.platform.sewage.entity.SewageJob;
import digital.laboratory.platform.sewage.entity.SewageJobIdentificationMaterial;
import digital.laboratory.platform.sewage.entity.SewageTreatmentPlant;
import digital.laboratory.platform.sys.entity.SysOrg;
import digital.laboratory.platform.sewage.vo.SewageJobIdentificationMaterialVO;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 污水任务的检材信息服务实现类
 *
 * @author Zhang Xiaolong created at  2022-07-26
 * @describe 污水任务的检材信息 服务实现类
 */
@RequiredArgsConstructor
@Service
public class SewageJobIdentificationMaterialServiceImpl extends ServiceImpl<SewageJobIdentificationMaterialMapper, SewageJobIdentificationMaterial> implements SewageJobIdentificationMaterialService {
    @Resource
    private SewageJobService sewageJobService;
    @Resource
    private SewageJobItemService sewageJobItemService;
    @Resource
    private SewageTreatmentPlantService sewageTreatmentPlantService;

    private final RemoteTemplate2htmlService remoteTemplate2htmlService;


    /**
     * 取新的检材编号
     * 前缀为 WJ, 加任务号后7位, 加6位 污水厂地区码, 加2位顺序号
     * 格式为: "WJ" + jobYear + jobSeason + "nn" + AreaCode + "nn"
     * 例如: WJ2022201 520111 001
     *
     * @param jobId
     * @param jobItemId
     * @return
     */
    @Override
    public String getNewCode(String jobId, String jobItemId) {
        SewageJob sj = sewageJobService.getById(jobId);
        if (sj == null) {
            throw new RuntimeException("这个检材对应的污水任务没有查询到");
        }
        String jobNo = sj.getJobNo();
        if (StrUtil.isBlank(jobNo)) {
            throw new RuntimeException("这个检材对应的污水任务的任务编号为空, 无法为检材生成编号");
        }
        String prefix = CommonConstants.CODE_PREFIX_SEWAGE_IDENTIFICATION_MATERIAL + StrUtil.removePrefixIgnoreCase(jobNo, CommonConstants.CODE_PREFIX_SEWAGE_JOB);
        //-- 到这里, prefix 中包含 WJ + 任务号后7位


        SewageJobItem sji = sewageJobItemService.getById(jobItemId);
        if (sji == null) {
            throw new RuntimeException("这个检材对应的污水任务清单项没有查询到");
        }
        SewageTreatmentPlant stp = sewageTreatmentPlantService.getById(sji.getPlantId());
        if (stp == null) {
            throw new RuntimeException("这个检材对应的污水任务清单项的污水处理厂没有查询到");
        }

        prefix = prefix + stp.getDistrictCode();    // 前缀 = "WJ" + <纯任务号> + 地区码

        List<SewageJobIdentificationMaterial> list = this.list(Wrappers.<SewageJobIdentificationMaterial>query()
                .likeRight("im_no", prefix)        // LIKE '值%'
                .orderByDesc("im_no")
        );

        int newNo = 1;
        if ((list != null) && (list.size() > 0)) {
            SewageJobIdentificationMaterial sjimMax = list.get(0);
            String strMaxNo = StrUtil.removePrefixIgnoreCase(sjimMax.getImNo(), prefix);
            try {
                int maxno = Integer.parseUnsignedInt(strMaxNo);
                newNo = maxno + 1;
            } catch (NumberFormatException e) {
                // 如果后缀有非数字, 则无视之, 重头编码
                newNo = 1;
            }
        }

        String newCode = prefix + String.format("%02d", newNo);

        return newCode;
    }

    @Override
    public SewageJobIdentificationMaterialVO getVOById(String id) {
        List<SewageJobIdentificationMaterialVO> list = this.getSewageJobIdentificationMaterialVOList(Wrappers.<SewageJobIdentificationMaterial>query()
                .eq("sjim.id", id)
        );
        if (list.size() == 1) {
            return list.get(0);
        } else if (list.size() == 0) {
            throw new RuntimeException(String.format("没有找到指定的污水检材, id=%s", id));
        } else {
            throw new RuntimeException(String.format("找到多个 id 相同的污水检材, id=%s", id));
        }
    }

    @Override
    public IPage<SewageJobIdentificationMaterialVO> getSewageJobIdentificationMaterialVOPage(IPage<SewageJobIdentificationMaterial> page, QueryWrapper<SewageJobIdentificationMaterial> qw) {
        IPage<SewageJobIdentificationMaterialVO> r = baseMapper.getSewageJobIdentificationMaterialVOPage(page, qw);
        return r;
    }

    @Override
    public List<SewageJobIdentificationMaterialVO> getSewageJobIdentificationMaterialVOList(QueryWrapper<SewageJobIdentificationMaterial> qw) {
        List<SewageJobIdentificationMaterialVO> list = baseMapper.getSewageJobIdentificationMaterialVOList(qw);
        return list;
    }

    @Override
    public IPage<SewageJobIdentificationMaterialVO> getSewageJobGABIdentificationMaterialVOPage(IPage<SewageJobIdentificationMaterial> page, QueryWrapper<SewageJobIdentificationMaterial> qw) {
        IPage<SewageJobIdentificationMaterialVO> r = baseMapper.getSewageJobGABIdentificationMaterialVOPage(page, qw);
        return r;
    }

    @Override
    public List<SewageJobIdentificationMaterialVO> getSewageJobGABIdentificationMaterialVOList(QueryWrapper<SewageJobIdentificationMaterial> qw) {
        List<SewageJobIdentificationMaterialVO> list = baseMapper.getSewageJobGABIdentificationMaterialVOList(qw);
        return list;
    }

    @Override
    public String buildSewageSampleLableContent(String id, String sampleNo) throws Exception {
        SewageJobIdentificationMaterial sewageJobIdentificationMaterial = this.getById(id);
        if (sewageJobIdentificationMaterial == null) {
            throw new RuntimeException(String.format("没有找到 id 为 %s 的检材", id));
        }
        Map<String, Object> data = new HashMap<>();
        data.put("sewageJobIdentificationMaterial", sewageJobIdentificationMaterial);
        if (StrUtil.isNotBlank(sampleNo)) {
            if (StrUtil.equals(sampleNo, "1")) {
                data.put("sampleNo", sewageJobIdentificationMaterial.getSample1No());
            }
            if (StrUtil.equals(sampleNo, "2")) {
                data.put("sampleNo", sewageJobIdentificationMaterial.getSample2No());
            }
            String templateFileName = AppStartupRunner.getCfg(CommonConstants.DLP_CODE_SEWAGE_JOB_IDENTIFICATION_MATERIAL_SAMPLE_LABEL);
            return remoteTemplate2htmlService.getHtml(templateFileName, data);
        } else {
            String templateFileName = AppStartupRunner.getCfg(CommonConstants.DLP_CODE_SEWAGE_JOB_IDENTIFICATION_MATERIAL_LABEL);
            return remoteTemplate2htmlService.getHtml(templateFileName, data);
        }
    }


    @Override
    public List<SewageJobIdentificationMaterial> getSewageJobIdentificationMaterialList(List<String> ids) {
        return this.list(Wrappers.<SewageJobIdentificationMaterial>query().in("id", ids));
    }

    /**
     * 批量送检污水检材
     *
     * @param materialDTO
     * @return
     */
    @Override
    public List<SewageJobIdentificationMaterial> batchSendMaterial(SewageJobIdentificationMaterialDTO materialDTO/*,int userLv,DLPUser dlpUser,SysOrg userOrg*/) {
        List<String> materialIdList = materialDTO.getIds();
        String expressNumber = materialDTO.getExpressNumber();
        String expressCompany = materialDTO.getExpressCompany();
        String sendType = materialDTO.getDeliverMethod();
        List<SewageJobIdentificationMaterial> materialList = this.list(Wrappers.<SewageJobIdentificationMaterial>query()
                .in("id", materialIdList));
        materialList.forEach(item -> {
            //StringUtils.isNotBlank(checkOptOk(userLv,item,dlpUser,));
            item.setStatus(2);
            item.setDeliverMethod(sendType);
            if (sendType.equals("2")) {
                //如果送检方式是快递，则填写快递公司和快递单号
                item.setExpressCompany(expressCompany);
                item.setExpressNumber(expressNumber);
            }
            if (sendType.equals("1")) {
                //则需要填写送检人信息
                item.setDelivererName(materialDTO.getDelivererName());
                item.setDelivererPhone(materialDTO.getDelivererPhone());
                item.setDelivererCert(materialDTO.getDelivererCert());
                item.setDelivererId(materialDTO.getDelivererId());
            }
        });
        if (this.updateBatchById(materialList)) {
            return materialList;
        } else {
            return null;
        }
    }

    //作废污水检材
    @Override
    public List<SewageJobIdentificationMaterial> cancellationMaterial(List<String> materialIdList, String causeOfCancellation) {
        //
        List<SewageJobIdentificationMaterial> materialBeanList = this.list(Wrappers.<SewageJobIdentificationMaterial>query()
                .in("id", materialIdList));
        materialBeanList.forEach(item -> {
            item.setStatus(-2);
            item.setCauseOfCancellation(causeOfCancellation);
        });
        this.updateBatchById(materialBeanList);
        return materialBeanList;
    }

    //获取送检联系人信息
    @Override
    public String getContactsInfo() {

        return null;
    }

    @Override
    public int getSewageMaterialNoSignCount(List<Integer> materialStatus) {
        return this.list(Wrappers.<SewageJobIdentificationMaterial>query()
                .in("status", materialStatus)).size();
    }

    //判断是否有符合操作
    private String checkOptOk(int userLevel, SewageJobIdentificationMaterial sewageJobIdentificationMaterial,
                              DLPUser dlpUser, SewageTreatmentPlant stp, SysOrg userOrg) {
        if (userLevel == 3) {
            // 区县一级的机构码必须完全一致
            if (!StrUtil.equals(sewageJobIdentificationMaterial.getDeliverOrgId(), dlpUser.getOrgId())) {
                return String.format("你没有权限修改这个检材, id=%s", sewageJobIdentificationMaterial.getId());
            }
        } else if (userLevel == 2) {
            // 市一级要确认是送检机构的上级
            if (!StrUtil.equalsIgnoreCase(stp.getCityCode(), userOrg.getAreaCode())) {
                return String.format("你没有权限修改这个检材, id=%s", sewageJobIdentificationMaterial.getId());
            }
        } else if (userLevel == 1) {
            // 省一级要确认是送检机构的上级的上级
            if (!StrUtil.equalsIgnoreCase(stp.getProvinceCode(), userOrg.getAreaCode())) {
                return String.format("你没有权限修改这个检材, id=%s", sewageJobIdentificationMaterial.getId());
            }
        }
        return "";
    }

    @Override//通过样品编号查询污水检材信息
    public SewageJobIdentificationMaterial getByImNo(String imNo) {
        List<SewageJobIdentificationMaterialVO> list = new ArrayList<>();
        //公安部污水检材的编号是以H开头的，所以我们要分开查询
        if (imNo.startsWith("H")) {
            list = baseMapper.getByImNoGAB(imNo);
        } else {
            list = baseMapper.getByImNo(imNo);
        }
        if (list.size() == 1) {
            return list.get(0);
        } else return null;
    }
}


