package digital.laboratory.platform.sewage.service.impl;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.common.core.constant.CommonConstants;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.sewage.constant.SewageConstants;
import digital.laboratory.platform.sewage.dto.StatisticsDiffStatusJobDTO;
import digital.laboratory.platform.sewage.entity.SewageJob;
import digital.laboratory.platform.sewage.entity.SewageJobIdentificationMaterial;
import digital.laboratory.platform.sewage.entity.SewageTreatmentPlant;
import digital.laboratory.platform.sewage.entity.SewageJobItem;
import digital.laboratory.platform.sewage.enums.SewageJobStatusConstants;
import digital.laboratory.platform.sewage.handler.AppStartupRunner;
import digital.laboratory.platform.sewage.mapper.SewageJobIdentificationMaterialMapper;
import digital.laboratory.platform.sewage.mapper.SewageJobMapper;
import digital.laboratory.platform.sewage.service.SewageJobIdentificationMaterialService;
import digital.laboratory.platform.sewage.service.SewageJobItemService;
import digital.laboratory.platform.sewage.service.SewageJobService;
import digital.laboratory.platform.sewage.service.SewageTreatmentPlantService;
import digital.laboratory.platform.sewage.vo.SewageMaterialVo;
import digital.laboratory.platform.sys.entity.DrugLite;
import digital.laboratory.platform.sys.entity.SysOrg;
import digital.laboratory.platform.sys.entity.entrustment.IMAdditionalProperty;
import digital.laboratory.platform.sewage.vo.SewageJobVO;
import digital.laboratory.platform.sewage.vo.SewageTreeVO;
import digital.laboratory.platform.sewage.vo.TreeVo;
import digital.laboratory.platform.sys.vo.entrustment.MarkersVO;
import digital.laboratory.platform.sys.entity.entrustment.Sample;
import digital.laboratory.platform.sys.feign.RemoteOrgService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.time.temporal.TemporalAdjusters;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 污水检测任务服务实现类
 *
 * @author Zhang Xiaolong created at  2022-07-26
 * @describe 污水检测任务 服务实现类
 */
@Service
public class SewageJobServiceImpl extends ServiceImpl<SewageJobMapper, SewageJob> implements SewageJobService {

    @Resource
    SewageJobIdentificationMaterialService sewageJobIdentificationMaterialService;
    @Resource
    SewageJobItemService sewageJobItemService;
    @Resource
    SewageTreatmentPlantService sewageTreatmentPlantService;

    @Resource
    RemoteOrgService remoteOrgService;

    @Resource
    SewageJobIdentificationMaterialMapper sewageJobIdentificationMaterialMapper;

    /**
     * 取新的任务号
     * 前缀为 WS, 加4位 年份, 加1位季度, 加 2 位序号
     * 格式为: "WS" + jobYear + jobSeason + "nn"
     * 例如: WS2022201
     *
     * @param jobYear
     * @param jobSeason
     * @return
     */
    @Override
    public String getNewJobNo(int jobYear, int jobSeason) {
        String prefix = CommonConstants.CODE_PREFIX_SEWAGE_JOB + String.format("%d%d", jobYear, jobSeason);

        List<SewageJob> list = this.list(Wrappers.<SewageJob>query()
                .likeRight("job_no", prefix)        // LIKE '值%'
                .orderByDesc("job_no")
        );

        int newNo = 1;
        if ((list != null) && (list.size() > 0)) {
            SewageJob sj = list.get(0);
            String strMaxNo = StrUtil.removePrefixIgnoreCase(sj.getJobNo(), prefix);
            try {
                int maxno = Integer.parseUnsignedInt(strMaxNo);
                newNo = maxno + 1;
            } catch (NumberFormatException e) {
                // 如果后缀有非数字, 则无视之, 重头编码
                newNo = 1;
            }
        }

        String newCode = prefix + String.format("%02d", newNo);

        return newCode;
    }

    @Override
    public String getNewSampleNo(String IdentificationMaterialNo, int sampleSerialNumber) {
        String pureIMNo = StrUtil.removePrefixIgnoreCase(IdentificationMaterialNo, CommonConstants.CODE_PREFIX_IDENTIFICATION_MATERIAL);	// 去掉司法鉴定委托检材的前缀 JC
        pureIMNo = StrUtil.removePrefixIgnoreCase(pureIMNo, CommonConstants.CODE_PREFIX_SEWAGE_IDENTIFICATION_MATERIAL);	// 去掉污水任务检材的前缀 WJ

        String prefix = CommonConstants.CODE_PREFIX_SAMPLE + pureIMNo;
        String newCode = prefix + String.format("%d", sampleSerialNumber);
        return newCode;
    }

    //撤销任务
    @Transactional(rollbackFor = {RuntimeException.class, Exception.class})
    @Override
    public Boolean undoTask(String taskID) {
        SewageJob sewageJob = this.getById(taskID);
        //检测是否符合撤销退条件，不福很直接返回
        //如果符合，则先删除生成的检材，再改变任务的状态
        List<SewageJobIdentificationMaterial> materialList =
                sewageJobIdentificationMaterialService.list(Wrappers.<SewageJobIdentificationMaterial>query()
                        .eq("job_id", taskID)
                        .gt("status", 0));
        if (materialList.size() > 0) {
            System.out.println("该任务下的检材已经被处理，无法撤回任务");
            return false;
        }
        //删除检材//删除清单项
        delMaterialAndTaskItem(taskID);
        //更新
        sewageJob.setStatus(0);
        return this.updateById(sewageJob);
    }

    //删除一个任务下的检材信息和任务清单项信息
    private Boolean delMaterialAndTaskItem(String taskID) {
        List<SewageJobItem> jobItemList = sewageJobItemService.list(Wrappers.<SewageJobItem>query()
                .eq("job_id", taskID));
        List<SewageJobIdentificationMaterial> materialList =
                sewageJobIdentificationMaterialService.list(Wrappers.<SewageJobIdentificationMaterial>query()
                        .eq("job_id", taskID));
        if (jobItemList.size() > 0) {
            System.out.println("删除清单项:" + jobItemList.size() + "条");
            sewageJobItemService.removeBatchByIds(jobItemList);
        }
        if (materialList.size() > 0) {
            System.out.println("删除检材:" + jobItemList.size() + "条");
            sewageJobIdentificationMaterialService.removeBatchByIds(materialList);
        }
        return true;
    }

    @Override
    public IPage<SewageJobVO> getSewageJobVOPage(IPage<SewageJob> page, QueryWrapper<SewageJob> qw) {
        IPage<SewageJobVO> r = baseMapper.getSewageJobVOPage(page, qw);
        return r;
    }

    @Override
    public List<SewageJobVO> getSewageJobVOList(QueryWrapper<SewageJob> qw) {
        List<SewageJobVO> list = baseMapper.getSewageJobVOList(qw);
        return list;
    }
    ///////////////////////*******////////////////////////////

    @Override
    public Boolean generateSewageMaterial(SewageJob sewageJob) {
        //查询该任务下的污水厂
        List<SewageJobItem> taskItemList = sewageJobItemService.list(Wrappers.<SewageJobItem>query()
                .eq("job_id", sewageJob.getId()));

        return genSewageMaterialList(taskItemList, sewageJob);
    }

    @Override
    public List<TreeVo> querySewageJcSendData(String jobId, String areaCode, String materialStatus, Integer treeLayer) {
        int treeLv = 0;
        QueryWrapper<SewageTreeVO> updateWrapper = Wrappers.<SewageTreeVO>query();
        updateWrapper.eq(StrUtil.isNotBlank(jobId), "job_id", jobId);
        updateWrapper.eq(StrUtil.isNotBlank(materialStatus), "jc_status", materialStatus);
        if (StrUtil.isNotBlank(areaCode) && !areaCode.equals("0")) {
            if (areaCode.endsWith("0000")) {
                updateWrapper.eq("province_code", areaCode);//省
                treeLv = 1;
            } else if (areaCode.endsWith("00")) {
                updateWrapper.eq("city_code", areaCode);//市
                treeLv = 2;
            } else if (!areaCode.endsWith("00")) {
                updateWrapper.eq("district_code", areaCode);//市
                treeLv = 3;
            }
        }
        List<SewageTreeVO> sewageTreeVOList = this.baseMapper.getSewageSendData(updateWrapper);

        List<TreeVo> retTreeData = new ArrayList<>();
        retTreeData.add(buildTreeDataByList(sewageTreeVOList, treeLv, treeLayer));
        return retTreeData;
    }

    @Override
    public List<SewageTreeVO> querySewageJcSendData(String areaCode, int lv) {

        QueryWrapper<SewageTreeVO> queryWrapper = Wrappers.<SewageTreeVO>query();
        if (lv == 1) {
            queryWrapper.eq("province_code", areaCode);
        } else if (lv == 2) {
            queryWrapper.eq("city_code", areaCode);
        } else if (lv == 3) {
            queryWrapper.eq("district_code", areaCode);
        }
        List<SewageTreeVO> sewageTreeVOList = this.baseMapper.getSewageSendData(queryWrapper);
        return sewageTreeVOList;
    }

    List<TreeVo> buildTreeVo(Map<String, SewageTreeVO> sourceMap, int lvNo) {
        List<TreeVo> treeVoList = new ArrayList<TreeVo>();
        sourceMap.forEach((k, v) -> {
            TreeVo treeVo = new TreeVo();
            treeVo.setId(k);
            treeVo.setDataObj(v);
            if (lvNo == 1) {
                treeVo.setLabel(v.getProvinceName());
            } else if (lvNo == 2) {
                treeVo.setLabel(v.getCityName());
            } else if (lvNo == 3) {
                treeVo.setLabel(v.getDistrictName());
            } else if (lvNo == 4) {
                if (v.getWaterFactoryName() != null) {
                    treeVo.setLabel(v.getWaterFactoryName());
                }
            } else {
                treeVo.setLabel(v.getJcName());
            }
            //表示没有污水厂，没有污水厂，我们就不加这条数据
            if (v.getWaterFactoryName() != null) {
                treeVoList.add(treeVo);
            } else {
                System.out.println("--------------------------污水厂为空，不添加---------------------");
            }
        });
        return treeVoList;
    }

    List<Integer> getTreeNodeJcCount(List<TreeVo> childTreeNodeList) {
        List<Integer> retList = new ArrayList<Integer>();
        int jcAllCount = 0;//总数
        int noFillOut = 0;//未填报
        int noSendCount = 0;//未送检
        int noAcceptCount = 0;//未签收
        int cancelledCount = 0;//作废
        for (TreeVo treevo : childTreeNodeList) {
            jcAllCount = jcAllCount + treevo.getJcCount();
            noFillOut = noFillOut + treevo.getNoFillOut();
            noSendCount = noSendCount + treevo.getNoSendCount();
            noAcceptCount = noAcceptCount + treevo.getNoAcceptCount();
            cancelledCount = cancelledCount + treevo.getCancelledCount();
        }
        retList.add(jcAllCount);
        retList.add(noFillOut);
        retList.add(noSendCount);
        retList.add(noAcceptCount);
        retList.add(cancelledCount);
        return retList;
    }
    //将list转换成树形结构

    /**
     * @param sourceDataList
     * @param treeLv         判断树的根节点，比如treeLv=0,那么根节点就是国家毒品实验室陕西分中心，比如=1，就是省级根
     * @param treeLayer      这个参数表示树的层数，比如是到污水厂，还是只到县
     * @return
     */
    TreeVo buildTreeDataByList(List<SewageTreeVO> sourceDataList, int treeLv, int treeLayer) {
        //int treeLv=0;
        //int treeLayer=0;
        //根据污水厂的省，市，县
        Map<String, SewageTreeVO> provinceCodeMap = new HashMap<String, SewageTreeVO>();
        Map<String, SewageTreeVO> cityCodeMap = new HashMap<String, SewageTreeVO>();
        Map<String, SewageTreeVO> districtCodeMap = new HashMap<String, SewageTreeVO>();
        Map<String, SewageTreeVO> waterFactoryMap = new HashMap<String, SewageTreeVO>();
        Map<String, SewageTreeVO> jcMap = new HashMap<String, SewageTreeVO>();
        //树的各个层的节点List
        List<TreeVo> baseTreeList = new ArrayList<>();
        List<TreeVo> secondTreeList = new ArrayList<>();
        List<TreeVo> thirdTreeList = new ArrayList<>();
        List<TreeVo> fourthTreeList = new ArrayList<>();
        List<TreeVo> fifthTreeList = new ArrayList<>();
        //将取到的数据进行分解，并将分解到树的各层中去
        for (SewageTreeVO sewageTreeObj : sourceDataList) {
            String provinceCode = sewageTreeObj.getProvinceCode();
            String cityCode = sewageTreeObj.getCityCode();
            String districtCode = sewageTreeObj.getDistrictCode();
            //
            provinceCodeMap.put(provinceCode, sewageTreeObj);//查询有多少个省
            cityCodeMap.put(cityCode, sewageTreeObj);//查询有多少个市
            districtCodeMap.put(districtCode, sewageTreeObj);//查询有多少个县

            waterFactoryMap.put(sewageTreeObj.getWaterFactoryId(), sewageTreeObj);//污水厂
            jcMap.put(sewageTreeObj.getJcId(), sewageTreeObj);
        }
        //构建省的子节点，就是市
        baseTreeList = buildTreeVo(provinceCodeMap, 1);
        secondTreeList = buildTreeVo(cityCodeMap, 2);
        thirdTreeList = buildTreeVo(districtCodeMap, 3);
        fourthTreeList = buildTreeVo(waterFactoryMap, 4);
        fifthTreeList = buildTreeVo(jcMap, 5);
        //遍历四级节点，找出第四层节点的孩子
        for (TreeVo fourthTree : fourthTreeList) {
            //获取四级节点前4位
            String factoryCode = fourthTree.getId();
            //遍历5级节点，找出污水厂ID相同的
            List<TreeVo> tempList1 = fifthTreeList.stream().filter(e -> e.getDataObj().getWaterFactoryId().equals(factoryCode)).collect(Collectors.toList());
            List<TreeVo> tempList2 = fifthTreeList.stream().filter(e -> e.getDataObj().getWaterFactoryId().equals(factoryCode) && e.getDataObj().getJcStatus() == 0).collect(Collectors.toList());//
            List<TreeVo> tempList3 = fifthTreeList.stream().filter(e -> e.getDataObj().getWaterFactoryId().equals(factoryCode) && e.getDataObj().getJcStatus() == 1).collect(Collectors.toList());
            List<TreeVo> tempList4 = fifthTreeList.stream().filter(e -> e.getDataObj().getWaterFactoryId().equals(factoryCode) && e.getDataObj().getJcStatus() == 2).collect(Collectors.toList());
            List<TreeVo> tempList5 = fifthTreeList.stream().filter(e -> e.getDataObj().getWaterFactoryId().equals(factoryCode) && e.getDataObj().getJcStatus() == -2).collect(Collectors.toList());
            //设置4级节点的孩子和其他属性


            fourthTree.setJcCount(tempList1.size());
            fourthTree.setNoFillOut(tempList2.size());//未填报
            fourthTree.setNoSendCount(tempList3.size());
            fourthTree.setNoAcceptCount(tempList4.size());
            fourthTree.setCancelledCount(tempList5.size());
            if (treeLayer >= 4) {
                fourthTree.setChildren(tempList1);
            }
        }

        //遍历三级节点，找出第3层节点的孩子
        for (TreeVo threeTree : thirdTreeList) {
            //获取3级节点
            String districtCode = threeTree.getId();
            //遍历4级节点，找出县级编码相同的污水厂
            List<TreeVo> tempList = fourthTreeList.stream().filter(e -> e.getDataObj().getDistrictCode().equals(districtCode)).collect(Collectors.toList());
            List<Integer> retListCount = getTreeNodeJcCount(tempList);
            threeTree.setTreeNodeStatics(retListCount);
            //设置3级节点的孩子
            if (treeLayer >= 4) {
                threeTree.setChildren(tempList);
            }
        }
        //遍历二级节点，找出第2层节点的孩子
        for (TreeVo secondTree : secondTreeList) {
            //获取二级节点前4位
            String secondTree0_4 = secondTree.getId().substring(0, 4);
            //遍历3级节点，找出前4位与2级节点相同的
            List<TreeVo> secondChildList = thirdTreeList.stream().filter(e -> e.getId().substring(0, 4).equals(secondTree0_4)).collect(Collectors.toList());
            List<Integer> secondTreeCount = getTreeNodeJcCount(secondChildList);
            secondTree.setTreeNodeStatics(secondTreeCount);
            //设置二级节点的三级节点
            secondTree.setChildren(secondChildList);
        }
        //遍历根节点,找出根节点的孩子
        for (TreeVo baseTree : baseTreeList) {
            //获取根节点的前两位
            String baseTree0_2 = baseTree.getId().substring(0, 2);
            //遍历2级节点，找出前两位与根节点相同的
            List<TreeVo> tempList = secondTreeList.stream().filter(e -> e.getId().substring(0, 2).equals(baseTree0_2))
                    .collect(Collectors.toList());
            List<Integer> baseTreeCount = getTreeNodeJcCount(tempList);
            baseTree.setTreeNodeStatics(baseTreeCount);
            //设置根节点的二级节点
            baseTree.setChildren(tempList);
        }
        //以上已经构建好了整个树的数据，我们根据需要选择不同的根来返回树，
        // 如果是分中心看，那就是分中心，如果是省看，根就是省，如果是市看
        //根就是市，以此类推来返回不同的树根
        TreeVo parentTreeNode = new TreeVo();
        //等于0，表示分中心
        if (treeLv == 0) {
            parentTreeNode.setLabel(SewageConstants.GY_DRUG_INSPECT_CENTER);
            parentTreeNode.setId("99999");
            parentTreeNode.setChildren(baseTreeList);
            parentTreeNode.setTreeNodeStatics(getTreeNodeJcCount(baseTreeList));
        } else if (treeLv == 1) {
            parentTreeNode = baseTreeList.get(0);
        } else if (treeLv == 2) {
            parentTreeNode = secondTreeList.get(0);
        } else if (treeLv == 3) {
            parentTreeNode = thirdTreeList.get(0);
        }

        return parentTreeNode;
    }

    //生成检材的操作----------------
    private Boolean genSewageMaterialList(List<SewageJobItem> taskItemList, SewageJob sewageJob) {

        Map<String, SewageJobItem> mapStpItem = new HashMap<>();
        taskItemList.forEach(item -> {
            mapStpItem.put(item.getPlantId(), item);
        });

        List<SewageTreatmentPlant> stpObjBeanList = new ArrayList<>();
        List<String> stpsId = new ArrayList<>();

        mapStpItem.forEach((k, v) -> {
            stpsId.add(k);
        });
        if (stpsId.size() >= 0) {
            //获取污水厂
            stpObjBeanList = sewageTreatmentPlantService.list(Wrappers.<SewageTreatmentPlant>query()
                    .in("id", stpsId));
        } else {
            throw new RuntimeException(String.format("任务项为0，不能发布任务"));
        }
        List<SewageJobIdentificationMaterial> sewageMatialBeanList = new ArrayList<>();
        stpObjBeanList.forEach(item -> {
            List<SewageMaterialVo> sewageMaterialNoList = GenMaterialNo(item, sewageJob);//一个污水厂对应的检材总数
            String taskItemId = mapStpItem.get(item.getId()).getId();
            //拿到每个污水厂生成的编号后，将编号转化为检材对象，方便保存
            sewageMaterialNoList.forEach(materialVo -> {
                SewageJobIdentificationMaterial eg = new SewageJobIdentificationMaterial();
                eg.setId(IdWorker.get32UUID());
                eg.setImNo(materialVo.getMaterialNo());
                eg.setJobId(sewageJob.getId());
                eg.setJobItemId(taskItemId);
                eg.setSampleName(materialVo.getMaterialName());
                eg.setSampleType(Integer.parseInt(materialVo.getMaterialType()));
                sewageMatialBeanList.add(eg);
            });
        });
        System.out.println("本次共创建" + sewageMatialBeanList.size() + "份检材");
        //持久化到数据库
        return sewageJobIdentificationMaterialService.saveBatch(sewageMatialBeanList);
    }

    /**
     * 生成检材的编号
     *
     * @param sewageTreatmentPlant
     * @param sewageJob
     * @return
     */
    public List<SewageMaterialVo> GenMaterialNo(SewageTreatmentPlant sewageTreatmentPlant, SewageJob sewageJob) {
        List<SewageMaterialVo> ImNoList = new ArrayList<SewageMaterialVo>();
        //如果进水口=1，那么规则就是年份+月份+污水厂编号+A/B
        if (sewageTreatmentPlant.getWaterInletCount() <= 1) {
            String No1 = sewageJob.getJobYear() + getQuarterNumber(sewageJob.getJobSeason()) + sewageTreatmentPlant.getNumber();
        // 2024-4-15 现在送检只会送检一份工作日与节假日的混合样本了
        // String No2 = sewageJob.getJobYear() + getQuarterNumber(sewageJob.getJobSeason()) + sewageTreatmentPlant.getNumber() + "B";
            SewageMaterialVo eg1 = getSewageMaterialVo(sewageTreatmentPlant, No1, "混合样本");
            eg1.setMaterialType("1");
//            SewageMaterialVo eg2 = getSewageMaterialVo(sewageTreatmentPlant, No2, "节假日样本");
//            eg2.setMaterialType("2");
            ImNoList.add(eg1);
//            ImNoList.add(eg2);

        } else {
            //如果进水口>=1，那么规则就是年份+月份+污水厂编号+序号+A/B
            for (int j = 1; j < sewageTreatmentPlant.getWaterInletCount(); j++) {
                String No1 = sewageJob.getJobYear() + getQuarterNumber(sewageJob.getJobSeason()) + sewageTreatmentPlant.getNumber() + j;
                //String No2 = sewageJob.getJobYear() + getQuarterNumber(sewageJob.getJobSeason()) + sewageTreatmentPlant.getNumber() + j + "B";
                SewageMaterialVo eg1 = getSewageMaterialVo(sewageTreatmentPlant, No1, "混合样本");
                eg1.setMaterialType("1");
                // 2024-4-15 现在送检只会送检一份工作日与节假日的混合样本了
                //SewageMaterialVo eg2 = getSewageMaterialVo(sewageTreatmentPlant, No2, "节假日样本");
                //eg2.setMaterialType("2");
                ImNoList.add(eg1);
                // ImNoList.add(eg2);
            }
        }
        return ImNoList;
    }

    private static SewageMaterialVo getSewageMaterialVo(SewageTreatmentPlant sewageTreatmentPlant, String No, String type) {
        SewageMaterialVo eg1 = new SewageMaterialVo();
        eg1.setMaterialNo(No);
        eg1.setMaterialName(sewageTreatmentPlant.getName() + "-" + type);
        return eg1;
    }

    //获取月份
    public String getQuarterNumber(int no) {
        if (no < 10) {
            return "0" + no;
        } else {
            return no + "";
        }
    }

    @Override
    @Transactional
    public boolean LeadIntoJob(List<SewageJobIdentificationMaterial> sewageJobIdentificationMaterialList, Integer year, Integer month, DLPUser dlpUser) {
        //先创建污水任务的一些信息
        SewageJob sewageJob = new SewageJob();
        String newJobNo = getNewJobNo(year, month);
        sewageJob.setJobNo(newJobNo);
        sewageJob.setJobYear(year);
        sewageJob.setJobSeason(month);
        sewageJob.setName(year + "年" + month + "月份污水任务");
        sewageJob.setId(IdWorker.get32UUID().toUpperCase());
        sewageJob.setDescription("任务来源为导入历史数据");

        LocalDateTime startDate = LocalDateTime.of(year, month, 1, 0, 0);
        //结束时间是任务开始时间的下下个月的最后一天
        LocalDateTime expirationDate = startDate.plusMonths(2).with(TemporalAdjusters.lastDayOfMonth());

        sewageJob.setStartDate(startDate);
        sewageJob.setExpirationDate(expirationDate);
        sewageJob.setLauncheOrgId("0");
        sewageJob.setLauncheUserId(dlpUser.getId());
        sewageJob.setLauncheDate(startDate);
        sewageJob.setStatus(1);
        //先持久化，因为后续会调用通过任务ID查询任务的地方
        this.save(sewageJob);

        List<SewageTreatmentPlant> treatmentPlantArrayList = new ArrayList<>();
        ArrayList<String> arrayList = new ArrayList<>();

        for (SewageJobIdentificationMaterial sewageJobIdentificationMaterial : sewageJobIdentificationMaterialList) {
            //通过导入的污水厂名称，去系统中查询
            String sampleName = sewageJobIdentificationMaterial.getSampleName();

            SewageTreatmentPlant sewageTreatmentPlant = sewageTreatmentPlantService.getOne(new LambdaQueryWrapper<SewageTreatmentPlant>()
                    .eq(SewageTreatmentPlant::getName, sampleName).eq(SewageTreatmentPlant::getCityName, sewageJobIdentificationMaterial.getCollectPlace())
                    .eq(SewageTreatmentPlant::getDistrictName, sewageJobIdentificationMaterial.getCollectPlaceAreaCode()));
            //没查到，就创建一个，但是这个污水厂状态需设置为停用，因为导入的是历史数据
            if (sewageTreatmentPlant == null) {
                String prefix = "";
                switch (sewageJobIdentificationMaterial.getCollectPlace()) {
                    case "西安市":
                        prefix = "610100";
                        break;
                    case "铜川市":
                        prefix = "610200";
                        break;
                    case "宝鸡市":
                        prefix = "610300";
                        break;
                    case "咸阳市":
                        prefix = "610400";
                        break;
                    case "渭南市":
                        prefix = "610500";
                        break;
                    case "延安市":
                        prefix = "610600";
                        break;
                    case "汉中市":
                        prefix = "610700";
                        break;
                    case "榆林市":
                        prefix = "610800";
                        break;
                    case "安康市":
                        prefix = "610900";
                        break;
                    case "商洛市":
                        prefix = "611000";
                        break;
                }
                SewageTreatmentPlant plant = new SewageTreatmentPlant();
                String number = this.getNumber(prefix);

                String collectPlaceAreaCode = sewageJobIdentificationMaterial.getCollectPlaceAreaCode();
                String collectPlace = sewageJobIdentificationMaterial.getCollectPlace();
                plant.setId(IdWorker.get32UUID().toUpperCase());
                plant.setNumber(number);
                plant.setName(sewageJobIdentificationMaterial.getSampleName());
                plant.setAddress("陕西省" + collectPlace + collectPlaceAreaCode);
                plant.setProvinceCode("610000");
                plant.setProvinceName("陕西省");
                plant.setCityCode(prefix);
                plant.setCityName(sewageJobIdentificationMaterial.getCollectPlace());
                String districtCode = baseMapper.getArea("陕西省" + collectPlace + collectPlaceAreaCode);
                if (StrUtil.isBlank(districtCode)) {
                    String code = this.getDistrictCode("陕西省", collectPlace, collectPlaceAreaCode);
                    plant.setDistrictCode(code);
                } else {
                    plant.setDistrictCode(districtCode);
                }
                plant.setDistrictName(collectPlaceAreaCode);
                plant.setWaterInletCount(sewageJobIdentificationMaterial.getWaterInletCount());
                plant.setStatus(-1);
                treatmentPlantArrayList.add(plant);
                sewageTreatmentPlantService.save(plant);
                arrayList.add(plant.getId());
                //因为同一个污水厂会送两个检材，但是只生成一条清单，所以需对污水厂进行去重
            } else if (!arrayList.contains(sewageTreatmentPlant.getId())) {
                arrayList.add(sewageTreatmentPlant.getId());
                treatmentPlantArrayList.add(sewageTreatmentPlant);
            }
        }
        SysOrg sysOrg = remoteOrgService.getById(dlpUser.getOrgId()).getData();
        //创建任务清单项
        this.batchAddJobItemToTask(sewageJob.getId(), sysOrg, treatmentPlantArrayList, dlpUser);

        for (SewageJobIdentificationMaterial sewageJobIdentificationMaterial : sewageJobIdentificationMaterialList) {

            sewageJobIdentificationMaterial.setJobId(sewageJob.getId());
            sewageJobIdentificationMaterial.setId(IdWorker.get32UUID().toUpperCase());
            String sampleName = sewageJobIdentificationMaterial.getSampleName();
            SewageTreatmentPlant sewageTreatmentPlant = sewageTreatmentPlantService.getOne(new LambdaQueryWrapper<SewageTreatmentPlant>()
                    .eq(SewageTreatmentPlant::getName, sampleName)
                    .eq(SewageTreatmentPlant::getCityName, sewageJobIdentificationMaterial.getCollectPlace())
                    .eq(SewageTreatmentPlant::getDistrictName, sewageJobIdentificationMaterial.getCollectPlaceAreaCode()));
            SewageJobItem jobItem = sewageJobItemService.getOne(new LambdaQueryWrapper<SewageJobItem>()
                    .eq(SewageJobItem::getJobId, sewageJob.getId())
                    .eq(SewageJobItem::getPlantId, sewageTreatmentPlant.getId()));

            sewageJobIdentificationMaterial.setJobItemId(jobItem.getId());

            sewageJobIdentificationMaterial.setStatus(3);
            sewageJobIdentificationMaterial.setJobId(sewageJob.getId());
            sewageJobIdentificationMaterial.setJobItemId(jobItem.getId());
            sewageJobIdentificationMaterial.setCollectPlace("");
            sewageJobIdentificationMaterial.setCollectPlaceAreaCode("");

            if (StrUtil.isBlank(sewageJobIdentificationMaterial.getSample1No())) {
                sewageJobIdentificationMaterial.setSample1No(this.getNewSampleNo(sewageJobIdentificationMaterial.getImNo(), 1));
            }
            if (StrUtil.isBlank(sewageJobIdentificationMaterial.getSample2No())) {
                sewageJobIdentificationMaterial.setSample2No(this.getNewSampleNo(sewageJobIdentificationMaterial.getImNo(), 2));
            }

            String imNo = sewageJobIdentificationMaterial.getImNo();
            //导入的样本编号有两种格式：2023-M3-XA-19A1，2023-M3-XA-18B, 要通过后缀筛别一下
            String suffix = "";
            String s = imNo.substring(imNo.length() - 1);
            if (!(s.equals("A") || s.equals("B"))) {
                String s1 = imNo.substring(imNo.length() - 2, imNo.length() - 1);
                suffix = s1;
            } else {
                suffix = s;
            }
            //节假日或者工作日类型
            if (suffix.equals("A")) {
                sewageJobIdentificationMaterial.setSampleType(1);
            } else if (suffix.equals("B")) {
                sewageJobIdentificationMaterial.setSampleType(2);
            } else {
                sewageJobIdentificationMaterial.setSampleType(1);
            }
            sewageJobIdentificationMaterial.setStorageMethod("常温");
            sewageJobIdentificationMaterial.setExpressNumber("/");
            sewageJobIdentificationMaterial.setExpressCompany("/");

            sewageJobIdentificationMaterial.setComments("初始化导入");
            sewageJobIdentificationMaterial.setInformantId(dlpUser.getId());
            sewageJobIdentificationMaterial.setAcceptTime(startDate);
            sewageJobIdentificationMaterial.setDomesticSewageProportion(sewageJobIdentificationMaterial.getDomesticSewageProportion() * 100);
        }

        return sewageJobIdentificationMaterialService.saveBatch(sewageJobIdentificationMaterialList) ? true : false;
    }


    //由于一些字段参数的不一致，所以仿照SewageJobItemServiceImpl，重写了一个这样的方法，目的是为了接收List集合
    public List<SewageJobItem> batchAddJobItemToTask(String taskJobId, SysOrg userOrg, List<SewageTreatmentPlant> sewageTreatmentPlantList, DLPUser dlpUser) {
        List<SewageJobItem> thisTimeItemList = new ArrayList<SewageJobItem>();//存储本次任务清单项
        Map<String, List<String>> sewageAreaHashMap = new HashMap<String, List<String>>();//区域
        //构造本次提交的清单项
        for (SewageTreatmentPlant sewageTreatmentPlant : sewageTreatmentPlantList) {
            String stpID = sewageTreatmentPlant.getId();
            String provinceCode = sewageTreatmentPlant.getProvinceCode();
            String cityCode = sewageTreatmentPlant.getCityCode();
            String districtCode = sewageTreatmentPlant.getDistrictCode();
            String stpName = sewageTreatmentPlant.getName();
            //判断如果不是分中心员工，则需要判断是否在管辖范围内
            if (!dlpUser.isStaff()) {
                if (!(StrUtil.equals(userOrg.getAreaCode(), provinceCode)
                        || StrUtil.equals(userOrg.getAreaCode(), cityCode)
                        || StrUtil.equals(userOrg.getAreaCode(), districtCode)
                )) {
                    throw new RuntimeException(String.format("[%s]不在你管辖范围内，你没有权限添加到任务清单中", stpName));
                }
            }
            //构建清单项
            SewageJobItem eg = new SewageJobItem();
            eg.setId(IdWorker.get32UUID().toUpperCase());
            eg.setJobId(taskJobId);
            eg.setCreateByOrgLevel(userOrg.getOrgLevel());
            eg.setPlantId(stpID);
            eg.setComments("初始化导入");
            thisTimeItemList.add(eg);
            //设置任务的区域数据
            String haskKey = provinceCode + "-" + cityCode + "-" + districtCode;
            List<String> areaCodeValue = new ArrayList<>();
            areaCodeValue.add(provinceCode);
            areaCodeValue.add(cityCode);
            areaCodeValue.add(districtCode);
            sewageAreaHashMap.put(haskKey, areaCodeValue);
        }
        //根据任务ID判断是否有历史清单
        List<SewageJobItem> historyItemList = sewageJobItemService.list(Wrappers.<SewageJobItem>query()
                .eq("job_id", taskJobId));
        if (historyItemList.size() > 0) {
            sewageJobItemService.removeBatchByIds(historyItemList);//如果有就删除
        }
        sewageJobItemService.saveBatch(thisTimeItemList);//保存清单项
        //添加之后，更新任务的区域列表
        List<List<String>> taskAreaList = new ArrayList<List<String>>();
        sewageAreaHashMap.forEach((k, v) -> {
            taskAreaList.add(v);
        });
        //持久化到数据库
        SewageJob sjob = this.getById(taskJobId);
        sjob.setSewageAreaList(JSONArray.toJSONString(taskAreaList));
        this.updateById(sjob);
        return thisTimeItemList;
    }

    @Override
    public String getDistrictCode(String provinceName, String cityName, String districtName) {
        String area = baseMapper.getArea(provinceName + cityName);
        if (!StrUtil.isNotBlank(area)) {
            throw new RuntimeException(String.format("查找出错了，未能找到%s的市级编码", provinceName + cityName));
        }
        String prefix = area.substring(0, 4);
        //看看系统是否已经录 入了这个地区的污水厂
        List<SewageTreatmentPlant> sewageTreatmentPlantList = sewageTreatmentPlantService.list(new LambdaQueryWrapper<SewageTreatmentPlant>().eq(SewageTreatmentPlant::getDistrictName, districtName)
                .eq(SewageTreatmentPlant::getCityName, cityName).eq(SewageTreatmentPlant::getProvinceName, provinceName));
        if (sewageTreatmentPlantList != null && sewageTreatmentPlantList.size() > 0) {
            return sewageTreatmentPlantList.get(0).getDistrictCode();
        } else {
            //没有的话就按99递减生成地区编码
            List<SewageTreatmentPlant> plantList = sewageTreatmentPlantService.list(new LambdaQueryWrapper<SewageTreatmentPlant>().like(SewageTreatmentPlant::getCityCode, area).orderByDesc(SewageTreatmentPlant::getDistrictCode));
            Integer index = 99;
            //
            boolean r = false;
            int s = 0;
            //用99递减去与该市下的所有污水厂的最后两位比较，如果没有一个重复，就代表可以使用这个尾号了
            for (int i = 0; i < plantList.size(); i++) {
                for (int j = 0; j < plantList.size(); j++) {
                    SewageTreatmentPlant sewageTreatmentPlant = plantList.get(j);
                    String substring = sewageTreatmentPlant.getDistrictCode().substring(4, 6);
                    if (!substring.equals(index.toString())) {
                        s++;
                    }
                }
                if (s == plantList.size()) {
                    return prefix + index;
                } else {
                    s = 0;
                    index--;
                }
            }
            return prefix + (index - 1);
        }
    }

    @Override
    //通过前缀去获取污水厂编号，默认从99开始递减
    public String getNumber(String prefix) {
        Integer index = 99;
        List<SewageTreatmentPlant> list = sewageTreatmentPlantService.list(new LambdaQueryWrapper<SewageTreatmentPlant>()
                .like(SewageTreatmentPlant::getNumber, prefix).orderByDesc(SewageTreatmentPlant::getNumber));

        //没有查到这个市的污水厂，就证明这个市还没有污水厂，我们直接使用99
        if (list == null || list.size() == 0) {
            return prefix + index.toString();
        } else {
            //否则就遍历这个市的所有污水厂，从99一直递减，直到匹配合适的数值
            for (int i = 0; i < list.size(); i++) {
                SewageTreatmentPlant plant = list.get(i);
                if (!plant.getNumber().substring(6, 8).equals(index.toString())) {
                    return prefix + index.toString();
                } else {
                    index--;
                }
            }
        }
        return prefix + index.toString();
    }

    /*
     * 查询这个账户（机构）下面所送检且已受理的所有污水检材的数量*/
    @Override
    public Integer getIMQuantity(String orgId) {
        return sewageJobIdentificationMaterialMapper.getAcceptIMQuantity(orgId).size();
    }

    /**
     * 查询各个状态下的检材数量
     * @param orgId
     * @return
     */
    @Override
    public List<MarkersVO> getSewageMarkers(String orgId) {

        ArrayList<MarkersVO> markersVOS = new ArrayList<>();

        List<SewageJobIdentificationMaterial> list1 = sewageJobIdentificationMaterialMapper.getFoundIMQuantity(orgId);
        markersVOS.add(new MarkersVO("污水任务", list1.size(), "未填报"));

        List<SewageJobIdentificationMaterial> list2 = sewageJobIdentificationMaterialMapper.getWriteIMQuantity(orgId);
        markersVOS.add(new MarkersVO("污水任务", list2.size(), "未送检"));

        List<SewageJobIdentificationMaterial> list3 = sewageJobIdentificationMaterialMapper.getInspectIMQuantity(orgId);
        markersVOS.add(new MarkersVO("污水任务", list3.size(), "已送检"));

        List<SewageJobIdentificationMaterial> list4 = sewageJobIdentificationMaterialMapper.getRejectionIMQuantity(orgId);
        markersVOS.add(new MarkersVO("污水任务", list4.size(), "已作废"));

        return markersVOS;

    }

    @Override
    public Map<String, List> getDetails(DLPUser dlpUser, Integer status) {

        HashMap<String, List> map = new HashMap<>();

        switch (status) {
            case 0:
                List<SewageJobIdentificationMaterial> list1 = sewageJobIdentificationMaterialMapper.getFoundIMQuantity(dlpUser.getOrgId());
                map.put("污水任务：未填报", list1);
                return map;
            case 1:
                List<SewageJobIdentificationMaterial> list2 = sewageJobIdentificationMaterialMapper.getWriteIMQuantity(dlpUser.getOrgId());
                map.put("污水任务：未送样", list2);
                return map;

            case -1:
                List<SewageJobIdentificationMaterial> list3 = sewageJobIdentificationMaterialMapper.getInspectIMQuantity(dlpUser.getOrgId());
                map.put("污水任务：已送检", list3);
                return map;

            case -2:
                List<SewageJobIdentificationMaterial> list4 = sewageJobIdentificationMaterialMapper.getRejectionIMQuantity(dlpUser.getOrgId());
                map.put("污水任务：已作废", list4);
                return map;
        }
        return null;
    }

    /**
     * 受理系统统计已受理的污水检材数量
     * @return
     */
    @Override
    public Integer getImQuantityForAccept() {
        int size = sewageJobIdentificationMaterialService.list(new LambdaQueryWrapper<SewageJobIdentificationMaterial>()
                .eq(SewageJobIdentificationMaterial::getStatus, 3)).size();

        return size;
    }

    @Override
    public IPage<SewageJob> getJobAndMaterialPage(Page page, Integer status, String keywords) {
//        Page<SewageJob> sewagePage = this.page(page, Wrappers.<SewageJob>lambdaQuery()
//                .and(StrUtil.isNotBlank(keywords), qw -> qw
//                        .like(SewageJob::getName, keywords)
//                        .or()
//                        .like(SewageJob::getJobNo, keywords))
//                .eq(status != null, SewageJob::getStatus, status));
        IPage<SewageJob> sewagePage = baseMapper.getAllJob(page, keywords);
        List<SewageJob> records = sewagePage.getRecords();
        for (SewageJob sewageJob : records) {
            List<SewageJobIdentificationMaterial> materialList = sewageJobIdentificationMaterialService.list(Wrappers.<SewageJobIdentificationMaterial>lambdaQuery()
                    .eq(SewageJobIdentificationMaterial::getJobId, sewageJob.getId())
                    .eq(SewageJobIdentificationMaterial::getStatus, 3)
                    .orderByDesc(SewageJobIdentificationMaterial::getImNo));
            sewageJob.setMaterialList(materialList);
        }
        return sewagePage;
    }

    /**
     * 根据任务状态 统计不同状态下的任务数量
     * @return
     */
    @Override
    public R<StatisticsDiffStatusJobDTO> statisticsDiffStatusJob() {
        StatisticsDiffStatusJobDTO statisticsDiffStatusJobDTO = baseMapper.statisticsDiffStatusJob(
                SewageJobStatusConstants.JOB_STATUS_CREATED.getStatus(),
                SewageJobStatusConstants.JOB_STATUS_PUBLISH.getStatus(),
                SewageJobStatusConstants.JOB_STATUS_STOP_RECEIVE.getStatus(),
                SewageJobStatusConstants.JOB_STATUS_WAITING_FINISH.getStatus());
        // 统计在启用的污水厂数量
        statisticsDiffStatusJobDTO.setSewageTreatmentPlantNum((int) sewageTreatmentPlantService.count(new LambdaQueryWrapper<SewageTreatmentPlant>()
                .eq(SewageTreatmentPlant::getStatus, 1)));
        return R.ok(statisticsDiffStatusJobDTO);
    }

    /**
     * 根据任务发布时间查询在这之前的任务，并根据发布时间降序，然后根据limit 取不同数量的值
     *
     * @param startDate
     * @param limit
     * @return
     */
    @Override
    public R<List<SewageJobVO>> querySewageJobByStartDate(String startDate, int limit) {
        List<SewageJobVO> sewageJobVOS = baseMapper.querySewageJobByStartDate(startDate, limit);
        return R.ok(sewageJobVOS);
    }

    /**
     * 根据污水检材创建样本
     *
     * @param sewageJobIdentificationMaterial
     *
     * @return
     */
    @Override
    public List<Sample> createSample(SewageJobIdentificationMaterial sewageJobIdentificationMaterial, String dlpUserId) {

        //污水的编号重新生成，逻辑发生改变，现在一个检材就对应一个样本  8-31

        List<Sample> sampleList=new ArrayList<Sample>();
        List<DrugLite> candidateDrugs = JSON.parseArray(AppStartupRunner.getCfg(CommonConstants.DLP_CODE_ENTRUSTMENT_SEWAGE_JOB_IDENTIFICATION_MATERIAL_CANDIDATE_DRUGS), DrugLite.class);
        List<IMAdditionalProperty> additionalProperties = JSON.parseArray(AppStartupRunner.getCfg(CommonConstants.DLP_CODE_ENTRUSTMENT_SEWAGE_JOB_IDENTIFICATION_MATERIAL_ADDITIONAL_PROPERTIES), IMAdditionalProperty.class);
        BigDecimal quantity = new BigDecimal(Double.parseDouble(AppStartupRunner.getCfg(CommonConstants.DLP_CODE_ENTRUSTMENT_SEWAGE_JOB_IDENTIFICATION_MATERIAL_QUANTITY)));
        // 生成 样本1
        //if ((sewageJobIdentificationMaterial.getSample1Present() != null) && (sewageJobIdentificationMaterial.getSample1Present())) {
//			SampleBox sb1 = sampleBoxMapper.selectById(sewageJobIdentificationMaterial.getSample1BoxId());
        Sample sample1 = new Sample();
        sample1.setId(IdWorker.get32UUID().toUpperCase());
        sample1.setIdentificationMaterialId(sewageJobIdentificationMaterial.getId());
        sample1.setSource("sewageJob");
        //sample1.setSampleNo(sewageJobIdentificationMaterial.getSample1No());
        sample1.setSampleNo(sewageJobIdentificationMaterial.getImNo());//编号规则改变，改成新的编号

        sample1.setCandidateDrugs(candidateDrugs);
        sample1.setFundName(AppStartupRunner.getCfg(CommonConstants.DLP_CODE_ENTRUSTMENT_SEWAGE_JOB_IDENTIFICATION_MATERIAL_FUND_NAME));
        sample1.setFundQuantity(Integer.parseInt(AppStartupRunner.getCfg(CommonConstants.DLP_CODE_ENTRUSTMENT_SEWAGE_JOB_IDENTIFICATION_MATERIAL_FUND_QUANTITY)));
        sample1.setFundUnit(AppStartupRunner.getCfg(CommonConstants.DLP_CODE_ENTRUSTMENT_SEWAGE_JOB_IDENTIFICATION_MATERIAL_FUND_UNIT));
        sample1.setQuantity(quantity);
        sample1.setUnit(AppStartupRunner.getCfg(CommonConstants.DLP_CODE_ENTRUSTMENT_SEWAGE_JOB_IDENTIFICATION_MATERIAL_UNIT));
        sample1.setAdditionalProperties(additionalProperties);
        sample1.setDrawWay(AppStartupRunner.getCfg(CommonConstants.DLP_CODE_ENTRUSTMENT_SEWAGE_JOB_IDENTIFICATION_MATERIAL_DRAWWAY));

        sample1.setBoxId(sewageJobIdentificationMaterial.getSample1BoxId());
//			sample1.setSampleType(sb1.getSampleType());
        sample1.setStorageMethod(sewageJobIdentificationMaterial.getStorageMethod());
        sample1.setAcceptTime(sewageJobIdentificationMaterial.getAcceptTime());

        sample1.setSampleType(sewageJobIdentificationMaterial.getSampleType().equals("1")?"A":"B");//设置类型
        sample1.setSewageJobId(sewageJobIdentificationMaterial.getJobId()); //设置任务ID
        sample1.setHolder(dlpUserId);
        sample1.setOrgId(sewageJobIdentificationMaterial.getDeliverOrgId());
        sample1.setName(sewageJobIdentificationMaterial.getSampleName());

        sampleList.add(sample1);

        return sampleList;
    }

    /**
     * 根据污水检材创建样本
     *
     * @param sewageJobIdentificationMaterialList
     *
     * @return
     */
    @Override
    public List<Sample> createSample(List<SewageJobIdentificationMaterial> sewageJobIdentificationMaterialList, String dlpUserId) {

        //污水的编号重新生成，逻辑发生改变，现在一个检材就对应一个样本  8-31

        List<Sample> sampleList=new ArrayList<Sample>();

        List<DrugLite> candidateDrugs = JSON.parseArray(AppStartupRunner.getCfg(CommonConstants.DLP_CODE_ENTRUSTMENT_SEWAGE_JOB_IDENTIFICATION_MATERIAL_CANDIDATE_DRUGS), DrugLite.class);
        List<IMAdditionalProperty> additionalProperties = JSON.parseArray(AppStartupRunner.getCfg(CommonConstants.DLP_CODE_ENTRUSTMENT_SEWAGE_JOB_IDENTIFICATION_MATERIAL_ADDITIONAL_PROPERTIES), IMAdditionalProperty.class);
        String fundName = AppStartupRunner.getCfg(CommonConstants.DLP_CODE_ENTRUSTMENT_SEWAGE_JOB_IDENTIFICATION_MATERIAL_FUND_NAME);
        String fundQuantity = AppStartupRunner.getCfg(CommonConstants.DLP_CODE_ENTRUSTMENT_SEWAGE_JOB_IDENTIFICATION_MATERIAL_FUND_QUANTITY);
        String fundUnit = AppStartupRunner.getCfg(CommonConstants.DLP_CODE_ENTRUSTMENT_SEWAGE_JOB_IDENTIFICATION_MATERIAL_FUND_UNIT);
        BigDecimal quantity = new BigDecimal(Double.parseDouble(AppStartupRunner.getCfg(CommonConstants.DLP_CODE_ENTRUSTMENT_SEWAGE_JOB_IDENTIFICATION_MATERIAL_QUANTITY)));
        String drawWay = AppStartupRunner.getCfg(CommonConstants.DLP_CODE_ENTRUSTMENT_SEWAGE_JOB_IDENTIFICATION_MATERIAL_DRAWWAY);
        String unit = AppStartupRunner.getCfg(CommonConstants.DLP_CODE_ENTRUSTMENT_SEWAGE_JOB_IDENTIFICATION_MATERIAL_UNIT);
        // 生成 样本1
        for (SewageJobIdentificationMaterial sewageJobIdentificationMaterial : sewageJobIdentificationMaterialList) {
            Sample sample1 = new Sample();
            sample1.setId(IdWorker.get32UUID().toUpperCase());
            sample1.setIdentificationMaterialId(sewageJobIdentificationMaterial.getId());
            sample1.setSource("sewageJob");
            //sample1.setSampleNo(sewageJobIdentificationMaterial.getSample1No());
            sample1.setSampleNo(sewageJobIdentificationMaterial.getImNo());//编号规则改变，改成新的编号

            sample1.setCandidateDrugs(candidateDrugs);
            sample1.setFundName(fundName);
            sample1.setFundQuantity(Integer.parseInt(fundQuantity));
            sample1.setFundUnit(fundUnit);
            sample1.setQuantity(quantity);
            sample1.setUnit(unit);
            sample1.setAdditionalProperties(additionalProperties);
            sample1.setDrawWay(drawWay);

            sample1.setBoxId(sewageJobIdentificationMaterial.getSample1BoxId());
//			sample1.setSampleType(sb1.getSampleType());
            sample1.setStorageMethod(sewageJobIdentificationMaterial.getStorageMethod());
            sample1.setAcceptTime(sewageJobIdentificationMaterial.getAcceptTime());

            sample1.setSampleType(sewageJobIdentificationMaterial.getSampleType().equals("1")?"A":"B");//设置类型
            sample1.setSewageJobId(sewageJobIdentificationMaterial.getJobId()); //设置任务ID
            sample1.setHolder(dlpUserId);
            sample1.setOrgId(sewageJobIdentificationMaterial.getDeliverOrgId());
            sample1.setName(sewageJobIdentificationMaterial.getSampleName());

            sampleList.add(sample1);
        }

        return sampleList;
    }

}
