package digital.laboratory.platform.sewage.utils;

import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.*;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpPut;
import org.apache.http.config.Registry;
import org.apache.http.config.RegistryBuilder;
import org.apache.http.conn.socket.ConnectionSocketFactory;
import org.apache.http.conn.socket.PlainConnectionSocketFactory;
import org.apache.http.conn.ssl.NoopHostnameVerifier;
import org.apache.http.conn.ssl.SSLConnectionSocketFactory;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.impl.conn.PoolingHttpClientConnectionManager;
import org.apache.http.message.BasicHeader;
import org.apache.http.ssl.SSLContextBuilder;
import org.apache.http.ssl.TrustStrategy;
import org.apache.http.util.EntityUtils;

import javax.net.ssl.SSLContext;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;

@Slf4j
public class HttpClientUtils {
    private static CloseableHttpClient closeableHttpClient;
    private static PoolingHttpClientConnectionManager cm;
    // 自定义Http
    static {
        HttpClientBuilder httpClientBuilder = HttpClients.custom();

        /**
         * 1  绕过不安全的https请求证书认证
         */
        Registry<ConnectionSocketFactory> registry = RegistryBuilder.<ConnectionSocketFactory>create()
                .register("http", PlainConnectionSocketFactory.INSTANCE)
                .register("https", trustHttpsCertificates())
                .build();

        /**
         * 2 创建连接池管理对象
         */
        cm = new PoolingHttpClientConnectionManager(registry);
        cm.setMaxTotal(50); // 最大连接池有50个连接
        /**
         * ip+port/域名  就是一个路由
         *
         */
        cm.setDefaultMaxPerRoute(50); // 每个路由默认最大有多少连接

        httpClientBuilder.setConnectionManager(cm);

        /**
         * 3 设置请求默认配置
         */
        RequestConfig requestConfig = RequestConfig.custom()
                .setConnectTimeout(5000)
                .setSocketTimeout(3000)
                .setConnectionRequestTimeout(5000)
                .build();
        httpClientBuilder.setDefaultRequestConfig(requestConfig);

        /**
         * 设置默认的一些header
         */
        List<Header> defHeaders = new ArrayList<>();
        BasicHeader basicHeader = new BasicHeader("User-Agent", "Mozilla/5.0 (Windows NT 10.0; Win64; x64) " +
                "AppleWebKit/537.36 (KHTML, like Gecko) " +
                "Chrome/114.0.0.0 Safari/537.36");
        defHeaders.add(basicHeader);
        httpClientBuilder.setDefaultHeaders(defHeaders);

        // 设置连接管理共享
        httpClientBuilder.setConnectionManagerShared(true);

        // 线程安全，此处初始化一次即可，通过上面的配置来生成一个用于管理多个连接池closeableHttpClient
        closeableHttpClient = httpClientBuilder.build();
    }

    // 创建安全链接协议得工厂
    private static ConnectionSocketFactory trustHttpsCertificates(){
        SSLContextBuilder sslContextBuilder = new SSLContextBuilder();
        try {
            sslContextBuilder.loadTrustMaterial(null, new TrustStrategy() {
                @Override
                public boolean isTrusted(X509Certificate[] chain, String authType) throws CertificateException {
                    return true;
                }
            });

            SSLContext sslContext = sslContextBuilder.build();
            return new SSLConnectionSocketFactory(sslContext,
                    new String[]{"SSLv2Hello", "SSLv3", "TLSv1", "TLSv1.1", "TLSv1.2"},
                    null,
                    NoopHostnameVerifier.INSTANCE
            );
        } catch (Exception e) {
            log.error("构造连接工厂失败", e);
            throw new RuntimeException("构造连接工厂失败");
        }
    }

    /**
     * post方式访问，通过携带json 参数
     * @param url
     * @param body
     * @param headers
     * @return
     */
    public static JSONObject postJson(String url, String body, Map<String, String> headers) {
        HttpPost httpPost = new HttpPost(url);

        // 设置请求头
        if (headers != null) {
            Set<Map.Entry<String, String>> entries = headers.entrySet();
            for (Map.Entry<String, String> entry : entries) {
                httpPost.addHeader(new BasicHeader(entry.getKey(), entry.getValue()));
            }
        }
        // 文档要求Content-Type 必须是 application/x-www-form
        httpPost.addHeader("Content-Type", "application/x-www-form");

        // 设置请求体
        StringEntity jsonEntity = new StringEntity(body, Consts.UTF_8);
        // 设置entity 里面得内容
        jsonEntity.setContentType(new BasicHeader("Content-Type", "application/json; charset=utf-8"));
        // 设置entity里面得编码
        jsonEntity.setContentEncoding(Consts.UTF_8.name());
        httpPost.setEntity(jsonEntity);

        CloseableHttpResponse response = null;
        try {
            response = closeableHttpClient.execute(httpPost);
            StatusLine statusLine = response.getStatusLine();
            if (HttpStatus.SC_OK == statusLine.getStatusCode()) {
                HttpEntity entity = response.getEntity();
//                EntityUtils.consume(entity);
                return JSONObject.parseObject(EntityUtils.toString(entity, StandardCharsets.UTF_8));
            } else {
                log.error("响应失败，响应码:"+statusLine.getStatusCode());
            }
        } catch (Exception e) {
            log.error("postJson error, url:{}", url, e);
            e.printStackTrace();
        } finally {
            consumeRes(response);
        }
        return null;
    }

    /**
     * 发送get请求
     * @param url 请求的geturl
     * @param headers 请求头
     * @return
     */
    public static JSONObject executeGet(String url, Map<String, String> headers) {
        // 构造httpGet 请求对象
        HttpGet httpGet = new HttpGet(url);
        // 设置请求头
        if (headers != null) {
            Set<Map.Entry<String, String>> entries = headers.entrySet();
            for (Map.Entry<String, String> entry : entries) {
                httpGet.addHeader(new BasicHeader(entry.getKey(), entry.getValue()));
            }
        }

        // 可关闭的响应
        CloseableHttpResponse response = null;
        try {
            log.info("prepare to execute url:{}", url);
            response = closeableHttpClient.execute(httpGet);
            StatusLine statusLine = response.getStatusLine();
            if (statusLine.getStatusCode() == HttpStatus.SC_OK) {
                HttpEntity entity = response.getEntity();
                return JSONObject.parseObject(EntityUtils.toString(entity, StandardCharsets.UTF_8));
            } else {
                log.error("响应失败，响应码:"+statusLine.getStatusCode());
            }
        } catch (Exception e) {
            log .error("executeGet error,url:(]",url,e);
            e.printStackTrace();
        } finally {
            consumeRes(response);
        }
        return null;
    }

    /**
     * put方式访问，通过携带json 参数
     * @param url
     * @param body
     * @param headers
     * @return
     */
    public static JSONObject putJson(String url, String body, Map<String, String> headers) {
        HttpPut httpPut = new HttpPut(url);

        // 设置请求头
        if (headers != null) {
            Set<Map.Entry<String, String>> entries = headers.entrySet();
            for (Map.Entry<String, String> entry : entries) {
                httpPut.addHeader(new BasicHeader(entry.getKey(), entry.getValue()));
            }
        }

        // 设置请求体
        StringEntity jsonEntity = new StringEntity(body, Consts.UTF_8);
        // 设置entity 里面得内容
        jsonEntity.setContentType(new BasicHeader("Content-Type", "application/json; charset=utf-8"));
        // 设置entity里面得编码
        jsonEntity.setContentEncoding(Consts.UTF_8.name());
        httpPut.setEntity(jsonEntity);

        CloseableHttpResponse response = null;
        try {
            response = closeableHttpClient.execute(httpPut);
            StatusLine statusLine = response.getStatusLine();
            if (HttpStatus.SC_OK == statusLine.getStatusCode()) {
                HttpEntity entity = response.getEntity();
//                EntityUtils.consume(entity);
                return JSONObject.parseObject(EntityUtils.toString(entity, StandardCharsets.UTF_8));
            } else {
                log.error("响应失败，响应码:"+statusLine.getStatusCode());
            }
        } catch (Exception e) {
            log.error("putJson error, url:{}", url, e);
            e.printStackTrace();
        } finally {
            consumeRes(response);
        }
        return null;
    }

    /**
     * 关闭流
     * @param response
     */
    public static void consumeRes(CloseableHttpResponse response) {

        if (closeableHttpClient != null) {
            try {
                closeableHttpClient.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        if (response != null) {
            try {
                response.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

}
